/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: addressbook_select.c
 *         Selection functions for the address book
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>

#include "eucalyptus.h"

static GnomeUIInfo select_addr_toolbar[] =
{
	{ GNOME_APP_UI_ITEM, N_("To"), N_("Add address(es) to \'To:\' list"), 
			address_select_add_to_cb, NULL, NULL, 
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_NEW,
			0, 0, NULL, },
	{ GNOME_APP_UI_ITEM, N_("Cc"), N_("Add address(es) to \'Cc:\' list"), 
			address_select_add_cc_cb, NULL, NULL, 
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_NEW,
			0, 0, NULL, },
	{ GNOME_APP_UI_ITEM, N_("Bcc"), N_("Add address(es) to \'Bcc:\' list"), 
			address_select_add_bcc_cb, NULL, NULL, 
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_NEW,
			0, 0, NULL, },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("New"), N_("Create a new address entry"), 
			address_select_new_item_cb, NULL, NULL, 
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_NEW,
			0, 0, NULL, },
	{ GNOME_APP_UI_ITEM, N_("New group"), N_("Create a new address group"), 
			address_select_new_group_cb, NULL, NULL, 
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_NEW,
			0, 0, NULL, },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Info"), N_("Show details of the selected address(es)"), 
			address_select_info_cb, NULL, NULL, 
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_NEW,
			0, 0, NULL, },
	GNOMEUIINFO_END,
};

gboolean address_select_open_window(Composition *comp, gchar type, gboolean multi_select)
{
	GtkWidget *scrw, *vbox;
	gchar *w_title, *titles[] = { N_("Name"), N_("Address"), };
	
	g_return_val_if_fail(comp != NULL, FALSE);
	
	switch(type)
	{
		case CAT_CC:
			w_title = _("Select 'Cc:' addresses");
			break;

		case CAT_BCC:
			w_title = _("Select 'Bcc:' addresses");
			break;

		default:
			w_title = _("Select 'To:' addresses");
			break;
	}
	
	if(comp->addr_sel.Window)
	{
		if(comp->addr_sel.Type != type)
		{
			gtk_widget_show_all(comp->addr_sel.Window);
			gtk_window_set_title(GTK_WINDOW(comp->addr_sel.Window), w_title);

			comp->addr_sel.Type = type;
		}
		
		return TRUE;
	}

	/* Set the window type.
	 */	
	comp->addr_sel.Type = type;

	/* Create the window.
	 */
	comp->addr_sel.Window = gnome_app_new(APPNAME, _("Select addresses"));
	
	/* Connect the signals.
	 */
	gtk_signal_connect(GTK_OBJECT(comp->addr_sel.Window), "destroy", 
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &comp->addr_sel.Window);
	
	/* Create the toolbar.
	 */
	gnome_app_create_toolbar_with_data(GNOME_APP(comp->addr_sel.Window), select_addr_toolbar, comp);
	
	/* Create the contents box.
	 */
	vbox = gtk_vbox_new(FALSE, 0);
	gnome_app_set_contents(GNOME_APP(comp->addr_sel.Window), vbox);
	
	/* Create the selection list.
	 */
	scrw = gtk_scrolled_window_new(NULL, NULL);
	{
		comp->addr_sel.AddrList = gtk_ctree_new_with_titles(2, 0, titles);
		gtk_clist_set_column_auto_resize(GTK_CLIST(comp->addr_sel.AddrList), 0, TRUE);
		gtk_clist_set_selection_mode(GTK_CLIST(comp->addr_sel.AddrList), multi_select ? GTK_SELECTION_MULTIPLE : GTK_SELECTION_SINGLE);

		gtk_signal_connect(GTK_OBJECT(comp->addr_sel.AddrList), "select-row",
				GTK_SIGNAL_FUNC(address_select_item_cb), NULL);
				
		/* Add to scrolled window.
		 */
		gtk_container_add(GTK_CONTAINER(scrw), comp->addr_sel.AddrList);

		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrw),
				GTK_POLICY_AUTOMATIC,	/* horizontal */
				GTK_POLICY_AUTOMATIC);	/* vertical */
	}
	gtk_box_pack_start(GTK_BOX(vbox), scrw, TRUE, TRUE, 0);

	/* Create the matching entry widget.
	 */
	comp->addr_sel.MatchAddr = gnome_entry_new("addr-sel-match");
	gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(comp->addr_sel.MatchAddr))), "changed",
			GTK_SIGNAL_FUNC(address_select_match_changed_cb), comp);
	gtk_box_pack_start(GTK_BOX(vbox), comp->addr_sel.MatchAddr, FALSE, TRUE, 0);
	
	/* Populate the list.
	 */
	address_select_populate_list(comp);
	
	/* Show the window.
	 */
	gtk_widget_show_all(comp->addr_sel.Window);
	
	return TRUE;
}

void address_select_close_window(Composition *comp)
{
	g_return_if_fail(comp != NULL);
	
	gtk_widget_destroy(comp->addr_sel.Window);
	comp->addr_sel.Window = NULL;
}

void address_select_populate_list(Composition *comp)
{
}

AddressEntry *address_select_one_from_many_matches(struct List *list, GtkWidget *parentw, gboolean quiet)
{
	AddressEntry *ae = NULL;
	
	
	
	return ae;
}

gboolean address_select_add(Composition *comp, gchar type)
{
	gchar *text;


}

/* Callbacks. 
 */
void address_select_item_cb(GtkCTree *tree, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Composition *comp = (Composition *)data;
	
}

void address_select_add_to_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	address_select_add(comp, CAT_TO);
}

void address_select_add_cc_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	address_select_add(comp, CAT_CC);
}

void address_select_add_bcc_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	address_select_add(comp, CAT_BCC);
}

void address_select_new_item_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;
	
}

void address_select_new_group_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;
	
}

void address_select_info_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;
	
}

void address_select_match_changed_cb(GtkEditable *widget, gpointer data)
{
	/* Scan address list for a match and
	 * select the item.
	 */
}

#if 0
struct List *SelectAddress(WORD flags)
{
     struct Window *window;
     Object *winobj = NULL;
     struct Gadget *layout, *browser, *okay;
     struct List *list = NULL, *alist = NULL;
     static struct ColumnInfo ci[] =
     {
          {50, "", 0,},
          {50, "", 0,},
          {-1, 0, 0,},
     };
     BOOL done = FALSE, abort = FALSE;

     // build address book list
     if(!(list = (struct List *)AllocVec(sizeof(struct List), MEMF_CLEAR)))
          goto thatsit;
     NewList(list);
     if(BuildAddressBook(list, NULL))
          goto thatsit;

     if(!(alist = (struct List *)AllocVec(sizeof(struct List), MEMF_CLEAR)))
          goto thatsit;
     NewList(alist);

     if(!(winobj = NewObject(WINDOW_GetClass(), NULL,
               WINDOW_Position,         WPOS_CENTERMOUSE,
               WA_InnerWidth,           350,
               WA_InnerHeight,          200,

               WA_Activate,             TRUE,
               WA_CloseGadget,          TRUE,
               WA_DragBar,              TRUE,
               WA_DepthGadget,          TRUE,
               WA_SizeGadget,           TRUE,
               WA_SizeBBottom,          TRUE,
               WA_RMBTrap,              TRUE,
               WINDOW_BackFillName,     sprefs->Backfill,

               WA_PubScreen,            euc.Screen,
               WA_Title,                LocaleString(ETXT_TITLE_SELECTADDRESS),
               WA_ScreenTitle,          eucsctitle,

               WINDOW_Layout,           layout = VGroupObject,
                    LAYOUT_DeferLayout, TRUE,
                    LAYOUT_SpaceOuter,  TRUE,
                    StartMember,        browser = ListBrowserObject,
                         LISTBROWSER_Labels,      list,
                         LISTBROWSER_ColumnInfo,  &ci,
                         LISTBROWSER_Hierarchical,TRUE,
                         LISTBROWSER_MultiSelect, !(flags & SAF_ONE),
                         LISTBROWSER_ShowSelected,TRUE,
                         GA_ID,                   GAD_SELECTADDRESS_LISTBROWSER,
                         GA_RelVerify,            TRUE,
                    EndMember,
                    StartMember,   okay = ButtonObject,
                         GA_Text,            LocaleString(ETXT_BUTTON_OKAY),
                         GA_ID,              GAD_USE,
                         GA_RelVerify,       TRUE,
                         GA_Disabled,        TRUE,
                    EndMember,
                    CHILD_WeightedHeight,    0,
               End,
               TAG_END)))
          goto thatsit;

     // open window
     if(!(window = CA_OpenWindow(winobj)))
          goto thatsit;

     while(!done)
     {
          LONG signals;

          signals = Wait((1L << window->UserPort->mp_SigBit) |
                         SIG_KILL);

          if(signals & SIG_KILL)
          {
               abort = TRUE;
               Signal(FindTask(NULL), SIG_KILL);
               break;
          }

          if(signals & (1L << window->UserPort->mp_SigBit))
          {
               LONG result, code = 0;

               while(WMHI_LASTMSG != (result = CA_HandleInput(winobj, code)))
               {
                    switch(result & WMHI_CLASSMASK)
                    {
                         case WMHI_CLOSEWINDOW:
                              abort = TRUE;
                              done = TRUE;
                              break;
                         case WMHI_GADGETUP:
                              switch(result & WMHI_GADGETMASK)
                              {
                                   LONG val;

                                   case GAD_SELECTADDRESS_LISTBROWSER:
                                        GetAttr(LISTBROWSER_RelEvent, browser, (ULONG *)&val);
                                        if(val & LBRE_DOUBLECLICK)
                                        {
                                             GetAttr(LISTBROWSER_NumSelected, browser, (ULONG *)&val);
                                             if(val > 0)
                                                  done = TRUE;
                                        }
                                        GetAttr(LISTBROWSER_NumSelected, browser, (ULONG *)&val);
                                        if(SetGadgetAttrs(okay, window, NULL,
                                                  GA_Disabled,        (val < 1),
                                                  TAG_END))
                                             RefreshGList(okay, window, NULL, 1);
                                        break;
                                   case GAD_USE:
                                        GetAttr(LISTBROWSER_NumSelected, browser, (ULONG *)&val);
                                        if(val < 1)
                                             break;
                                        done = TRUE;
                                        break;
                              }
                              break;
                    }
               }
          }
     }

     thatsit:
     if(winobj)
          DisposeObject(winobj);

     if(list)
     {
          struct Node *node;
          LONG x = 0;

          while(node = RemHead(list))
          {
               if(!abort && alist)
               {    // compile list of addresses
                    AddressEntry *ae, *nae;
                    LONG sel;

                    GetListBrowserNodeAttrs(node,
                              LBNA_Selected,      &sel,
                              TAG_END);
                    if(!sel)
                         goto cont;

                    if(nae = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR))
                    {
                         if(ae = MatchAddress(x))
                         {
                              memcpy(nae, ae, sizeof(AddressEntry));
                              AddTail(alist, (struct Node *)nae);
                         }
                         else
                              FreeVec(nae);
                    }
               }

               cont:
               FreeListBrowserNode(node);
               x++;
          }

          FreeVec(list);
     }

     if(abort && alist)
     {
          FreeVec(alist);
          alist = NULL;
     }

     return(alist);
}

void FreeSelectedAddresses(struct List *list)
{
     if(list)
     {
          AddressEntry *ae;

          while(ae = (AddressEntry *)RemHead(list))
               FreeVec(ae);

          FreeVec(list);
     }
}

AddressEntry *SelectAddressGroup(void)
{
     struct Window *window;
     Object *winobj = NULL;
     struct Gadget *layout, *browser;
     AddressEntry *ae = NULL;
     struct Node *node;
     struct List list;
     BOOL done = FALSE, abort = FALSE;

     // build address group list
     NewList(&list);

     if(node = AllocListBrowserNode(1,
               LBNA_UserData,      1,
               LBNA_Column,        0,
               LBNCA_Text,         LocaleString(ETXT_SELECTADDRESSGROUP_ROOT),
               TAG_END))
     {
          AddTail(&list, node);
     }
     else
          return(0);

     ObtainSemaphore(&euc.Locks.Addresses);
     for(ae = (AddressEntry *)euc.Addresses.lh_Head;
         ae->Node.ln_Succ;
         ae = (AddressEntry *)ae->Node.ln_Succ)
     {
          if(!ae->Members)
               continue;

          if(node = AllocListBrowserNode(1,
                    LBNA_UserData,      ae,
                    LBNA_Column,        0,
                         LBNCA_CopyText,     TRUE,
                         LBNCA_Text,         ae->FirstName,
                    TAG_END))
          {
               AddTail(&list, node);
          }
     }
     ReleaseSemaphore(&euc.Locks.Addresses);
     ae = NULL;

     if(!(winobj = NewObject(WINDOW_GetClass(), NULL,
               WINDOW_Position,         WPOS_CENTERMOUSE,
               WA_InnerWidth,           100,
               WA_InnerHeight,          275,

               WA_Activate,             TRUE,
               WA_CloseGadget,          TRUE,
               WA_DragBar,              TRUE,
               WA_DepthGadget,          TRUE,
               WA_SizeGadget,           TRUE,
               WA_SizeBBottom,          TRUE,
               WA_RMBTrap,              TRUE,
               WINDOW_BackFillName,     sprefs->Backfill,

               WA_PubScreen,            euc.Screen,
               WA_Title,                LocaleString(ETXT_TITLE_SELECTADDRESSGROUP),
               WA_ScreenTitle,          eucsctitle,

               WINDOW_Layout,           layout = VGroupObject,
                    LAYOUT_DeferLayout, TRUE,
                    LAYOUT_ShrinkWrap,  TRUE,
                    StartMember,        browser = ListBrowserObject,
                         LISTBROWSER_Labels,      &list,
                         LISTBROWSER_ShowSelected,TRUE,
                         GA_ID,                   GAD_SELECTADDRESSGROUP_LISTBROWSER,
                         GA_RelVerify,            TRUE,
                    EndMember,
               End,
               TAG_END)))
          goto thatsit;

     // open window
     if(!(window = CA_OpenWindow(winobj)))
          goto thatsit;

     while(!done)
     {
          LONG signals;

          signals = Wait((1L << window->UserPort->mp_SigBit) |
                         SIG_KILL);

          if(signals & SIG_KILL)
          {
               abort = TRUE;
               ae = NULL;
               Signal(FindTask(NULL), SIG_KILL);
               break;
          }

          if(signals & (1L << window->UserPort->mp_SigBit))
          {
               LONG result, code = 0;

               while(WMHI_LASTMSG != (result = CA_HandleInput(winobj, code)))
               {
                    switch(result & WMHI_CLASSMASK)
                    {
                         case WMHI_CLOSEWINDOW:
                              done = TRUE;
                              break;
                         case WMHI_GADGETUP:
                              switch(result & WMHI_GADGETMASK)
                              {
                                   LONG val;

                                   case GAD_SELECTADDRESSGROUP_LISTBROWSER:
                                        GetAttr(LISTBROWSER_SelectedNode, browser, (ULONG *)&node);
                                        if(node)
                                        {
                                             GetListBrowserNodeAttrs(node,
                                                       LBNA_UserData,      (LONG *)&ae,
                                                       TAG_END);
                                        }

                                        GetAttr(LISTBROWSER_RelEvent, browser, (ULONG *)&val);
                                        if(val & LBRE_DOUBLECLICK)
                                             done = TRUE;
                                        break;
                              }
                              break;
                    }
               }
          }
     }

     thatsit:
     if(winobj)
          DisposeObject(winobj);

     while(node = RemHead(&list))
          FreeListBrowserNode(node);

     return(ae);
}

#endif
