/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: compose.c
 *   	 Mail composition functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "eucalyptus.h"

Composition *compose_new(void)
{
	Composition *comp;
	
	if(comp = g_malloc0(sizeof(Composition)))
	{
		compose_init(comp);
	}

	return comp;
}

void compose_free(Composition *comp)
{
	GList *node;
	
	if(!comp)
		return;

	/* Get rid of the window.
	 */
	if(comp->Window)
		gtk_widget_destroy(comp->Window);
		
	/* Free lists.
	 */
	while(node = g_list_first(comp->Attachments.List))
	{
		CompAttachment *catt = (CompAttachment *)node->data;
		
		comp->Attachments.List = g_list_remove(comp->Attachments.List, catt);
		compose_attachment_free(catt);
	}
	while(node = g_list_first(comp->CustomHeaders.List))
	{
		gchar *str = (gchar *)node->data;
		
		comp->CustomHeaders.List = g_list_remove(comp->CustomHeaders.List, str);
		g_free(str);
	}
	
	g_free(comp);
}

void compose_init(Composition *comp)
{
	list_init(&comp->Attachments);
	list_init(&comp->CustomHeaders);

	comp->str.To = g_string_new("");
	comp->str.Cc = g_string_new("");
	comp->str.Bcc = g_string_new("");
}


