/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: compose_file.c
 *   	 File access functions for composes
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>

#include "eucalyptus.h"

gboolean compose_insert_file_uuencoded(Composition *comp, gchar *infile)
{
	struct stat *st = NULL;
	gchar buf[1024];
	gboolean success = TRUE;
	FILE *fp;

	busy_pointer(comp->Window, TRUE);
	
	gtk_text_freeze(GTK_TEXT(comp->w.Textfield));
	gtk_editable_set_editable(GTK_EDITABLE(comp->w.Textfield), FALSE);

	st = path_stat(infile);
	if(fp = fopen(infile, "r"))
	{
		glong n, x;
		gchar inbuf[101];

		g_snprintf(inbuf, sizeof(inbuf), "\nbegin %o %s\n", 
				(st ? st->st_mode : 0644) & 0777,
				path_file_part(infile));
		gtk_text_insert(GTK_TEXT(comp->w.Textfield), NULL, NULL, NULL, inbuf, -1);

		// read file
		while(n = fread(buf, 1, 45, fp))
		{
			glong i;

			memset(inbuf, 0, sizeof(inbuf));

			inbuf[0] = ENC(n);

			i = 0;
			x = 1;
			while(i < n)
			{
				if((inbuf[x] = ENC(buf[i] >> 2)) == ' ')
					inbuf[x] = '`';
				if((inbuf[x + 1] = ENC(((buf[i] << 4) & 060) | ((buf[i + 1] >> 4) & 017))) == ' ')
					inbuf[x + 1] = '`';
				if((inbuf[x + 2] = ENC(((buf[i + 1] << 2) & 074) | ((buf[i + 2] >> 6) & 03))) == ' ')
					inbuf[x + 2] = '`';
				if((inbuf[x + 3] = ENC(buf[i + 2] & 077)) == ' ')
					inbuf[x + 3] = '`';
				i += 3;
				x += 4;
			}
			inbuf[x] = '\n';

			gtk_text_insert(GTK_TEXT(comp->w.Textfield), NULL, NULL, NULL, inbuf, -1);
		}

		gtk_text_insert(GTK_TEXT(comp->w.Textfield), NULL, NULL, NULL, "end\n", -1);

		fclose(fp);
	}
	else
		success = FALSE;
		
	g_free(st);
	
	gtk_text_thaw(GTK_TEXT(comp->w.Textfield));
	gtk_editable_set_editable(GTK_EDITABLE(comp->w.Textfield), TRUE);

	busy_pointer(comp->Window, FALSE);
	
	return success;
}

gboolean compose_insert_form(Composition *comp, gchar *fname)
{
	gchar buf[1024];
	gboolean success = FALSE;
	FILE *fp;

	gtk_text_freeze(GTK_TEXT(comp->w.Textfield));
	gtk_editable_set_editable(GTK_EDITABLE(comp->w.Textfield), FALSE);

	if(fp = fopen(fname, "r"))
	{
		glong c;
		gchar *ch, *data;

		while(fgets(buf, sizeof(buf) - 1, fp))
		{
			if(ch = strchr(buf, '\r'))
				*ch = '\0';
			if(ch = strchr(buf, '\n'))
				*ch = '\0';

			if(!strlen(buf))
				break;

			if(ch = strchr(buf, ' '))
			{
				*ch = '\0';
				data = ch + 1;
			}
			else
				data = "";

#if 0
			if(!stricmp(buf, "Subject:"))
			{
				strncpy(comp->Subject, data, sizeof(comp->Subject) - 1);
				gtk_entry_set_text(GTK_ENTRY(comp->w.Subject), data);

				continue;
			}
			if(!stricmp(buf, "Reply-To:"))
			{
				strncpy(comp->ReplyTo, data, sizeof(comp->ReplyTo) - 1);
				gtk_entry_set_text(GTK_ENTRY(comp->w.ReplyTo), data);

				continue;
			}
			if(!stricmp(buf, "To:"))
			{
				AddCompAddress(comp, data, ACAFT_PARSE, CAT_TO);
				continue;
			}
			if(!stricmp(buf, "Cc:"))
			{
				AddCompAddress(comp, data, ACAFT_PARSE, CAT_CC);
				continue;
			}
			if(!stricmp(buf, "Bcc:"))
			{
				AddCompAddress(comp, data, ACAFT_PARSE, CAT_BCC);
				continue;
			}
			if(!stricmp(buf, "Attachment:"))
			{
				CompAttachment *catt;

				if(catt = NewAttachment(data))
				{
					SetListBrowserNodeAttrs((struct Node *)catt,
							LBNA_Column,   		0,
								LBNCA_Text,    		catt->Name,
							LBNA_Column,   		1,
								LBNCA_Text,    		catt->ContentType,
							TAG_END);
					SetGadgetAttrs(comp->AttachListBrowser, NULL, NULL,
							LISTBROWSER_Labels, 		~0,
							TAG_END);
					AddTail(&comp->Attachments, (struct Node *)catt);
					SetGadgetAttrs(comp->AttachListBrowser, NULL, NULL,
							LISTBROWSER_Labels, 		&comp->Attachments,
							LISTBROWSER_AutoFit,		TRUE,
							TAG_END);
				}
				continue;
			}
#endif
		}

		while((c = fgetc(fp)) != -1)
		{
#if 0
			switch(c)
			{
				gboolean date;

				case '%':
					date = FALSE;
					switch(c = fgetc(fp))
					{
						struct timerequest *tr;
						struct ClockData clockdata;

						case '%':
							strncpy(buf, "%", WORKLEN - 1);
							break;
						case 'f': // from
							sprintf(buf, "%s <%s>",
										uprefs->RealName,
										uprefs->Address);
							break;
						case 'n': // from (name only)
							strncpy(buf, uprefs->RealName, WORKLEN - 1);
							break;
						case 'a': // from (addr only)
							strncpy(buf, uprefs->Address, WORKLEN - 1);
							break;
						case 's': // subject
							strncpy(buf, comp->Subject, WORKLEN - 1);
							break;
						case 'd': // date
							date = TRUE;
						case 't': // time
							if(tr = AllocVec(sizeof(struct timerequest), MEMF_CLEAR))
							{
								struct timeval *tv;

								if(tv = AllocVec(sizeof(struct timeval), MEMF_CLEAR))
								{
									if(!OpenDevice("timer.device", UNIT_VBLANK, (struct IORequest *)tr, 0))
									{
										struct Library *TimerBase;

										TimerBase = (struct Library *)tr->tr_node.io_Device;

#pragma msg 88 ignore push
										GetSysTime((struct timeval *)tv);
#pragma msg 88 pop
										Amiga2Date(tv->tv_secs, &clockdata);

										if(date)
										{
											LONG locale_weekdays[] = {DAY_1, DAY_2, DAY_3, DAY_4, DAY_5, DAY_6, DAY_7, };
											LONG locale_months[] = {MON_1, MON_2, MON_3, MON_4, MON_5, MON_6, MON_7, MON_8, MON_9, MON_10, MON_11, MON_12, };
											STRPTR weekdays[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday",};
											STRPTR months[] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December",};

											sprintf(comp->work, "%s, %ld %s %ld",
														euc.Locale ? GetLocaleStr(euc.Locale, locale_weekdays[clockdata.wday]) : weekdays[clockdata.wday],
														(LONG)clockdata.mday,
														euc.Locale ? GetLocaleStr(euc.Locale, locale_months[clockdata.month - 1]) : months[clockdata.month - 1],
														(LONG)clockdata.year);
										}
										else
										{
											sprintf(comp->work, "%ld%c%ld%c%ld",
														(LONG)clockdata.hour, ':',
														(LONG)clockdata.min, ':',
														(LONG)clockdata.sec);
										}

										CloseDevice((struct IORequest *)tr);
									}

									FreeVec(tv);
								}

								FreeVec(tr);
							}
							break;
						case 'V': // program version
							strncpy(buf, __EUCALYPTUS_VERSION__, WORKLEN - 1);
							break;
						case 'R': // program release
							strncpy(buf, euc_rel, WORKLEN - 1);
							break;
						case 'O': // OS version
							{
								UBYTE *os_table[] = {"2.0", "2.1", "3.0", "3.1", "3.5?"};

								strncpy(buf, os_table[min(41, max(37, SysBase->LibNode.lib_Version)) - 37], WORKLEN - 1);
							}
							break;
						case 'P':  // target-processor of program
							strncpy(buf, target_os, WORKLEN - 1);
							break;
						default:
							sprintf(buf, "%%%c", c);
							break;
					}
					break;
				case '\r':
					break;
				default:
					sprintf(buf, "%c", c);
					break;
			}
			
			gtk_text_insert(GTK_TEXT(comp->w.Textfield), 
					NULL, NULL, NULL,
					buf,
					-1);
#endif
		}

		fclose(fp);
	}
	else
		success = FALSE;

	gtk_text_thaw(GTK_TEXT(comp->w.Textfield));
	gtk_editable_set_editable(GTK_EDITABLE(comp->w.Textfield), TRUE);

	return success;
}

gboolean compose_insert_file(Composition *comp, glong type, gchar *infile)
{
	gchar buf[1024];
	gboolean success = TRUE;
	FILE *fp;
	
	busy_pointer(comp->Window, TRUE);

	gtk_text_freeze(GTK_TEXT(comp->w.Textfield));
	gtk_editable_set_editable(GTK_EDITABLE(comp->w.Textfield), FALSE);

	if(type == COMPOSE_INSERTFILE_BRACKETED)
	{
		gtk_text_insert(GTK_TEXT(comp->w.Textfield),
				NULL, NULL, NULL,
				"\n*** INSERTED TEXT ***\n\n",
				-1);
	}

	if(fp = fopen(infile, "r"))
	{
		// read actual text
		while(fgets(buf, sizeof(buf) - 1, fp))
		{
			gchar *c;

			if(type == COMPOSE_INSERTFILE_INDENTED)
			{
				gtk_text_insert(GTK_TEXT(comp->w.Textfield),
						NULL, NULL, NULL,
						prefs.QuotePrefix,
						-1);
				gtk_text_insert(GTK_TEXT(comp->w.Textfield),
						NULL, NULL, NULL,
						" ",
						-1);
			}

			if(c = strchr(buf, '\r'))
				*c = '\0';
			if(c = strchr(buf, '\n'))
				*c = '\0';

			gtk_text_insert(GTK_TEXT(comp->w.Textfield),
					NULL, NULL, NULL,
					buf,
					-1);
			gtk_text_insert(GTK_TEXT(comp->w.Textfield),
					NULL, NULL, NULL,
					"\n",
					-1);
		}

		fclose(fp);
	}
	else
		success = FALSE;

	if(type == COMPOSE_INSERTFILE_BRACKETED)
	{
		gtk_text_insert(GTK_TEXT(comp->w.Textfield),
				NULL, NULL, NULL,
				"\n*** END INSERTED TEXT ***\n\n",
				-1);
	}
	
	gtk_text_thaw(GTK_TEXT(comp->w.Textfield));
	gtk_editable_set_editable(GTK_EDITABLE(comp->w.Textfield), TRUE);

	busy_pointer(comp->Window, FALSE);
	
	return success;
}

gboolean compose_save_file(Composition *comp, gchar *outfile)
{
	gchar *text;
	FILE *fp;
	
	g_return_val_if_fail(comp != NULL, FALSE);
	g_return_val_if_fail(outfile != NULL, FALSE);

	busy_pointer(comp->Window, TRUE);

	text = gtk_editable_get_chars(GTK_EDITABLE(comp->w.Textfield), 0, -1);

	if(text)
	{
		if(fp = fopen(outfile, "w"))
		{
			fputs(text, fp);

			fclose(fp);
		}
		
		g_free(text);
	}

	busy_pointer(comp->Window, FALSE);
}

gboolean compose_insert_file_form(Composition *comp, glong type)
{
#if 0
			if(getfile = NewObject(GETFILE_GetClass(), NULL,
					GETFILE_Drawer,		"PROGDIR:Forms",
					TAG_END))
			{
				if(CA_RequestFile(getfile, comp->Window))
				{
					STRPTR infile;

					GetAttr(GETFILE_FullFile, getfile, (ULONG *)&infile);
					if(infile)
					{
						InsertFormFile(comp, infile);
					}
				}

				DisposeObject(getfile);
			}
#endif
}

