#ifndef EUCALYPTUS_EMSGS_H
#define EUCALYPTUS_EMSGS_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Header: emsgs.h
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

/* Message passing structures.
 */
struct _eMsg
{
	long	type;
	long subtype;
};

/* eMsg.type values
 */
enum
{
	MSG_TYPE_NEW=1,	/* new msg in folder */
	MSG_TYPE_GENERIC,	/* general msg handling command */
	MSG_TYPE_SEND,		/* send msg */
	MSG_TYPE_PROGRESS,	/* visual update notification */
	MSG_TYPE_COPY,		/* copy/move msg to folder */
	MSG_TYPE_UPDATE,	/* tell folder to rebuild index */
	MSG_TYPE_ERROR,	/* display an error requester */
	MSG_TYPE_STATUS,	/* display something in the main status bar */
	MSG_TYPE_DIE=50	/* thread-killer */
};

/* eMsg.subtype values
 */
enum
{
     MSG_SUBTYPE_PROGRESS_OPEN=1,
	MSG_SUBTYPE_PROGRESS_CLOSE
};

/* === NewMsg ======================= */
 
struct _NewMsg
{
	eMsg msg;
	gint16 flags;
	Folder *folder;
	gchar filename[FILENAME_MAX];
};

/* NewMsg.flags values
 */
enum
{
	NMF_DELETED = 1 << 0,
	NMF_UPDATED = 1 << 1
};

/* === SendMsg ======================= */
 
struct _SendMsg
{
	eMsg msg;
	gint16 flags;
	gchar filename[FILENAME_MAX];
	gchar id[33];
};

/* SendMsg.flags values
 */
enum
{
	CMF_SKIP = 1 << 0
};

/* These values are special constants for
 * MoveMsg.dest for the required folders.
 */
#define MSG_DEST_INCOMING	((Folder *)1)
#define MSG_DEST_OUTGOING	((Folder *)2)
#define MSG_DEST_POSTPONED	((Folder *)3)
#define MSG_DEST_SENT		((Folder *)4)

/* === ProgressMsg ======================= */
 
struct _ProgressMsg
{
	eMsg msg;
	gchar who;
	long total;
	long current;
	gchar text[201];
	gchar id[301];
	Server *server;
};

/* VisualMsg.who values
 */
enum
{
	MSG_WHO_SMTP=1,
	MSG_WHO_POP
};

/* === CopyMsg ======================= */

struct _CopyMsg
{
	eMsg msg;
	Folder *source;
	Folder *dest;
	gboolean move;
	gchar filename[FILENAME_MAX];
};

/* === GenericMsg ======================= */
 
struct _GenericMsg
{
	eMsg msg;
	glong action;
	Folder *folder;
	gchar filename[FILENAME_MAX];
};

/* GenericMsg.action values
 */
enum
{
	MSG_ACTION_DELETE=1,	/* delete the msg */
	MSG_ACTION_HOLD,		/* hold msg from being sent */
	MSG_ACTION_PRINT,		/* print msg */
};

/* === UpdateMsg ======================= */
 
struct _UpdateMsg
{
	eMsg msg;
	Folder *folder;
};


/* === StatusMsg ======================= */
 
struct _StatusMsg
{
	eMsg msg;
	gchar text[201];
};

/* === ErrorMsg ======================= */
 
struct _ErrorMsg
{
	eMsg msg;
	glong error;
	gchar text[301];
};

#endif /* EUCALYPTUS_EMSGS_H */
