/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: folder_info.c
 *   	 Folder information requester
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>

#include "eucalyptus.h"

void folder_info(Folder *folder)
{
	FolderInfo *finfo;
	GtkWidget *table, *vbox;

	finfo = g_new0(FolderInfo, 1);
	if(finfo->folder = folder)
	{
		memcpy(&finfo->info, &folder->Data, sizeof(struct FolderData));
	}

	finfo->window = gnome_dialog_new(
			folder ? _("Edit folder") : _("Create new folder"),
			GNOME_STOCK_BUTTON_OK, 
			GNOME_STOCK_BUTTON_CANCEL, 
			NULL);

	table = gtk_table_new(7, 4, FALSE);
	{
		GtkWidget *label, *button;
		glong top = 0;

		/* Name.
		 * (Row 0)
		 */
		label = gtk_label_new(_("Name"));
		gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
		gtk_table_attach(GTK_TABLE(table), label,
				0, 1, top, top + 1,
				GTK_FILL,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		
		finfo->w.name = gtk_entry_new_with_max_length(sizeof(finfo->info.Name) - 1);
		gtk_entry_set_text(GTK_ENTRY(finfo->w.name), finfo->info.Name);
		gtk_signal_connect(GTK_OBJECT(finfo->w.name), "changed",
				GTK_SIGNAL_FUNC(folder_info_name_changed), finfo);
		gtk_table_attach(GTK_TABLE(table), finfo->w.name,
				1, 3, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		top++;

		/* Parent.
		 * (Row 1)
		 */
		label = gtk_label_new(_("Parent"));
		gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
		gtk_table_attach(GTK_TABLE(table), label,
				0, 1, top, top + 1,
				GTK_FILL,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		
		finfo->w.parent = gtk_entry_new_with_max_length(sizeof(finfo->info.BelongsTo) - 1);
		gtk_entry_set_text(GTK_ENTRY(finfo->w.parent), finfo->info.BelongsTo);
		gtk_table_attach(GTK_TABLE(table), finfo->w.parent,
				1, 3, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);

		button = gtk_button_new_with_label("...");
		gtk_signal_connect(GTK_OBJECT(button), "clicked",
				GTK_SIGNAL_FUNC(folder_info_select_parent), finfo);
		gtk_table_attach(GTK_TABLE(table), button,
				3, 4, top, top + 1,
				GTK_FILL,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		top++;

		/* Spool file.
		 * Not for Postponed, Outgoing, or Sent folders.
		 * (Row 2)
		 */
		if(!(finfo->info.Type & (FTF_POSTPONED | FTF_OUTGOING | FTF_SENT)))
		{
			label = gtk_label_new(_("Spool file"));
			gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
			gtk_table_attach(GTK_TABLE(table), label,
					0, 1, top, top + 1,
					GTK_FILL,
					GTK_FILL,
					GNOME_PAD_SMALL, GNOME_PAD_SMALL);
			
			finfo->w.spool = gtk_entry_new_with_max_length(sizeof(finfo->info.SpoolFile) - 1);
			gtk_entry_set_text(GTK_ENTRY(finfo->w.spool), finfo->info.SpoolFile);
			gtk_table_attach(GTK_TABLE(table), finfo->w.spool,
					1, 3, top, top + 1,
					GTK_FILL | GTK_EXPAND,
					GTK_FILL,
					GNOME_PAD_SMALL, GNOME_PAD_SMALL);

			button = gtk_button_new_with_label("...");
			gtk_signal_connect(GTK_OBJECT(button), "clicked",
					GTK_SIGNAL_FUNC(folder_info_select_spool_file), finfo);
			gtk_table_attach(GTK_TABLE(table), button,
					3, 4, top, top + 1,
					GTK_FILL,
					GTK_FILL,
					GNOME_PAD_SMALL, GNOME_PAD_SMALL);
			top++;
		}

		/* Default "to"
		 * (Row 3)
		 */
		label = gtk_label_new(_("Default \"To:\""));
		gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
		gtk_table_attach(GTK_TABLE(table), label,
				0, 1, top, top + 1,
				GTK_FILL,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		
		finfo->w.defaultto = gtk_entry_new_with_max_length(sizeof(finfo->info.DefaultTo) - 1);
		gtk_entry_set_text(GTK_ENTRY(finfo->w.defaultto), finfo->info.DefaultTo);
		gtk_table_attach(GTK_TABLE(table), finfo->w.defaultto,
				1, 3, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		top++;

		/* Sort.
		 * (Row 4)
		 */
		label = gtk_label_new(_("Sort"));
		gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
		gtk_table_attach(GTK_TABLE(table), label,
				0, 1, top, top + 1,
				GTK_FILL,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		finfo->w.sort = gtk_option_menu_new();
		{
			GtkWidget *menu, *item;
			GSList *group;
			glong i;
			gchar *items[] = { N_("None"), N_("Date"), N_("Name"), N_("Subject"), N_("Size"), N_("Status"), NULL, };

			menu = gtk_menu_new();
			group = NULL;

			for(i=0; items[i]; i++)
			{
				item = gtk_radio_menu_item_new_with_label(group, _(items[i]));
				group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
				gtk_menu_append(GTK_MENU(menu), item);

				gtk_object_set_user_data(GTK_OBJECT(item), GINT_TO_POINTER(i));

				gtk_signal_connect(GTK_OBJECT(item), "activate",
						GTK_SIGNAL_FUNC(folder_info_sort_select), finfo);
			}

			gtk_option_menu_set_menu(GTK_OPTION_MENU(finfo->w.sort), menu);
		}
		gtk_table_attach(GTK_TABLE(table), finfo->w.sort,
				1, 3, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		top++;

		/* Open at startup.
		 * (Row 5, Column 1)
		 */
		finfo->w.startup = gtk_check_button_new_with_label(_("Open at startup?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(finfo->w.startup), finfo->info.OpenAtStartup);
		gtk_table_attach(GTK_TABLE(table), finfo->w.startup,
				1, 2, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);

		/* Reverse sort.
		 * (Row 5, Column 2)
		 */
		finfo->w.reverse = gtk_check_button_new_with_label(_("Reverse sort?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(finfo->w.reverse), finfo->info.SortBy & FSF_REVERSE);
		gtk_table_attach(GTK_TABLE(table), finfo->w.reverse,
				2, 3, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		top++;

		/* Apply filters on import.
		 * (Row 6, Column 1)
		 */
		finfo->w.applyfilters = gtk_check_button_new_with_label(_("Apply filters on import?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(finfo->w.applyfilters), finfo->info.ApplyFiltersOnImport);
		gtk_table_attach(GTK_TABLE(table), finfo->w.applyfilters,
				1, 2, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);

		/* Thread messages.
		 * (Row 6, Column 2)
		 */
		finfo->w.thread = gtk_check_button_new_with_label(_("Thread messages?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(finfo->w.thread), finfo->info.SortBy & FSF_THREAD);
		/* set value */
		gtk_table_attach(GTK_TABLE(table), finfo->w.thread,
				2, 3, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		top++;

		/* Protect messages.
		 * (Row 7, Column 1)
		 */
		finfo->w.protect = gtk_check_button_new_with_label(_("Protect messages?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(finfo->w.protect), finfo->info.ProtectMsgs);
		gtk_table_attach(GTK_TABLE(table), finfo->w.protect,
				1, 2, top, top + 1,
				GTK_FILL | GTK_EXPAND,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);

		/* Window position.
		 * (Rows 8-9, Columns 1-2)
		 */
	}
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(finfo->window)->vbox), table, TRUE, TRUE, 0);

	gtk_signal_connect(GTK_OBJECT(finfo->window), "clicked",
			GTK_SIGNAL_FUNC(folder_info_clicked), finfo);
	gtk_signal_connect(GTK_OBJECT(finfo->window), "close",
			GTK_SIGNAL_FUNC(folder_info_close), finfo);

	gnome_dialog_set_sensitive(GNOME_DIALOG(finfo->window), 0, FALSE);

	gtk_widget_show_all(finfo->window);
}

void folder_info_clicked(GnomeDialog *dialog, gint button, gpointer data)
{
	Folder *folder = NULL;
	FolderInfo *finfo = (FolderInfo *)data;
	gchar *text;
	gboolean success = TRUE;

	if(button == 1)
	{
		gnome_dialog_close(dialog);
		return;
	}

	/* Grab values.
	 */
	text = gtk_editable_get_chars(GTK_EDITABLE(finfo->w.name), 0, -1);
	strncpy(finfo->info.Name, text, sizeof(finfo->info.Name) - 1);
	g_free(text);
	text = gtk_editable_get_chars(GTK_EDITABLE(finfo->w.parent), 0, -1);
	strncpy(finfo->info.BelongsTo, text, sizeof(finfo->info.BelongsTo) - 1);
	g_free(text);
	text = gtk_editable_get_chars(GTK_EDITABLE(finfo->w.spool), 0, -1);
	strncpy(finfo->info.SpoolFile, text, sizeof(finfo->info.SpoolFile) - 1);
	g_free(text);
	text = gtk_editable_get_chars(GTK_EDITABLE(finfo->w.defaultto), 0, -1);
	strncpy(finfo->info.DefaultTo, text, sizeof(finfo->info.DefaultTo) - 1);
	g_free(text);
	finfo->info.ApplyFiltersOnImport = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(finfo->w.applyfilters));
	finfo->info.ProtectMsgs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(finfo->w.protect));
	finfo->info.OpenAtStartup = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(finfo->w.startup));
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(finfo->w.thread)))
		finfo->info.SortBy |= FSF_THREAD;
	else
		finfo->info.SortBy &= ~FSF_THREAD;
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(finfo->w.reverse)))
		finfo->info.SortBy |= FSF_REVERSE;
	else
		finfo->info.SortBy &= ~FSF_REVERSE;

	if(finfo->folder)
	{
		/* Preserve some current folder values.
		 */
		finfo->info.Flags = finfo->folder->Data.Flags;
		finfo->info.NextMsgNum = finfo->folder->Data.NextMsgNum;
		finfo->info.MsgCount = finfo->folder->Data.MsgCount;
		finfo->info.NewMsgs = finfo->folder->Data.NewMsgs;
		finfo->info.UnreadMsgs = finfo->folder->Data.UnreadMsgs;
		finfo->info.NotifyCount = finfo->folder->Data.NotifyCount;
		finfo->info.Size = finfo->folder->Data.Size;

		/* Copy.
		 */
		memcpy(&finfo->folder->Data, &finfo->info, sizeof(struct FolderData));

		folder = finfo->folder;
	}
	else
	{
		strncpy(finfo->info.Directory, finfo->info.Name, sizeof(finfo->info.Directory) - 1);
		errno = 0;
		if(folder = folder_new(&finfo->info))
		{
			folder_gather_info(folder, TRUE);
			list_append(&euc.Folders, folder);

			euc_update_listview();
		}
		else
		{
			gchar *msg;
			msg = g_strdup_printf("Unable to create new folder:\n%s", strerror(errno));
			euc_request(finfo->window,
					_("Eucalyptus error"),
					msg,
					euc_ok_button, 0,
					0);
			g_free(msg);
			success = FALSE;
		}
	}

	if(success)
	{
		gnome_dialog_close(dialog);
		folder_write_data(folder, TRUE);
	}
}

void folder_info_close(GtkWidget *widget, gpointer data)
{
	FolderInfo *finfo = (FolderInfo *)data;

	gtk_widget_unref(finfo->window);
	if(finfo->filesel)
		gtk_widget_unref(finfo->filesel);
	if(finfo->sf)
		folder_select_cancel(NULL, (gpointer)finfo->sf);
	g_free(finfo);
}

void folder_info_select_spool_file(GtkButton *button, gpointer data)
{
	FolderInfo *finfo = (FolderInfo *)data;

	if(finfo->filesel)
		return;
	
	finfo->filesel = gtk_file_selection_new(_("Select folder spool file"));
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(finfo->filesel), finfo->info.SpoolFile);

	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(finfo->filesel)->ok_button), "clicked",
			GTK_SIGNAL_FUNC(folder_info_spool_selected), finfo);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(finfo->filesel)->ok_button), "clicked",
			GTK_SIGNAL_FUNC(folder_info_spool_destroy), finfo);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(finfo->filesel)->cancel_button), "clicked",
			GTK_SIGNAL_FUNC(folder_info_spool_destroy), finfo);

	gtk_widget_show_all(finfo->filesel);
}

void folder_info_spool_selected(GtkWidget *widget, gpointer data)
{
	FolderInfo *finfo = (FolderInfo *)data;
	gchar *file;

	file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(finfo->filesel));
	strncpy(finfo->info.SpoolFile, file, sizeof(finfo->info.SpoolFile) - 1);
}

void folder_info_spool_destroy(GtkWidget *widget, gpointer data)
{
	FolderInfo *finfo = (FolderInfo *)data;

	gtk_widget_unref(finfo->filesel);
	finfo->filesel = NULL;
}

void folder_info_select_parent(GtkButton *button, gpointer data)
{
	FolderInfo *finfo = (FolderInfo *)data;

	folder_select_open_window(SELECT_FOLDER_INFO, SFF_ROOT, (gpointer)finfo);
}

void folder_info_sort_select(GtkMenuItem *item, gpointer data)
{
	FolderInfo *finfo = (FolderInfo *)data;
	gint num = 0;

	num = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(item)));
	finfo->info.SortBy = num;
}

void folder_info_name_changed(GtkEntry *entry, gpointer data)
{
	FolderInfo *finfo = (FolderInfo *)data;
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
	gnome_dialog_set_sensitive(GNOME_DIALOG(finfo->window), 0, *text);
	g_free(text);
}

