/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: gui_folder.c
 *   	 GUI-related functions for folders
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "eucalyptus.h"
#include "gui_menus.h"
#include "gui_toolbars.h"

gboolean folder_display_contents(Folder *folder)
{
	GnomeUIInfo *menu, *toolbar;
	GtkWidget *hbox, *scrw;
	gchar *titles[] = { N_("From"), N_("Date"), N_("Size"), N_("Flags"), N_("Subject"), };
	glong i;
	gboolean success = TRUE;

	g_return_val_if_fail(folder != NULL, 0);

	if(euc.SelectedFolder == folder)
		return TRUE;

	if(euc.FolderLayout)
		gtk_container_remove(GTK_CONTAINER(euc.Pane), euc.FolderLayout);

	/* Create the folder pane contents.
	 */
	euc.FolderLayout = gtk_vbox_new(FALSE, 5);
	gtk_signal_connect(GTK_OBJECT(euc.FolderLayout), "destroy",
			GTK_SIGNAL_FUNC(folder_layout_destroy_cb), folder);
	gtk_paned_add2(GTK_PANED(euc.Pane), euc.FolderLayout);

	euc.SelectedFolder = folder;

	/* Create folder list tree.
	 */
	scrw = gtk_scrolled_window_new(NULL, NULL);
	{
		glong i;

		/* List.
		 */
		if(folder->Data.Type & (FTF_POSTPONED | FTF_OUTGOING | FTF_SENT))
			   titles[0] = N_("To");
		for(i=0; i<5; i++)
			   titles[i] = _(titles[i]);
		euc.MsgList = gtk_ctree_new_with_titles(5, 0, titles);
		gtk_clist_set_selection_mode(GTK_CLIST(euc.MsgList), GTK_SELECTION_MULTIPLE);
		GTK_CLIST_SET_FLAG(euc.MsgList, CLIST_REORDERABLE);
		gtk_clist_set_column_min_width(GTK_CLIST(euc.MsgList), 0, 150);
		gtk_clist_set_column_auto_resize(GTK_CLIST(euc.MsgList), 0, TRUE);
		gtk_clist_set_column_auto_resize(GTK_CLIST(euc.MsgList), 3, TRUE);
		gtk_clist_set_column_auto_resize(GTK_CLIST(euc.MsgList), 4, TRUE);

		gtk_signal_connect(GTK_OBJECT(euc.MsgList), "tree-select-row",
				GTK_SIGNAL_FUNC(folder_select_msg_cb), folder);
		gtk_signal_connect(GTK_OBJECT(euc.MsgList), "tree-unselect-row",
				GTK_SIGNAL_FUNC(folder_unselect_msg_cb), folder);
		gtk_signal_connect(GTK_OBJECT(euc.MsgList), "resize-column",
				GTK_SIGNAL_FUNC(folder_resize_column_cb), folder);
				
		/* Add to scrolled window.
		 */
		gtk_container_add(GTK_CONTAINER(scrw), euc.MsgList);
		
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrw), 
				GTK_POLICY_AUTOMATIC,	/* horizontal */
				GTK_POLICY_AUTOMATIC);	/* vertical */
	}
	gtk_box_pack_start(GTK_BOX(euc.FolderLayout), scrw, TRUE, TRUE, 0);
	
	/* Show it.
	 */
	gtk_widget_show_all(euc.FolderLayout);

	/* Populate it.
	 */
	folder_update_visual(folder); 
	 
	return success;
}

void folder_update_visual(Folder *folder)
{
	GtkCTreeNode *parent = NULL, *sibling = NULL, *ctnode;
	GList *node;
	glong row = -1;
	
	g_return_if_fail(folder != NULL);

	/* Update msg list.
	 */
	gtk_clist_freeze(GTK_CLIST(euc.MsgList));
	gtk_clist_clear(GTK_CLIST(euc.MsgList));

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		MailMsg *msg = (MailMsg *)node->data;
		GdkColor *pen = NULL;
		gchar *cols[6], flags[21];

		/* Set row.
	      */
		row++;

		memset(cols, 0, sizeof(cols));

		/* From. (Column 0)
		 */
		cols[0] = msg->Index.From;

		/* Date. (Column 1)
		 */
		mailmsg_build_date_string(msg);
	     if(folder->Data.Type & FTF_POSTPONED)
     	     memset(msg->t.Date, 0, sizeof(msg->t.Date));
     	     
     	cols[1] = msg->t.Date;

		/* Msg size. (Column 2)
		 */
	     if(msg->Index.Size > (1024 * 1024))
	     {
	     	g_snprintf(msg->t.Size, sizeof(msg->t.Size), _("%.1fMB"), msg->Index.Size / (1024.0 * 1024.0));
	     }
	     else if(msg->Index.Size > 1024)
	     {
	     	g_snprintf(msg->t.Size, sizeof(msg->t.Size), _("%.1fKB"), msg->Index.Size / 1024.0);
	     }
	     else
	     {
	     	g_snprintf(msg->t.Size, sizeof(msg->t.Size), "%d", msg->Index.Size);
	     }
	     
	     cols[2] = msg->t.Size;

		/* Flags. (Column 3)
		 */
		strcpy(msg->t.Flags, "");

	     if(!(folder->Data.Type & (FTF_OUTGOING | FTF_SENT | FTF_POSTPONED)))
	     {
	          if(msg->Index.Flags & MIEF_NEW)
	               strncat(msg->t.Flags, _("N"), sizeof(msg->t.Flags) - 1);
	          if(msg->Index.Flags & MIEF_UNREAD)
	               strncat(msg->t.Flags, _("U"), sizeof(msg->t.Flags) - 1);
	          if(msg->Index.Flags & MIEF_REPLIED)
	               strncat(msg->t.Flags, _("R"), sizeof(msg->t.Flags) - 1);
	          if(msg->Index.Flags & MIEF_FORWARDED)
	               strncat(msg->t.Flags, _("F"), sizeof(msg->t.Flags) - 1);
	          if(msg->Index.Flags & MIEF_BOUNCED)
	               strncat(msg->t.Flags, _("B"), sizeof(msg->t.Flags) - 1);
	     }
	     if(msg->Index.Flags & MIEF_DELETED)
	          strncat(msg->t.Flags, _("D"), sizeof(msg->t.Flags) - 1);
	     if(msg->Index.Flags & MIEF_DONTDELETE)
	          strncat(msg->t.Flags, _("P"), sizeof(msg->t.Flags) - 1);
	     if(msg->Index.Flags & MIEF_NOTIFY)
	          strncat(msg->t.Flags, _("I"), sizeof(msg->t.Flags) - 1);
	     if(msg->Index.Flags & MIEF_ATTACHMENTS)
	          strncat(msg->t.Flags, _("A"), sizeof(msg->t.Flags) - 1);
	     if((msg->Index.Flags & MIEF_SUSPEND) &&
	        (msg->Folder->Data.Type & FTF_OUTGOING))
			strncat(msg->t.Flags, _("S"), sizeof(msg->t.Flags) - 1);
	     if(msg->Index.Flags & MIEF_PERSONAL)
	          pen = &prefs.PersonalPen;
	     if(msg->Index.Flags & MIEF_PRIORITY)
	          pen = &prefs.PriorityPen;

		cols[3] = msg->t.Flags;
		
		/* Subject. (Column 4)
		 */
		cols[4] = msg->Index.Subject;
		
		/* Add the node to the list.
		 */
		ctnode = msg->TreeNode = gtk_ctree_insert_node(GTK_CTREE(euc.MsgList),
				parent, sibling,
				cols, 0,
				NULL, NULL,
				NULL, NULL,
				TRUE,
				TRUE);
		gtk_ctree_node_set_row_data(GTK_CTREE(euc.MsgList), ctnode, msg);

		if(pen)
			gtk_ctree_node_set_foreground(GTK_CTREE(euc.MsgList),
					ctnode,
					pen);

		if(msg->Flags & MMF_SELECTED)
			 gtk_ctree_select(GTK_CTREE(euc.MsgList), ctnode);
	}

	gtk_clist_thaw(GTK_CLIST(euc.MsgList));

}

void folder_update_info(Folder *folder)
{
	if(!folder)
		return;
		
	if(folder->Data.MsgCount)
	{
		if(!(folder->Data.Type & FTF_POSTPONED))
		{
		     if(folder->Data.Size > (1024 * 1024))
		     {
				g_snprintf(folder->t.Size, sizeof(folder->t.Size),
		     			_("%.1fMB"), 
						folder->Data.Size / (1024.0 * 1024.0));
		     }
		     else if(folder->Data.Size > 1024)
		     {
				g_snprintf(folder->t.Size, sizeof(folder->t.Size),
		     			_("%.1fKB"), 
						folder->Data.Size / 1024.0);
		     }
		     else
		     {
				g_snprintf(folder->t.Size, sizeof(folder->t.Size),
		     			_("%d"), 
						folder->Data.Size);
		     }
		}

		if(folder->Data.Type & (FTF_OUTGOING | FTF_POSTPONED | FTF_SENT))
		{
			g_snprintf(folder->t.Counts, sizeof(folder->t.Counts),
					"%d",
					folder->Data.MsgCount);
		}
		else
		{
			g_snprintf(folder->t.Counts, sizeof(folder->t.Counts),
					"%d/%d",
					folder->Data.UnreadMsgs + folder->Data.NewMsgs,
					folder->Data.MsgCount);
		}
	}
	else
	{
		strncpy(folder->t.Counts, _("Empty"), sizeof(folder->t.Counts) - 1);
		strncpy(folder->t.Size, _("n/a"), sizeof(folder->t.Size) - 1);
	}
}

void folder_set_status(Folder *folder, gchar *text)
{
	if(!text)
		text = folder->t.Status;

	gnome_appbar_set_status(GNOME_APPBAR(euc.StatusBar), text);
}

gboolean folder_open_msg(Folder *folder, MailMsg *msg)
{
	gboolean success = FALSE;

	if(!folder ||
	   !msg)
		return FALSE;

	busy_pointer(euc.Window, TRUE);

	if(mailmsg_open_window(msg))
	{
		if(msg->Index.Flags & (MIEF_NEW | MIEF_UNREAD))
		{
			msg->Index.Flags &= ~(MIEF_NEW | MIEF_UNREAD);
			mailmsg_update_status(msg);
			if(mailmsg_update_info(msg, TRUE))
				;

			folder_update_index(folder);
			folder_update_visual(folder);

//				SetFolderIndexValid(folder, FALSE);

			if((msg->Index.Flags & MIEF_RETURNRECEIPTREQ) &&
			   !prefs.DisableReturnReceipt)
			{
				/* Ask user's permission.
				 */
				if(!euc_request(msg->Window,
						_("Eucalyptus request"),
						_("The sender of this message requests a\n"
						  "return-receipt.  Do you wish to send one?"),
						euc_yes_no_buttons, 0,
						0))
					mailmsg_generate_receipt(msg);
			}
		}

		errno = 0;

		if(!msgpart_display(msg, (MsgPart *)g_list_nth_data(msg->Parts.List, msg->CurrentPart)))
		{
			gchar *emsg;

			euc_request(euc.Window,
					_("Eucalyptus error"),
					emsg = g_strdup_printf("%s:\n%s", _("Unable to display message part"), strerror(errno)),
					euc_ok_button, 0,
					0);
			g_free(emsg);
		}
	}
	else
	{
		gchar *emsg;
		
		euc_request(euc.Window,
				_("Eucalyptus error"),
				emsg = g_strdup_printf("%s:\n%s", _("Unable to display message"), strerror(errno)),
				euc_ok_button, 0,
				0);
		g_free(emsg);
	}

	busy_pointer(euc.Window, FALSE);

	return success;
}

/* Callbacks.
 */
gint folder_close_requested_cb(GtkWidget *window, GdkEventAny *e, gpointer data)
{
	 /* Destroy the window.
	  */
	 return FALSE;
}

void folder_layout_destroy_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		folder = euc.SelectedFolder;

	euc.Layout = NULL;
}

void folder_select_msg_cb(GtkCTree *tree, GList *node, gint column, gpointer data)
{
	Folder *folder;
	MailMsg *msg, *old_msg;
	GdkEvent *event;
	gboolean sepw = FALSE;

	/* Set selected folder.
	 */
	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
	if(msg = (MailMsg *)gtk_ctree_node_get_row_data(tree, (GtkCTreeNode *)node))
	{
		msg->Flags |= MMF_SELECTED;
	}

	/* Check for right-click.
	 */
	event = gdk_event_peek();

	/* Check for double-click.
	 */
	if(event && (event->type == GDK_2BUTTON_PRESS))
	{
		folder->SelectedMsg = msg;
		folder_open_msg(folder, msg);
		euc.CurrentMsg = msg;
		return;
	}

	folder->SelectedMsg = msg;
	euc.CurrentMsg = msg;
}

void folder_unselect_msg_cb(GtkCTree *tree, GList *node, gint column, gpointer data)
{
	MailMsg *msg;
	GdkEvent *event;

	/* Set selected folder.
	 */
	if(msg = (MailMsg *)gtk_ctree_node_get_row_data(tree, (GtkCTreeNode *)node))
		msg->Flags &= ~MMF_SELECTED;
}

void folder_resize_column_cb(GtkCList *clist, gint column, gint width, gpointer data)
{
	/* FIXME */
}

void folder_destroy_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	/* Do cleanup.
	 */
	folder_close(folder);

	/* Reset some vars.
	 */
	euc.Window = NULL;
}

void folder_check_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	glong count = 0;

	if(!folder)
		   folder = euc.SelectedFolder;
	if(!folder_verify_directory(folder))
		return;
	
	/* If this is the incoming folder, check for 
	 * mail in the spool file, and read it.
	 */
	if(folder->Data.Type & FTF_INCOMING)
	{
		gchar *spool_file;
		
		spool_file = g_strdup_printf("/var/spool/mail/%s", g_get_user_name());
		count = folder_import_msgs(folder, spool_file, TRUE);
		g_free(spool_file);
	}
	
	/* Check for mail in a folder-specific
	 * spool.
	 */
	count += folder_import_msgs(folder, folder->Data.SpoolFile, TRUE);
	
	/* Re-read or build index, if necessary.
	 */
	if(count)
	{
		folder_update_index(folder);
		folder_update_visual(folder);
	}
}

void folder_refresh_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
	
	folder_build_index(folder);
	folder_update_visual(folder);
}

void folder_import_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

}

void folder_apply_filters_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

}

void folder_expunge_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	if(euc_request(euc.Window,
				    _("Eucalyptus question"),
				    _("Expunge all deleted messages from this folder?"),
				    euc_yes_no_buttons, 1,
				    0))
		   return;
	
	if(folder_expunge(folder))
	{
		folder_update_index(folder);
		folder_update_visual(folder);
		euc_update_listview();
	}
}

void folder_close_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;
	gboolean update = FALSE;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;
	
		mailmsg_close_window(msg);
	}
}

void folder_reply_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(!mailmsg_create_reply(msg, MRT_REPLY, 0, NULL))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to create reply message."),
					euc_ok_button, 0,
					EUCRF_ASYNC);
	}
}

void folder_reply_all_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(!mailmsg_create_reply(msg, MRT_REPLY, MRTT_ALL, NULL))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to create reply message."),
					euc_ok_button, 0,
					EUCRF_ASYNC);
	}
}

void folder_reply_notext_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(!mailmsg_create_reply(msg, MRT_REPLY, MRTT_NOTEXT, NULL))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to create reply message."),
					euc_ok_button, 0,
					EUCRF_ASYNC);
	}
}

void folder_reply_all_notext_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(!mailmsg_create_reply(msg, MRT_REPLY, MRTT_ALL | MRTT_NOTEXT, NULL))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to create reply message."),
					euc_ok_button, 0,
					EUCRF_ASYNC);
	}
}

void folder_forward_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(!mailmsg_create_reply(msg, MRT_FORWARD, 0, NULL))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to create reply message."),
					euc_ok_button, 0,
					EUCRF_ASYNC);
	}
}

void folder_forward_brief_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(!mailmsg_create_reply(msg, MRT_FORWARD, MRTT_SOMEHEADERS, NULL))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to create reply message."),
					euc_ok_button, 0,
					EUCRF_ASYNC);
	}
}

void folder_forward_full_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(!mailmsg_create_reply(msg, MRT_FORWARD, MRTT_ALLHEADERS, NULL))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to create reply message."),
					euc_ok_button, 0,
					EUCRF_ASYNC);
	}
}

void folder_select_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

}

void folder_select_all_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	gtk_clist_select_all(GTK_CLIST(euc.MsgList));	
}

void folder_status_read_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;
	gboolean update = FALSE;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;
		
		update = TRUE;
		msg->Index.Flags &= ~(MIEF_NEW | MIEF_UNREAD);
	}
	
	if(update)
	{
		folder_update_visual(folder);
		folder_update_index(folder);
	}
}

void folder_status_unread_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;
	gboolean update = FALSE;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;
		
		update = TRUE;
		msg->Index.Flags &= ~MIEF_NEW;
		msg->Index.Flags |= MIEF_UNREAD;
	}
	
	if(update)
	{
		folder_update_visual(folder);
		folder_update_index(folder);
	}
}

void folder_status_important_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;
	gboolean update = FALSE;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;
		
		update = TRUE;
		msg->Index.Flags |= MIEF_NOTIFY;
	}
	
	if(update)
	{
		folder_update_visual(folder);
		folder_update_index(folder);
	}
}

void folder_status_read_all_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		msg->Index.Flags &= ~(MIEF_NEW | MIEF_UNREAD);
	}
	
	folder_update_visual(folder);
	folder_update_index(folder);
}

void folder_status_unread_all_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		msg->Index.Flags &= ~MIEF_NEW;
		msg->Index.Flags |= MIEF_UNREAD;
	}
	
	folder_update_visual(folder);
	folder_update_index(folder);
}

void folder_status_set_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

}

void folder_read_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	MailMsg *msg;
	GList *node;

	folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;
		
		folder_open_msg(folder, msg);
	}
}

void folder_bounce_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

}

void folder_print_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

}

void folder_delete_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	gint disp = DELETE_DISPOSITION_DELETE;
	const gchar *buttons[] = { N_("Delete"), N_("Toggle"), N_("Undelete"), N_("Cancel"), NULL, };
	
	if(!folder)
		   folder = euc.SelectedFolder;
	/* Check the deletion state of all selected
	 * messages, and query the user accordingly.
	 */
	switch(folder_check_selections(folder, MIEF_DELETED))
	{
		case FCSR_ALL:
#if 0
			if(euc_request(euc.Window,
					_("Eucalyptus query"),
					_("Undelete selected messages?"),
					euc_yes_no_buttons, 1,
					0))
				return;
#endif
			disp = DELETE_DISPOSITION_UNDELETE;
			break;
			
		case FCSR_MIXED:
			switch(euc_request(euc.Window,
					_("Eucalyptus query"),
					_("What would you like to do with the selected messages:\n"
					  "Mark all deleted; toggle deletion; mark all undeleted?"),
					buttons, 1,
					0))
			{
				case 1:	/* toggle */
					disp = DELETE_DISPOSITION_TOGGLE;
					break;
					
				case 2:	/* undelete */
					disp = DELETE_DISPOSITION_UNDELETE;
					break;
					
				case 3:	/* cancel */
					return;
			}
			break;
			
		default:	/* FCSR_NONE */
			break;
	}
	
	g_list_foreach(folder->Msgs.List, folder_mark_deleted, GINT_TO_POINTER(disp));

	folder_update_index(folder);
	folder_update_visual(folder);
}

void folder_move_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_copy_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;

	folder_select_open_window(SELECT_FOLDER_COPY, 0, folder);
}

void folder_export_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_get_addr_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_resume_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_send_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_edit_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_send_again_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_send_now_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_sort_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder = (Folder *)data;
	if(!folder)
		   folder = euc.SelectedFolder;
}

void folder_compose_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp;

	if(comp = compose_new())
	{
		list_append(&euc.Composes, comp);

		compose_open_window(comp);
	}
}

