/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: main.c
 *   	 Program initialization and main control routine
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include "e-config.h"

#include <popt-gnome.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define EUCALYPTUS_MAIN_C
#include "eucalyptus.h"

struct Eucalyptus euc;
struct Prefs prefs;
struct User uprefs;

/* An enumeration for cmdline args parsing.
 */
enum
{
	EARG_ADDRTO,
	EARG_ADDRCC,
	EARG_ADDRBCC,
	EARG_SUBJ,
	EARG_ATCH,
};

/* Command line options and related variables.
 */
gboolean quiet, compose;
struct poptOption options[] =
{
	{ "quiet", 'q', POPT_ARG_NONE, &quiet, 0, N_("Don't display an intro/exit requesters"), NULL, },
	{ "compose", 0, POPT_ARG_NONE, &compose, 0, N_("Open a compose window only"), NULL, },
	{ NULL, },
};

int main(int argc, char **argv)
{
	poptContext context;
	gint rc = 0;
	gchar **args;
	
	/* Clear out main program struct.
	 */
	memset(&euc, 0, sizeof(struct Eucalyptus));
	
	/* Do some basic program initialization.
	 */
	init_stuff();
	
	/* Initialize the threading environment.
	 */
	g_thread_init(NULL);

	/* Initialize the localization environment.
	 */
	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
	textdomain(PACKAGE);

	/* Initialize the GNOME environment.
	 */
	euc.Orb = gnome_CORBA_init_with_popt_table(APPNAME, VERSION, &argc, argv, options, 0, &context, 0, &euc.CorbaEnv);

	/* Handle command line arguments.
	 */
	if(quiet)
		euc.Flags |= EGF_QUIET;
	if(compose)
		euc.Flags |= EGF_COMPOSEONLY;
		
	args = (gchar **)poptGetArgs(context);
	if(args && compose)
	{
		glong i = 0;
		
		/* Create a compose object.
		 */
		
		
		/* Parse the arguments.
		 */
		while(args[i])
		{
			gchar *param, *c;
			glong type = EARG_ADDRTO;

			/* Handle "to:", "cc:", "bcc:", etc., from the cmdline.
			 */
			if(c = strchr(args[i], ':'))
			{
				*c = 0;
				param = c + 1;
				
				if(!strcasecmp(args[i], "cc"))
					type = EARG_ADDRCC;
				else if(!strcasecmp(args[i], "bcc"))
					type = EARG_ADDRBCC;
				else if(!strcasecmp(args[i], "subj"))
					type = EARG_SUBJ;
				else if(!strcasecmp(args[i], "atch"))
					type = EARG_ATCH;
			}
			else
				param = args[i];
				
			/* Add the specified items to the compose.
			 */
			switch(type)
			{
				case EARG_ADDRCC:
					break;
					
				case EARG_ADDRBCC:
					break;
					
				case EARG_SUBJ:
					break;
					
				case EARG_ATCH:
					break;
					
				default:	// A "to" address
					break;
			}
		
			i++;
		}
		
		/* Add the object to the pending list for
		 * starting later.
		 */
		 
		 
	}
	poptFreeContext(context);

#ifdef USE_SESSIONMGMT
	/* Start session management.
	 */
	session_start_client();
#endif
	
	/* Initialize settings structures.
	 */
	prefs_init();	

	/* Get User info.
	 */
	user_get_info();

	/* Get user settings.
	 */
	prefs_load();
	
	/* Create the main window.
	 */
	if(!euc_startup(TRUE))
		euc_all_done(1);
	
	/* Start the program.
	 */	
	gdk_threads_enter();
	gtk_main();

	/* Clean up.
	 */	
	euc_all_done(0);
	return 0;
}

