/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: misc.c
 *   	 Miscellaneous functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "eucalyptus.h"

#if 0
void swap(glong *v1, glong *v2)
{
	glong tv;

	tv = *v1;
	*v1 = *v2;
	*v2 = tv;
}

struct Node *FindPartialName(struct List *list, UBYTE *name)
{
	struct Node *node = NULL;

	if(!list)
		return(NULL);

	if(IsListEmpty(list))
		return(NULL);

	for(node = list->lh_Head;
	    node->ln_Succ;
	    node = node->ln_Succ)
	{
		if(node->ln_Name)
			if(!strnicmp(node->ln_Name, name, strlen(name)))
				return(node);
	}

	return(NULL);
}

UBYTE toUpper(UBYTE c)
{
	UBYTE *lowchars = "abcdefghijklmnopqrstuvwxyz",
		 *highchars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
		  cpos = 0, rc;

	for(cpos = 0, rc = c; lowchars[cpos]; cpos++)
	{
		if(lowchars[cpos] == c)
		{
			rc = highchars[cpos];
			break;
		}
	}

	return(rc);
}

#endif

void euc_temp_filename(gchar *dir, gchar *buf, glong buflen)
{
     glong x;
     gchar *name = NULL;

     for(x=0;; x++)
     {
     	gchar path[FILENAME_MAX];
          FILE *fp;
          
          g_snprintf(path, sizeof(path) - 1, ".{eucalyptus,%d-%d}", getpid(), x);
          name = path_build(dir ? dir : g_get_tmp_dir(), path, NULL);

          if(fp = fopen(name, "r"))
          {
          	g_free(name);
               fclose(fp);
          }
          else
               break;
     }
     
     strncpy(buf, name, buflen - 1);
     g_free(name);
}

#if 0
BYTE CompareMsgDate(ULONG date1, ULONG date2)
{
	return((BYTE)((date1 < date2) ? -1 : ((date1 > date2) ? 1 : 0)));
}

/*
BYTE CompareMsgDate(struct MsgDate *date1, struct MsgDate *date2)
{
	LONG year1, year2;
	BYTE comp = 0;

	year1 = date1->Year;
	year2 = date2->Year;
	if(date1->AbbrevYear)
	{
		if(year1 < 50)
			year1 += 2000;
		else
			year1 += 1900;
	}
	if(date2->AbbrevYear)
	{
		if(year2 < 50)
			year2 += 2000;
		else
			year2 += 1900;
	}

	if(year1 == year2)
	{
		if(date1->Month == date2->Month)
		{
			if(date1->Day == date2->Day)
			{
				LONG hours1, hours2;

				hours1 = date1->Hours + (gprefs->UseGMTOffset ? (date1->Offset / 100) : 0);
				hours2 = date2->Hours + (gprefs->UseGMTOffset ? (date2->Offset / 100) : 0);
				if(hours1 == hours2)
				{
					if(date1->Minutes == date2->Minutes)
					{
						if(date1->Seconds == date2->Seconds)
						{
							return(0);
						}
						else if(date1->Seconds < date2->Seconds)
							return(-1);
						else if(date1->Seconds > date2->Seconds)
							return(1);
					}
					else if(date1->Minutes < date2->Minutes)
						return(-1);
					else if(date1->Minutes > date2->Minutes)
						return(1);
				}
				else if(date1->Hours < date2->Hours)
					return(-1);
				else if(date1->Hours > date2->Hours)
					return(1);
			}
			else if(date1->Day < date2->Day)
				return(-1);
			else if(date1->Day > date2->Day)
				return(1);
		}
		else if(date1->Month < date2->Month)
			return(-1);
		else if(date1->Month > date2->Month)
			return(1);
	}
	else if(year1 < year2)
		return(-1);
	else if(year1 > year2)
		return(1);

	return(comp);
}
*/

LONG GetMsgAge(ULONG md /* struct MsgDate *md */)
{
	struct timerequest *tr;
	LONG age = 0, error = 0, now = 0;

	if(tr = AllocVec(sizeof(struct timerequest), MEMF_CLEAR))
	{
		struct timeval *tv;

		if(tv = AllocVec(sizeof(struct timeval), MEMF_CLEAR))
		{
			if(!OpenDevice("timer.device", UNIT_VBLANK, (struct IORequest *)tr, 0))
			{
				struct Library *TimerBase;

				TimerBase = (struct Library *)tr->tr_node.io_Device;

				GetSysTime(tv);
				now = tv->tv_secs;

				CloseDevice((struct IORequest *)tr);
			}
			else
				error = err_UNABLETOGETDATE;

			FreeVec(tv);
		}
		else
			error = err_UNABLETOGETDATE;

		FreeVec(tr);
	}
	else
		error = err_UNABLETOGETDATE;

	if(!error)
	{
		struct ClockData cdate;
		LONG year, then = md; // 0;

/*
		year = md->Year;
		if(md->AbbrevYear)
		{
			if(year < 50)
				year += 2000;
			else
				year += 1900;
		}
		cdate.year = year;
		cdate.sec = abs(min(md->Seconds, 59));
		cdate.min = abs(min(md->Minutes, 59));
		cdate.hour = abs(min(md->Hours, 23));
		cdate.mday = md->Day;
		cdate.month = md->Month;
		cdate.wday = md->Weekday;

		then = CheckDate(&cdate);
*/

		if(now && then)
			age = abs((now - then) / (24 * 60 * 60));

//   	if(age > 2500) /* If the age is over 6 years old, it's probably invalid. */
//   		age = 0;
	}

	return(age);
}

LONG ConvertTabs(UBYTE *text, LONG len, UBYTE tabwidth)
{
	LONG error = 0 /* , i, j, l, p;
	UBYTE *c */ ;

	if(!text)
		return(0);

/*
	while(c = strchr(text, '\t'))
	{
		p = c - text;

		strcpy(c, c + 1);
		l = strlen(text);

		for(j=l; j>p; j--)
		{
			if((j < len) &&
			   (j + tabwidth + 1 < len))
				text[j + tabwidth + 1] = text[j];
			else
				error = err_OVERFLOW;
		}
		text[len - 1] = '\0';

		for(i=0; i<tabwidth; i++)
			text[p + i] = ' ';
	}
*/

	return(error);
}


#endif
