/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: msg_security.c
 *         Message security functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include "eucalyptus.h"

gboolean mailmsg_is_signed(MailMsg *msg)
{
     HdrNode *content;
	gchar *str;

     if(!msg ||
        !msg->Folder)
          return FALSE;

     if(!(content = header_find_tag("Content-Type", msg->Headers, 0)))
          return FALSE;

     if(str = content_type_find_param(content->Data, "multipart/signed", FALSE))
     {
		g_free(str);

          if(!(str = content_type_find_param(content->Data, "micalg=", TRUE)))
               return FALSE;
          if(!(str = content_type_find_param(content->Data, "protocol=", TRUE)))
               return FALSE;

		g_free(str);

          return TRUE;
     }

     return FALSE;
}

gboolean mailmsg_is_encrypted(MailMsg *msg)
{
     HdrNode *content;
	gchar *str;

     if(!msg ||
        !msg->Folder)
          return FALSE;

     if(!(content = header_find_tag("Content-Type", msg->Headers, CHTF_CASECONV)))
          return FALSE;

     if(str = content_type_find_param(content->Data, "multipart/encrypted", FALSE))
     {
		g_free(str);

          if(!content_type_find_param(content->Data, "protocol=", TRUE))
          {
               return FALSE;
          }

		g_free(str);

          return TRUE;
     }

     return FALSE;
}

gboolean mailmsg_is_key_transport(MailMsg *msg)
{
     HdrNode *content;
	gchar *str;

     if(!msg ||
        !msg->Folder)
          return FALSE;

     if(!(content = header_find_tag("Content-Type", msg->Headers, CHTF_CASECONV)))
          return FALSE;

     if(str = content_type_find_param(content->Data, "multipart/keys", FALSE))
     {
		g_free(str);

#if 0 /* FIXME */
          if(IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_ATTACHEDKEY),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
          {
               ExtractMsgKeys(msg);
          }
#endif

          return TRUE;
     }

     return FALSE;
}

gboolean mailmsg_decrypt(MailMsg *msg)
{
	gboolean success = TRUE;
	
	
	
	return success;
}

gboolean mailmsg_check_signature(MailMsg *msg)
{
	gboolean success = TRUE;
	
	
	
	return success;
}

gboolean mailmsg_extract_keys(MailMsg *msg)
{
	gboolean success = TRUE;
	
	
	
	return success;
}

#if 0
LONG CheckMsgSignature(MailMsg *msg)
{
     struct MsgPart *mp, *mp1 = NULL, *mp2 = NULL;
     LONG error = 0;

     for(mp = (struct MsgPart *)msg->Parts.lh_Head;
         mp->Node.ln_Succ;
         mp = (struct MsgPart *)mp->Node.ln_Succ)
     {
          switch(mp->Number)
          {
               case 0:
                    mp1 = mp;
                    break;
               case 1:
                    mp2 = mp;
                    break;
          }
     }

     if(mp1 && mp2)
     {
          LONG CMDLEN = WORKLEN * 2, PASSLEN = WORKLEN;
          UBYTE tmpfile[256], *cmdline, *passphrase;
          BPTR cmd_fp;

          GetMsgTempName(msg, tmpfile, sizeof(tmpfile));

          /* save part 1 text (raw)
          if(error = SaveMsgPart(tmpfile, msg, mp1, SMPF_NODECODING | SMPF_SAVEHEADERS))
          {
               IsenRequest(euc.Screen,
                         ISENREQUEST_Text,        LocaleString(ETXT_ERRORPROCESSINGMSG),
                         TAG_END);
               return(error);
          }

          if(!strlen(mp2->Filename))
               if(error = ProcessMsgPart(msg, mp2, 0))
               {
                    IsenRequest(euc.Screen,
                              ISENREQUEST_Text,        LocaleString(ETXT_ERRORPROCESSINGMSG),
                              TAG_END);
                    return(error);
               }

          pgpcheck1:
          if(!strlen(oprefs->PGPPath))
          {
               struct List *list;

               if(list = GetFiles(LocaleString(ETXT_TITLE_SELECTPGPPATH), msg->Window,
                         TAG_END))
               {
                    strncpy(oprefs->PGPPath, list->lh_Head->ln_Name, sizeof(oprefs->PGPPath) - 1);

                    FreeFiles(list);

                    goto pgpcheck1;
               }
               else
                    return FALSE;
          }

          BusyPointer(msg->Window, TRUE);

          if(passphrase = (UBYTE *)AllocVec(PASSLEN, MEMF_CLEAR))
          {
               if(cmdline = (UBYTE *)AllocVec(CMDLEN, MEMF_CLEAR))
               {
                    if(GetVar("PGPPASS", passphrase, PASSLEN - 1, NULL) == -1)
                    {
                         RequestString(LocaleString(ETXT_ENTERPASSPHRASE), passphrase, PASSLEN, RSF_PASSWORD);
                    }

                    if(strlen(passphrase))
                    {
                         if(oprefs->UsePGP5)
                         {
                              strncpy(cmdline, "PGPv -zo ", CMDLEN - 1);
                              strncat(cmdline, tmpfile, CMDLEN - 1);
                              strncat(cmdline, " ", CMDLEN - 1);
                              strncat(cmdline, mp2->Filename, CMDLEN - 1);
                         }
                         else
                         {
                              strncpy(cmdline, oprefs->PGPPath, CMDLEN - 1);
                              strncat(cmdline, " ", CMDLEN - 1);
                              strncat(cmdline, mp2->Filename, CMDLEN - 1);
                              strncat(cmdline, " ", CMDLEN - 1);
                              strncat(cmdline, tmpfile, CMDLEN - 1);
                              strncat(cmdline, " -z \"", CMDLEN - 1);
                              strncat(cmdline, passphrase, CMDLEN - 1);
                              strncat(cmdline, "\"", CMDLEN - 1);
                         }

                         cmd_fp = Open("NIL:", MODE_OLDFILE);
                         switch(SystemTags(cmdline,
                                   SYS_Input,          cmd_fp,
                                   SYS_Output,         NULL,
                                   TAG_END))
                         {
                              case -1:  /* can't open shell
                                   IsenRequest(euc.Screen,
                                             ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOEXECUTECMD),
                                             TAG_END);
                                   if(cmd_fp)
                                        Close(cmd_fp);
                                   break;
                              case 0:   /* okay
                                   IsenRequest(euc.Screen,
                                             ISENREQUEST_Text,        LocaleString(ETXT_SIGNATUREGOOD),
                                             TAG_END);
                                   break;
                              default:  /* error from prog
                                   IsenRequest(euc.Screen,
                                             ISENREQUEST_Text,        LocaleString(ETXT_SIGNATUREBAD),
                                             TAG_END);
                                   break;
                         }

                         return FALSE;
                    }
                    else
                    {
                         IsenRequest(euc.Screen,
                                   ISENREQUEST_Text,        LocaleString(ETXT_PASSPHRASEREQUIRED),
                                   TAG_END);
                         error = err_NOPASSPHRASE;
                    }

                    FreeVec(cmdline);
               }
               else
                    error = err_INSUFFICIENTMEMORY;

               FreeVec(passphrase);
          }
          else
               error = err_INSUFFICIENTMEMORY;
     }

     BusyPointer(msg->Window, FALSE);

     return(error);
}

LONG DecryptMsg(MailMsg *msg)
{
     struct MsgPart *mp;
     LONG error = 0;

     if(!msg)
          return FALSE;

     if(mp = (struct MsgPart *)GetListNum(&msg->Parts, 1))
     {
          LONG CMDLEN = WORKLEN * 2, PASSLEN = WORKLEN;
          UBYTE tmpfile[256], *cmdline, *passphrase;
          BPTR cmd_fp;

          GetMsgTempName(msg, tmpfile, sizeof(tmpfile));

          pgpcheck1:
          if(!strlen(oprefs->PGPPath))
          {
               struct List *list;

               if(list = GetFiles(LocaleString(ETXT_TITLE_SELECTPGPPATH), msg->Window,
                         TAG_END))
               {
                    strncpy(oprefs->PGPPath, list->lh_Head->ln_Name, sizeof(oprefs->PGPPath) - 1);

                    FreeFiles(list);

                    goto pgpcheck1;
               }
               else
                    return FALSE;
          }

          BusyPointer(msg->Window, TRUE);

          if(passphrase = (UBYTE *)AllocVec(PASSLEN, MEMF_CLEAR))
          {
               if(cmdline = (UBYTE *)AllocVec(CMDLEN, MEMF_CLEAR))
               {
                    if(GetVar("PGPPASS", passphrase, PASSLEN - 1, NULL) == -1)
                    {
                         RequestString(LocaleString(ETXT_ENTERPASSPHRASE), passphrase, PASSLEN, RSF_PASSWORD);
                    }

                    if(strlen(passphrase))
                    {
                         if(oprefs->UsePGP5)
                         {
                              strncpy(cmdline, "PGPv -zo ", CMDLEN - 1);
                              strncat(cmdline, tmpfile, CMDLEN - 1);
                              strncat(cmdline, " ", CMDLEN - 1);
                              strncat(cmdline, msg->Index.Filename, CMDLEN - 1);
                         }
                         else
                         {
                              strncpy(cmdline, oprefs->PGPPath, CMDLEN - 1);
                              strncat(cmdline, " ", CMDLEN - 1);
                              strncat(cmdline, msg->Index.Filename, CMDLEN - 1);
                              strncat(cmdline, " -o ", CMDLEN - 1);
                              strncat(cmdline, tmpfile, CMDLEN - 1);
                              strncat(cmdline, " -z \"", CMDLEN - 1);
                              strncat(cmdline, passphrase, CMDLEN - 1);
                              strncat(cmdline, "\"", CMDLEN - 1);
                         }

                         cmd_fp = Open("NIL:", MODE_OLDFILE);
                         switch(SystemTags(cmdline,
                                   SYS_Input,          cmd_fp,
                                   SYS_Output,         NULL,
                                   TAG_END))
                         {
                              struct AsyncFile *old_fp, *new_fp;

                              case -1:  /* can't open shell
                                   IsenRequest(euc.Screen,
                                             ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOEXECUTECMD),
                                             TAG_END);
                                   if(cmd_fp)
                                        Close(cmd_fp);
                                   break;
                              case 0:   /* okay
                                   if(old_fp = OpenAsync(tmpfile, MODE_READ, ASYNCIO_BUF, SysBase, DOSBase))
                                   {
                                        UBYTE tmp2[331], buf[1024];

                                        GetMsgTempName(msg, tmp2, sizeof(tmp2));

                                        if(new_fp = OpenAsync(tmp2, MODE_WRITE, ASYNCIO_BUF, SysBase, DOSBase))
                                        {
                                             gboolean gothdr = FALSE;

                                             while(FGetsAsync(old_fp, buf, sizeof(buf) - 1))
                                             {
                                                  UBYTE *c;

                                                  if(gothdr)
                                                  {
                                                       WriteLineAsync(new_fp, buf);
                                                  }
                                                  else
                                                  {
                                                       if(c = strchr(buf, '\n'))
                                                            *c = '\0';
                                                       if(c = strchr(buf, '\r'))
                                                            *c = '\0';

                                                       if(!strlen(buf))
                                                            gothdr = TRUE;
                                                  }
                                             }

                                             strncpy(mp->Filename, tmp2, sizeof(mp->Filename) - 1);

                                             CloseAsync(new_fp);
                                        }

                                        CloseAsync(old_fp);
                                   }
                                   break;
                              default:  /* error from prog
                                   IsenRequest(euc.Screen,
                                             ISENREQUEST_Text,        LocaleString(ETXT_UNABLETODECRYPT),
                                             TAG_END);
                                   break;
                         }
                    }
                    else
                    {
                         IsenRequest(euc.Screen,
                                   ISENREQUEST_Text,        LocaleString(ETXT_PASSPHRASEREQUIRED),
                                   TAG_END);
                         error = err_NOPASSPHRASE;
                    }

                    FreeVec(cmdline);
               }
               else
                    error = err_INSUFFICIENTMEMORY;

               FreeVec(passphrase);
          }
          else
               error = err_INSUFFICIENTMEMORY;
     }
     else
          error = err_UNKNOWNMSGPART;

     BusyPointer(msg->Window, FALSE);

     return(error);
}

LONG ExtractMsgKeys(MailMsg *msg)
{
     LONG error = 0;
     UBYTE cmdline[256];
     BPTR cmd_fp;

     if(!msg)
          return FALSE;

     pgpcheck1:
     if(!strlen(oprefs->PGPPath))
     {
          struct List *list;

          if(list = GetFiles(LocaleString(ETXT_TITLE_SELECTPGPPATH), msg->Window,
                    TAG_END))
          {
               strncpy(oprefs->PGPPath, list->lh_Head->ln_Name, sizeof(oprefs->PGPPath) - 1);

               FreeFiles(list);

               goto pgpcheck1;
          }
          else
               return FALSE;
     }

     BusyPointer(msg->Window, TRUE);

     if(oprefs->UsePGP5)
     {
          strncpy(cmdline, "PGPk -za ", sizeof(cmdline) - 1);
          strncat(cmdline, msg->Index.Filename, sizeof(cmdline) - 1);
     }
     else
     {
          strncpy(cmdline, oprefs->PGPPath, sizeof(cmdline) - 1);
          strncat(cmdline, " -ka ", sizeof(cmdline) - 1);
          strncat(cmdline, msg->Index.Filename, sizeof(cmdline) - 1);
     }

     cmd_fp = Open("CON:0/0/640/200/Eucalyptus Output/AUTO/SIMPLE", MODE_OLDFILE);
     switch(SystemTags(cmdline,
               SYS_Input,          cmd_fp,
               SYS_Output,         NULL,
               TAG_END))
     {
          case -1:  /* can't open shell */
               IsenRequest(euc.Screen,
                         ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOEXECUTECMD),
                         TAG_END);
               if(cmd_fp)
                    Close(cmd_fp);
               break;
          case 0:   /* okay */
               break;
          default:  /* error from prog */
               IsenRequest(euc.Screen,
                         ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOEXTRACTKEYS),
                         TAG_END);
               break;
     }

     BusyPointer(msg->Window, FALSE);

     return(error);
}

#endif
