/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: msg_support.c
 *         Support functions for messages
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "eucalyptus.h"

static gchar days[7][4]    = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
static gchar months[12][4] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

MailMsg *mailmsg_find_by_filename(Folder *folder, gchar *fname)
{
	MailMsg *m1;
	GList *node;

	g_return_val_if_fail(folder != NULL, NULL);
	g_return_val_if_fail(fname != NULL, NULL);
	
	node = g_list_first(folder->Msgs.List);
	while(node)
	{
		m1 = (MailMsg *)node->data;
		
		if(!strcasecmp(path_file_part(fname), m1->Index.Filename))
			return m1;

		node = g_list_next(node);
	}

	return NULL;
}

gchar *mailmsg_get_temp_name(MailMsg *msg)
{
     glong x;
     gchar *name = NULL;

     for(x=0;; x++)
     {
     	gchar path[FILENAME_MAX];
          FILE *fp;
          
          g_snprintf(path, sizeof(path) - 1, "eucalyptus{%s,%d-%d}", msg->Index.Filename, getpid(), x);
          name = path_build(g_get_tmp_dir(), path, NULL);

          if(fp = fopen(name, "r"))
          {
          	g_free(name);
               fclose(fp);
          }
          else
               break;
     }

     return name;
}

gchar *mailmsg_get_full_path(MailMsg *msg)
{
	gchar *path = NULL, *fpath;
	
	g_return_val_if_fail(msg != NULL, NULL);

	fpath = folder_get_full_path(msg->Folder);
	path = path_build(fpath, msg->Index.Filename, NULL);
	g_free(fpath);
		
	return path;
}

void mailmsg_build_date_string(MailMsg *msg)
{
	struct tm *tm;
	
	g_return_if_fail(msg != NULL);
	
	tm = &msg->Index.Date;

	g_snprintf(msg->t.Date, sizeof(msg->t.Date), "%s, %d %s %4.4d, %2.2d:%2.2d:%2.2d",
			days[tm->tm_wday],
			tm->tm_mday, months[tm->tm_mon], tm->tm_year,
			tm->tm_hour, tm->tm_min, tm->tm_sec);

	/* FIXME
	 * Do a real, configurable date string!
	 */
}

glong mailmsg_check_content_type(MailMsg *msg)
{
	glong error = 0;
	
	
	
	return error;
}

void mailmsg_generate_receipt(MailMsg *msg)
{
#if 0

	Folder *folder;
	struct List *hdrs;
	struct HdrNode *hnode;
	struct AsyncFile *mfp;
	LONG error = 0;
	UBYTE work[256];
	STRPTR eoltypes[] = {"\r\n", "\r", "\n", };

	if(!msg)
		return(0);

	if(gprefs->DisableReturnReceipt)
		return(0);

	if(msg->Index.Flags & (MIEF_RECEIPTED | MIEF_PERSONAL))
		return(0);

	WriteLogEntry("GenerateReturnReceipt", LOGENTRY_Text, "Generating receipt...", TAG_END);

	if(mfp = OpenAsync(msg->Index.Filename, MODE_READ, ASYNCIO_BUF, SysBase, DOSBase))
	{
		if(hdrs = GetMsgHeader(GMHT_ASYNCIO, (APTR)mfp, NULL))
		{
			if(hnode = FindHeader("Return-Receipt-To", hdrs, 0))
			{
				struct ClockData clockdata;
				struct timerequest *tr;

				if(tr = AllocVec(sizeof(struct timerequest), MEMF_CLEAR))
				{
					struct timeval *tv;

					if(tv = AllocVec(sizeof(struct timeval), MEMF_CLEAR))
					{
						if(!OpenDevice("timer.device", UNIT_VBLANK, (struct IORequest *)tr, 0))
						{
							struct Library *TimerBase;

							TimerBase = (struct Library *)tr->tr_node.io_Device;

#pragma msg 88 ignore push
							GetSysTime((struct timeval *)tv);
#pragma msg 88 pop
							Amiga2Date(tv->tv_secs, &clockdata);

							CloseDevice((struct IORequest *)tr);
						}
						else
							error = err_UNABLETOGETDATE;

						FreeVec(tv);
					}
					else
						error = err_UNABLETOGETDATE;

					FreeVec(tr);
				}
				else
					error = err_UNABLETOGETDATE;

				if(error)
				{
					WriteLogEntry("GenerateReturnReceipt", LOGENTRY_Text, "Unable to get date.", TAG_END);
					goto out;
				}

				if(folder = GetSpecialFolder(FTF_OUTGOING))
				{
					BPTR dir;

					ObtainSemaphore(&folder->WriteLock);
					if(dir = CurrentDir(folder->DirLock))
					{
						struct AsyncFile *fp;
						UBYTE msgfile[31];

						GetNextMsgNum(folder, NULL, msgfile, sizeof(msgfile));

						if(fp = OpenAsync(msgfile, MODE_WRITE, ASYNCIO_BUF, SysBase, DOSBase))
						{
							struct HdrNode *node;

							sprintf(work, "From: %s <%s>%s", uprefs->RealName, uprefs->Address, eoltypes[gprefs->EOLType]);
							GenerateEncodedHeader(fp, work);
							if(strlen(uprefs->Organization))
							{
								sprintf(work, "Organization: %s%s", uprefs->Organization, eoltypes[gprefs->EOLType]);
								GenerateEncodedHeader(fp, work);
							}

							sprintf(work, "To: %s%s", hnode->Data, eoltypes[gprefs->EOLType]);
							GenerateEncodedHeader(fp, work);

							// date
							{
								STRPTR weekdays[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat",};
								STRPTR months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",};

								sprintf(work, "Date: %s, %ld %s %ld %02.2ld:%02.2ld:%02.2ld",
											weekdays[clockdata.wday],
											(LONG)clockdata.mday, months[clockdata.month - 1], (LONG)clockdata.year,
											(LONG)clockdata.hour, (LONG)clockdata.min, (LONG)clockdata.sec);
								WriteLineAsync(fp, work);
								if(euc.Locale)
								{
									UBYTE work[21];

									sprintf(work, " %+4.4ld", (LONG)((euc.Locale->loc_GMTOffset / 60) * 100));
									WriteLineAsync(fp, work);
								}
								sprintf(work, "%s", eoltypes[gprefs->EOLType]);
								WriteLineAsync(fp, work);
							}

							node = FindHeader("Subject", hdrs, 0);
							sprintf(work, "Subject: Receipt of: %s%s", node ? (STRPTR)node->Data : (STRPTR)"message", eoltypes[gprefs->EOLType]);
							GenerateEncodedHeader(fp, work);

							sprintf(work, "MIME-Version: 1.0%s", eoltypes[gprefs->EOLType]);
							WriteLineAsync(fp, work);
							sprintf(work, "X-Mailer: Eucalyptus " __EUCALYPTUS_VERSION__ " (Release 0." __BATCH_NUMBER__ ") " __AMIGADATE__ "%s", eoltypes[gprefs->EOLType]);
							WriteLineAsync(fp, work);

							sprintf(work, "Message-ID: <%04.4ld%02.2ld%02.2ld%02.2ld%02.2ld%02.2ld.%lX@eucalyptus>%s",
									clockdata.year, clockdata.month, clockdata.mday,
									clockdata.hour, clockdata.min, clockdata.sec,
									FindTask(NULL),
									eoltypes[gprefs->EOLType]);
							WriteLineAsync(fp, work);

							// reference info
							if(node = FindHeader("Message-ID", hdrs, 0))
							{
								sprintf(work, "References: %s%s", node->Data, eoltypes[gprefs->EOLType]);
								WriteLineAsync(fp, work);
							}

							sprintf(work, "Priority: low%s", eoltypes[gprefs->EOLType]);
							WriteLineAsync(fp, work);
							sprintf(work, "Content-Type: text/plain; charset=ISO-8859-1%s", eoltypes[gprefs->EOLType]);
							WriteLineAsync(fp, work);
							WriteLineAsync(fp, eoltypes[gprefs->EOLType]);

							sprintf(work, "This message is a return receipt automatically%s"
									  "generated automatically by Eucalyptus.%s",
									  eoltypes[gprefs->EOLType], eoltypes[gprefs->EOLType]);
							WriteLineAsync(fp, work);

							CloseAsync(fp);

							msg->Index.Flags |= MIEF_RECEIPTED;
							folder->Data.Flags |= (FFF_INDEXMODIFIED);
							UpdateMsgStatus(msg);
						}
						else
						{
							WriteLogEntry("GenerateReturnReceipt", LOGENTRY_Text, "Unable to create receipt.", TAG_END);
							error = err_UNABLETOCREATEFILE;
						}

						CurrentDir(dir);
					}
					else
					{
						WriteLogEntry("GenerateReturnReceipt", LOGENTRY_Text, "Unable to lock directory.", TAG_END);
						error = err_UNABLETOLOCKDIR;
					}
					ReleaseSemaphore(&folder->WriteLock);
				}
				else
				{
					WriteLogEntry("GenerateReturnReceipt", LOGENTRY_Text, "Unable to lock Outgoing folder.", TAG_END);
					error = err_UNABLETOLOCKFOLDER;
				}
			}

			out:
			FreeMsgHeader(hdrs);
		}
		else
		{
			WriteLogEntry("GenerateReturnReceipt", LOGENTRY_Text, "Unable to get message headers.", TAG_END);
			error = err_UNABLETOOPENFILE;
		}

		CloseAsync(mfp);
	}
	else
	{
		WriteLogEntry("GenerateReturnReceipt", LOGENTRY_Text, "Unable to create receipt.", TAG_END);
		error = err_UNABLETOOPENFILE;
	}

	return(error);

#endif
}

gboolean mailmsg_copy(gchar *fname, Folder *folder, gchar *nbuf, glong nbufsize)
{
     glong error = 0;
     gchar *dir;
     gboolean success = TRUE;
     FILE *fp, *new_fp;

     if(!fname ||
        !*fname ||
        !folder)
          return FALSE;

     if(fp = fopen(fname, "r"))
     {
          gchar newfname[FILENAME_MAX], *buf, buf_size = 32;

          if(folder_get_next_msg_num(folder, NULL, newfname, sizeof(newfname)))
          {
	          if(*newfname)
	          {
	          	gchar buf[1024];

	               if(new_fp = fopen(newfname, "w"))
	               {
	                    glong l;

	                    if(nbuf)
	                         strncpy(nbuf, newfname, nbufsize - 1);

	                    while(l = fread(buf, 1, sizeof(buf), fp))
	                         fwrite(buf, l, 1, new_fp);

	                    fclose(new_fp);

	                    folder->Data.Flags |= (FFF_NEWMSGS | FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
     	          }
	               else
	               	success = FALSE;
		     }
		     else
		     	success = FALSE;
	     }
	     else
	     	success = FALSE;

          fclose(fp);
     }
     else
     	success = FALSE;

     return success;
}

MailMsg *mailmsg_get_next_msg(MailMsg *msg, gchar dir)
{
     MailMsg *nmsg = NULL;
     GList *node;

     switch(dir)
     {
          case 1:   // up/prev
          	if(node = g_list_find(msg->Folder->Msgs.List, msg))
          	{
          		for(node = g_list_previous(node); node; node = g_list_previous(node))
          		{
          			nmsg = (MailMsg *)node->data;
          			
	                    if(nmsg->Index.Flags & (MIEF_NEW | MIEF_UNREAD))
     	                    break;
          	     }
          	     
          	     if(!node)
          	     	nmsg = NULL;
          	}
               break;
          case 2:   // down/prev
          default:  // no dir specified
          	if(node = g_list_find(msg->Folder->Msgs.List, msg))
          	{
          		for(node = g_list_next(node); node; node = g_list_next(node))
          		{
          			nmsg = (MailMsg *)node->data;
          			
	                    if(nmsg->Index.Flags & (MIEF_NEW | MIEF_UNREAD))
     	                    break;
          	     }
          	     
          	     if(!node)
          	     	nmsg = NULL;
          	}
               break;
     }

     return(nmsg);
}

gboolean mailmsg_edit_queued(MailMsg *msg)
{
	Composition *comp;
	gboolean success = FALSE;
#if 0
	LONG error = 0;
	UBYTE *text;

	if(comp = (Composition *)AllocVec(sizeof(Composition), MEMF_CLEAR))
	{
		InitComposition(comp);

		comp->Flags |= CF_EDITING;

		if(text = (UBYTE *)AllocVec(WORKLEN, MEMF_CLEAR))
		{
			struct AsyncFile *fp_in, *fp_out;
			BPTR addr_fp, info_fp;

			if(fp_in = OpenAsync(msg->Index.Filename, MODE_READ, ASYNCIO_BUF, SysBase, DOSBase))
			{
				if(GetTempFilename("T:", comp->Filename, sizeof(comp->Filename)))
				{
					if(fp_out = OpenAsync(comp->Filename, MODE_WRITE, ASYNCIO_BUF, SysBase, DOSBase))
					{
						struct List *list = NULL;
						struct HdrNode *hnode = NULL, *cte = NULL;

						/* Suspend sending this message. */
						msg->Index.Flags |= MIEF_SUSPEND;
						UpdateMsgStatus(msg);

						if(list = GetMsgHeader(GMHT_ASYNCIO, (APTR)fp_in,
								TAG_END))
						{
							struct List *ct_list = NULL;
							struct List *alist;
							struct Node *node;
							LONG content_type = CONTENTTYPE_TEXT, encoding = 0;
							UBYTE *boundary = NULL, addr_file[31], info_file[31];

							strncpy(addr_file, msg->Index.Filename, sizeof(addr_file) - 1);
							memset(addr_file, '_', 3);
							addr_file[2] = 'a';
							strncpy(info_file, msg->Index.Filename, sizeof(info_file) - 1);
							memset(info_file, '_', 3);
							info_file[2] = 'i';

							if(hnode = FindHeader("Subject", list, 0))
							{
								strncpy(comp->Subject, hnode->Data, sizeof(comp->Subject) - 1);
							}

							// look for __aXXXXXXXX file, if none, grab addresses from msg
							if(addr_fp = Open(addr_file, MODE_OLDFILE))
							{
								while(FGets(addr_fp, text, WORKLEN - 1))
								{
									LONG addr_type = CAT_TO;
									UBYTE *c, *data;

									if(c = strchr(text, '\r'))
										*c = '\0';
									if(c = strchr(text, '\n'))
										*c = '\0';

									if(c = strchr(text, ' '))
									{
										*c = '\0';
										data = c + 1;
									}

									if(!stricmp(text, "Cc:"))
										addr_type = CAT_CC;
									if(!stricmp(text, "Bcc:"))
										addr_type = CAT_BCC;

									AddCompAddress(comp, data, ACAFT_PARSE, addr_type);
								}

								Close(addr_fp);
							}
							else
							{
								if(hnode = FindHeader("To", list, 0))
								{
									if(alist = ParseAddressees(hnode->Data, PA_FULL))
									{
										for(node = alist->lh_Head;
										    node->ln_Succ;
										    node = node->ln_Succ)
										{
											AddCompAddress(comp, node->ln_Name, ACAFT_PARSE, CAT_TO);
										}
									}
								}

								if(hnode = FindHeader("Cc", list, 0))
								{
									if(alist = ParseAddressees(hnode->Data, PA_FULL))
									{
										for(node = alist->lh_Head;
										    node->ln_Succ;
										    node = node->ln_Succ)
										{
											AddCompAddress(comp, node->ln_Name, ACAFT_PARSE, CAT_TO);
										}
									}
								}
							}

							// look for __iXXXXXXXX file, for attachment info
							if(info_fp = Open(info_file, MODE_OLDFILE))
							{
								CompAttachment *catt;
								UBYTE *c;

								while(FGets(info_fp, text, WORKLEN - 1))
								{
									if(c = strchr(text, '\r'))
										*c = '\0';
									if(c = strchr(text, '\n'))
										*c = '\0';

									// get other info from line

									if(catt = NewAttachment(text))
									{
										SetListBrowserNodeAttrs((struct Node *)catt,
												LBNA_Column,   		0,
													LBNCA_Text,    		catt->Name,
												LBNA_Column,   		1,
													LBNCA_Text,    		catt->ContentType,
												TAG_END);
										AddTail(&comp->Attachments, (struct Node *)catt);
									}
								}

								Close(info_fp);
							}

							// extract msg reference and other semi-pertinent info
							if(hnode = FindHeader("Message-ID", list, 0))
							{
								strncpy(comp->Reference, hnode->Data, sizeof(comp->Reference) - 1);
							}

							if(hnode = FindHeader("Content-Type", list, 0))
							{
								ct_list = GetContentTypeParms(hnode->Data);
								if(-1 == (content_type = VerifyContentType(ct_list)))
									content_type = CONTENTTYPE_APPLICATION;
							}
							else
								content_type = CONTENTTYPE_TEXT;
							if(cte = FindHeader("Content-Transfer-Encoding", list, CHTF_CASECONV))
								encoding = VerifyContentTransferEncoding(cte->Data);

							switch(content_type & CONTENTTYPE_MASK)
							{
								struct List *part_hdr;
								struct HdrNode *p_cte /* , *p_cnt */ ;
								struct Node *node;

								case CONTENTTYPE_MULTIPART:
									if(!ct_list)
										break;
									if(!(node = FindPartialName(ct_list, "boundary=")))
										break;

									boundary = node->ln_Name + 9;
									RemoveWhitespace(boundary, 0);
									RemoveQuotes(boundary);

									// get opening boundary string
									while(FGetsAsync(fp_in, text, WORKLEN - 1))
									{
										if(!strncmp(text, "--", 2) &&
										   !strncmp(text + 2, boundary, strlen(boundary)))
											break;
									}

									// get header out of the way
									if(part_hdr = GetMsgHeader(GMHT_ASYNCIO, (APTR)fp_in, NULL))
									{
										if(p_cte = FindHeader("Content-Transfer-Encoding", part_hdr, CHTF_CASECONV))
											encoding = VerifyContentTransferEncoding(p_cte->Data);

										FreeMsgHeader(part_hdr);
									}
								default:
									while(FGetsAsync(fp_in, text, WORKLEN - 1))
									{
										LONG x;
										BOOL lastequals;

										if(boundary)
											if(!strncmp(text, "--", 2) &&
											   !strncmp(text + 2, boundary, strlen(boundary)))
												break;

										if(encoding == ENCODING_QUOTEDPRINTABLE)
										{
											x = 0;
											lastequals = FALSE;
											while(text[x])
											{
												switch(text[x])
												{
													case '=':
														lastequals = TRUE;
														x++;
														if((text[x] == '\r') ||
														   (text[x] == '\n'))
														{
															if(text[x + 1] == '\n')
															{
																x++;
															}
															lastequals = FALSE;
															break;
														}
													default:
														if(!text[x])
															break;
														if(lastequals)
														{
															LONG val;
															UBYTE hex[3] = {0,};

															hex[0] = text[x++];
															if(text[x])
																hex[1] = text[x];
															hex[2] = 0;
															if(isxdigit(hex[0]) &&
															   isxdigit(hex[1]))
															{
																stch_l(hex, &val);
																if(val)
																	WriteCharAsync(fp_out, val);
															}
														}
														else
															WriteCharAsync(fp_out, text[x]);
														lastequals = FALSE;
														break;
												}

												x++;
											}
										}
										else
										{
											WriteLineAsync(fp_out, text);
										}
									}
									break;
							}

							if(ct_list)
								FreeNodeNamesList(ct_list, TRUE);

							FreeMsgHeader(list);
						}

						CloseAsync(fp_out);
					}
				}

				CloseAsync(fp_in);
			}

			FreeVec(text);
		}

		strncpy(comp->OMsgFile, msg->Index.Filename, sizeof(comp->OMsgFile) - 1);
		AddCompose(comp);
	}
#endif

	return success;
}

void mailmsg_remove(MailMsg *msg, gchar *filename)
{
	gchar *c, *fname;

fprintf(stderr, "mailmsg_remove(%x, %s)\n", msg, filename);

	if(msg)
	{
fprintf(stderr, "Got a msg pointer, get filename\n");
		filename = msg->Index.Filename;
	}
	
	fname = g_strdup(filename);

fprintf(stderr, "unlink '%s'\n", fname);
fprintf(stderr, "(before unlink) list length = %d\n", g_list_length(euc.Folders.List));
	unlink(fname);
fprintf(stderr, "(after unlink) list length = %d\n", g_list_length(euc.Folders.List));
	if(c = strstr(fname, "/msg"))
	{
		memcpy(c + 1, "__a", 3);
fprintf(stderr, "(before unlink) list length = %d\n", g_list_length(euc.Folders.List));
fprintf(stderr, "unlink '%s'\n", fname);
		unlink(fname);
fprintf(stderr, "(after unlink) list length = %d\n", g_list_length(euc.Folders.List));
		memcpy(c + 1, "msg", 3);
	}
	if(c = strstr(fname, "/msg"))
	{
		memcpy(c + 1, "__i", 3);
fprintf(stderr, "(before unlink) list length = %d\n", g_list_length(euc.Folders.List));
fprintf(stderr, "unlink '%s'\n", fname);
		unlink(fname);
fprintf(stderr, "(after unlink) list length = %d\n", g_list_length(euc.Folders.List));
	}

fprintf(stderr, "Free '%s'\n", fname);
	g_free(fname);
fprintf(stderr, "(after free) list length = %d\n", g_list_length(euc.Folders.List));
}
