#ifndef EUCALYPTUS_MSGS_H
#define EUCALYPTUS_MSGS_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Header: msgs.h
 *         Message structures
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include "list.h"
#ifdef USE_GTKHTML
#include <gtkhtml/gtkhtml.h>
#endif

/* Disk-based msg index data.
 */
struct _MailIndexEntry
{
     struct tm            Date;
     gulong               Flags;
     glong                Size,
                          Position;
     gchar                Filename[31],
                          From[101],
                          ReplyTo[101],
                          Subject[101],
                          UIDL[51];
};

// Flags
#define MIEF_NEW              (1L << 0)      /* msg is new */
#define MIEF_UNREAD           (1L << 1)      /* msg hasn't been read yet */
#define MIEF_REPLIED          (1L << 2)      /* msg has been replied */
#define MIEF_DELETED          (1L << 3)      /* msg is marked for deletion */
#define MIEF_FORWARDED        (1L << 4)      /* msg has been forwarded */
#define MIEF_BOUNCED          (1L << 5)      /* msg has been bounced */
#define MIEF_DONTDELETE       (1L << 8)      /* msg is protected */
#define MIEF_NOTIFY           (1L << 9)      /* msg should be followed-up */
#define MIEF_PRIORITY         (1L << 16)     /* msg is important */
#define MIEF_PERSONAL         (1L << 17)     /* msg is directed to user (not ML) */
#define MIEF_RETURNRECEIPTREQ (1L << 18)     /* return receipt requested */
#define MIEF_RECEIPTED        (1L << 19)     /* receipt has been returned */
#define MIEF_SUSPEND          (1L << 20)     /* don't send this msg (Outgoing only) */
#define MIEF_ATTACHMENTS      (1L << 21)     /* msg has attachments */
#define MIEF_OLD              (1L << 22)     /* ??? */
#define MIEF_PURGED           (1L << 23)     /* deleted msg should be purged when possible */
#define MIEF_EDITING          (1L << 24)     /* msg is being edited (Outgoing only) */
#define MIEF_VERIFIED		(1L << 25)	/* msg is definitely in the folder */

struct _MailMsg
{
     GtkWidget		     *Window,
     			     *Layout,
     		          *Menu,
                         *Toolbar,
                         *PaneLayout,
                         *HeaderList,
                         *HeaderScrolledWindow,
                         *PartsList,
                         *PartsScrolledWindow,
                         *PartInfo,
					*DisplayLayout,
                         *Display,
                         *StatusBar;
	GtkObject			*HeaderListAdj,
                         *PartsListAdj;
     Folder	       	*Folder;
	GtkCTreeNode		*TreeNode;
     struct List		*Headers,		// actual headers
                          Parts;		// list of MsgParts
     MailIndexEntry		 Index;
     glong                CurrentPart,
                          PartCount,
                          Flags,
                          ContentFlags;
     gchar                ContentTypeCode,
                          ContentEncoding,
                          TempDir[FILENAME_MAX];
     struct {
	     gchar           Flags[21],		// "NURFBDPIAS"
                          Size[51],		// "x/x.xKB/x.xMB"
                          Status[51],		// ??
                          WindowTitle[301],	// "<Subject> (<From> [<Date>])"
                          Display[201],		// ??
                          Date[51];		// "Fri, 9 Oct 1999, 20:50:37"
     } t;
};

// MailMsg Flags
#define MMF_CLOSEWINDOW            (1L << 0)
#define MMF_GOTSYNC                (1L << 1)
#define MMF_ICONIFY                (1L << 2)
#define MMF_USINGLISTBROWSER       (1L << 3)
#define MMF_NEWSIZE                (1L << 4)
#define MMF_REMOVE                 (1L << 5)
#define MMF_SELECTED               (1L << 6)

// MailMsg ContentFlags
#define MMCF_HASSIGNATURE          (1L << 0)
#define MMCF_ISENCRYPTED           (1L << 1)
#define MMCF_HASKEY                (1L << 2)
#define MMCF_ISUUENCODED           (1L << 3)

struct _HdrNode
{
     gchar			*Tag,
                         *Data;
};

struct _MsgPart
{
     GtkWidget			*Button;
#ifdef USE_GTKHTML
	GtkHTMLStreamHandle	 StreamHandle;
#endif
     struct List         *Headers;
     gchar                Type,
     				 Number;
     gchar               *Filename, // [31],
                         *Name, // [51],
                         *Label, // [51],    // for the speedbar ("\n" prepended)
                         *Boundary; // [81],
};

// MsgPart Types (same as content-types [below])

// flags for header_remove_whitespace()
#define RWSF_NOATOMS          (1 << 0)
#define RWSF_ONLYCRLF         (1 << 1)

// content types
#define CONTENTTYPE_TEXT           (0)
#define CONTENTTYPE_IMAGE          (1)
#define CONTENTTYPE_AUDIO          (2)
#define CONTENTTYPE_VIDEO          (3)
#define CONTENTTYPE_APPLICATION    (4)
#define CONTENTTYPE_MULTIPART      (5)
#define CONTENTTYPE_MESSAGE        (6)

#define CONTENTTYPE_MASK           (0x0F)

// content sub-types
#define CONTENTSUBTYPE_TEXT_PLAIN       (0 << 4)
#define CONTENTSUBTYPE_TEXT_ENRICHED    (1 << 4)
#define CONTENTSUBTYPE_TEXT_HTML        (2 << 4)
#define CONTENTSUBTYPE_IMAGE_JPEG       (0 << 4)
#define CONTENTSUBTYPE_IMAGE_GIF        (1 << 4)
#define CONTENTSUBTYPE_IMAGE_PNG        (2 << 4)
#define CONTENTSUBTYPE_IMAGE_XIFF       (3 << 4)
#define CONTENTSUBTYPE_AUDIO_BASIC      (0 << 4)
#define CONTENTSUBTYPE_AUDIO_X8SVX      (1 << 4)
#define CONTENTSUBTYPE_AUDIO_XWAVE      (2 << 4)
#define CONTENTSUBTYPE_VIDEO_MPEG       (0 << 4)
#define CONTENTSUBTYPE_VIDEO_XAVI       (1 << 4)
#define CONTENTSUBTYPE_VIDEO_XQUICKTIME (2 << 4)
#define CONTENTSUBTYPE_APPLICATION_OCTETSTREAM    (0 << 4)
#define CONTENTSUBTYPE_APPLICATION_PGPSIGNATURE   (1 << 4)
#define CONTENTSUBTYPE_APPLICATION_PGPENCRYPTED   (2 << 4)
#define CONTENTSUBTYPE_APPLICATION_PGPKEYS        (3 << 4)
#define CONTENTSUBTYPE_APPLICATION_POSTSCRIPT     (4 << 4)
#define CONTENTSUBTYPE_MULTIPART_MIXED            (0 << 4)
#define CONTENTSUBTYPE_MULTIPART_OCTETSTREAM      (1 << 4)
#define CONTENTSUBTYPE_MULTIPART_SIGNED           (2 << 4)
#define CONTENTSUBTYPE_MULTIPART_ENCRYPTED        (3 << 4)
#define CONTENTSUBTYPE_MULTIPART_TRANSPORT        (4 << 4)
#define CONTENTSUBTYPE_MULTIPART_ALTERNATIVE      (5 << 4)
#define CONTENTSUBTYPE_MULTIPART_PARALLEL         (6 << 4)
#define CONTENTSUBTYPE_MULTIPART_REPORT           (7 << 4)
#define CONTENTSUBTYPE_MESSAGE_RFC822             (0 << 4)
#define CONTENTSUBTYPE_MESSAGE_EXTERNALBODY       (1 << 4)
#define CONTENTSUBTYPE_MESSAGE_PARTIAL            (2 << 4)
#define CONTENTSUBTYPE_MESSAGE_DELIVERYSTATUS     (3 << 4)

#define CONTENTSUBTYPE_MASK        (0xF0)

// content transfer encodings
#define ENCODING_7BIT              (0)
#define ENCODING_8BIT              (1)
#define ENCODING_BINARY            (2)
#define ENCODING_QUOTEDPRINTABLE   (3)
#define ENCODING_BASE64            (4)

// msgpart_save() flags
#define SMPF_NODECODING            (1L << 0)
#define SMPF_SAVEHEADERS           (1L << 1)
#define SMPF_CONVERTTABS           (1L << 2)
#define SMPF_PRINTERCODES          (1L << 3)
#define SMPF_BRIEFHEADERS          (1L << 4)
#define SMPF_FORDISPLAY            (1L << 5)

// mailmsg_get_header() types
#define GMHT_FILE                  (0)
#define GMHT_SOCKET                (1)

/* Special value returned from msgpart_display()
 * if the part was viewed externally.
 */
#define MSGPART_EXTERNAL			((GtkWidget *)2)

#endif /* EUCALYPTUS_MSGS_H */ 
