/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: pop.c
 *   	 POP client function server
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eucalyptus.h"
#include "md5.h"

struct POPControl pop_control;

void pop_init(void)
{
	memset(&pop_control, 0, sizeof(pop_control));
	
	/* Initialize thread list.
	 */
	list_init(&pop_control.Servers);

#if 0
	/* Clear out thread attribute info.
	 * Attributes for the thread get set during 
	 * thread creation.
	 */
	pthread_attr_destroy(&pop_control.th.Attr);
	 
	/* Clear out and initialize mutex info.
	 */
	pthread_mutex_destroy(&pop_control.th.Mutex);
	pthread_mutex_init(&pop_control.th.Mutex, NULL);
#endif
}

gboolean pop_start_threads(void)
{
	GList *node;
	gboolean success = TRUE;

	/* Go through the copied server list,
	 * looking for terminated threads.
	 */
	node = g_list_first(pop_control.Servers.List);
	while(node)
	{
		Server *server;
		GList *next;
		
		server = (Server *)node->data;
		next = g_list_next(node);
		
		if(!server->Running)
		{
			pop_control.Servers.List = list_remove(&pop_control.Servers, server);
		}
	
		node = next;
	}

	/* Copy the current server list.
	 */
	pthread_mutex_lock(&euc.mutex.Servers);
	for(node = g_list_first(euc.Servers.List); node; node = g_list_next(node))
	{
		Server *s, *server;
		
		s = (Server *)node->data;

		/* Check if the server is already in the list.
		 */		
		if(pop_server_lookup_id(pop_control.Servers.List, s->ID))
			continue;
			
		/* It's not, so make a copy of it.
		 */
		if(server = pop_server_new(s->Type))
		{
			memcpy(server, s, sizeof(Server));
		
			list_append(&pop_control.Servers, server);
		}
	}
	pthread_mutex_unlock(&euc.mutex.Servers);
	
	/* Start threads.
	 */
	g_list_foreach(pop_control.Servers.List, pop_start_server_thread, NULL);

	return success;
}

void pop_start_server_thread(gpointer data, gpointer user_data)
{
	Server *server = (Server *)data;

	/* Check if it's running.
	 */	
	if(server->Running)
		return;

	pthread_attr_destroy(&server->ThreadAttr);
	if(pthread_attr_init(&server->ThreadAttr))
	{
		if(!(euc.Flags & EGF_QUIET))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to initialize POP thread!"),
					euc_ok_button, 0, 
					EUCRF_ASYNC);
	}
	else
	{
		/* Set some thread attributes.
		 */
		pthread_attr_setdetachstate(&server->ThreadAttr, PTHREAD_CREATE_DETACHED);
		
		/* Start the SMTP thread.
		 */
		if(pthread_create(&server->Thread, &server->ThreadAttr, pop_thread, server))
		{	/* Failure :( */
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to start POP thread!"),
					euc_ok_button, 0,
					EUCRF_ASYNC);
		}
	}
}


