/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefsfilters.c
 *         Prefs handling code for filters settings panel
 *
 * Copyright  1997-2000 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "eucalyptus.h"

gchar *common_hdrs[] =
{
     "From",
     "Reply-To",
     "Subject",
     "To",
     "Cc",
     "Origanization",
     "Content-Type",
     "Priority",
     "X-Mailer",
     NULL,
};

gchar *action_types[] =
{
	N_("Move"),
	N_("Copy"),
	N_("Forward"),
	N_("Reply"),
	N_("Delete"),
	N_("Execute"),
	NULL,
};

enum
{
	FILTER_COLUMN_PRI,
	FILTER_COLUMN_TAG,
	FILTER_COLUMN_PATTERN,
	FILTER_COLUMN_TYPE,
	FILTER_COLUMN_ACTION,
	FILTER_COLUMNS
};

struct
{
	GtkWidget		  	*Window,
				  	*Layout,
					*FilterList,
					*ScrolledWindow,
					*New, *Del, *Sort,
					*Priority,
					*Tag,
					*Pattern,
					*Types,
					*Action, *SelectAction,
					*Disabled;
	Filter			*Current;
} filters_control;

gboolean filters_read(void)
{
}

gboolean filters_write(void)
{
	Filter *filter;
	GList *node;
	gboolean success = TRUE;
     FILE *fp;

	if(!*uprefs.Filters)
		return FALSE;

     if(fp = fopen(uprefs.Filters, "w"))
     {
          fprintf(fp, "# Eucalyptus .filters file\n"
                    "# $VER: .filters 1.0a (4.9.97)\n"
                    "#\n"
                    "# File format is as follows:\n"
                    "#   header-tag; pattern; action; action-text; priority; [D]\n"
                    "# where <header-tag> is a header to compare, <pattern> is any\n"
                    "# valid AmigaDOS pattern to match against\n"
                    "# incoming headers; <action> is any one of the following: MOVE,\n"
                    "# COPY, FORWARD, REPLY, DELETE, EXECUTE; <action-text> is a\n"
                    "# folder-name (MOVE, COPY), address (FORWARD), optional text\n"
                    "# file (REPLY), empty (DELETE), or a command (EXECUTE); and \n"
                    "# <priority> is an integer value between -128 and 127.  A \"D\" \n"
                    "# specified at the end means the filter is currently disabled.\n"
                    "#\n");

		for(node = g_list_first(euc.Filters.List); node; node = g_list_next(node))
          {
			Filter *filter = (Filter *)node->data;

               fprintf(fp, "%s; %s; %s; %s; %d; %s\n",
                         filter->Tag,
                         filter->Pattern,
                         action_types[filter->Type],
                         filter->Action,
                         filter->Priority,
                         filter->Disabled ? "D" : "");
          }

          fclose(fp);
     }
	else
		success = FALSE;

     return success;
}

void filters_free(void)
{
	   GList *node;

	   while(node = g_list_first(euc.Filters.List))
	   {
			 Filter *filter = (Filter *)node->data;

			 list_remove(&euc.Filters, filter);
			 filter_free(filter);
	   }
}

Filter *filter_new(void)
{
	   Filter *filter = NULL;

	   if(filter = g_malloc0(sizeof(Filter)))
	   {
	   }

	   return filter;
}

void filter_free(Filter *filter)
{
	   if(!filter)
			 return;

	   g_free(filter);
}

void filters_set_defaults(void)
{
}

gboolean filters_open_window(void)
{
	GtkWidget *vbox, *hbox, *label, *table;
	glong i;
	gchar *titles[] = { N_("Pri"), N_("Tag"), N_("Pattern"), N_("Action"), N_("Action info"), NULL, };
	
	if(filters_control.Window)
	{
		gtk_widget_show_all(filters_control.Window);
		return TRUE;
	}
	
	/* Create the window.
	 */
	filters_control.Window = gnome_app_new(APPNAME, _("Filters"));
	
	/* Set signals.
	 */
	
	/* Create the layout.
	 */
	vbox = filters_control.Layout = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	
	/* Fill it.
	 */
	filters_control.ScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
	{
		_(titles[0]); _(titles[1]); _(titles[2]); _(titles[3]); _(titles[4]); 
		filters_control.FilterList = gtk_clist_new_with_titles(5, titles);
		gtk_signal_connect(GTK_OBJECT(filters_control.FilterList), "select-row",
				GTK_SIGNAL_FUNC(filters_select_row_cb), NULL);

		/* Add to scrolled window.
		 */
		gtk_container_add(GTK_CONTAINER(filters_control.ScrolledWindow), filters_control.FilterList);
		
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(filters_control.ScrolledWindow),
				GTK_POLICY_AUTOMATIC,	/* horizontal */
				GTK_POLICY_AUTOMATIC);	/* vertical */
	}
	gtk_box_pack_start(GTK_BOX(filters_control.Layout), filters_control.ScrolledWindow, TRUE, TRUE, 0);

	hbox = gtk_hbox_new(TRUE, GNOME_PAD_SMALL);
	{
		filters_control.New = gtk_button_new_with_label(_("New"));
		gtk_signal_connect(GTK_OBJECT(filters_control.New), "clicked",
				GTK_SIGNAL_FUNC(filters_new_filter_cb), NULL);
		gtk_box_pack_start(GTK_BOX(hbox), filters_control.New, TRUE, TRUE, 0);

		filters_control.Del = gtk_button_new_with_label(_("Del"));
		gtk_signal_connect(GTK_OBJECT(filters_control.Del), "clicked",
				GTK_SIGNAL_FUNC(filters_del_filter_cb), NULL);
		gtk_widget_set_sensitive(filters_control.Del, FALSE);
		gtk_box_pack_start(GTK_BOX(hbox), filters_control.Del, TRUE, TRUE, 0);

		filters_control.Sort = gtk_button_new_with_label(_("Sort"));
		gtk_signal_connect(GTK_OBJECT(filters_control.Sort), "clicked",
				GTK_SIGNAL_FUNC(filters_sort_cb), NULL);
		gtk_box_pack_start(GTK_BOX(hbox), filters_control.Sort, TRUE, TRUE, 0);
	}
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	
	table = gtk_table_new(3, 2, FALSE);
	{
		guint top = 0;

		hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
		{
			filters_control.Tag = gnome_entry_new("filter-tag");
			for(i=0; common_hdrs[i]; i++)
				gnome_entry_append_history(GNOME_ENTRY(filters_control.Tag), FALSE, common_hdrs[i]);
			gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(filters_control.Tag))), "changed",
					GTK_SIGNAL_FUNC(filters_tag_changed_cb), NULL);
			gtk_box_pack_start(GTK_BOX(hbox), filters_control.Tag, TRUE, TRUE, 0);
			
			filters_control.Disabled = gtk_check_button_new_with_label(_("Disabled?"));
			gtk_signal_connect(GTK_OBJECT(filters_control.Disabled), "toggled",
					GTK_SIGNAL_FUNC(filters_disabled_toggled_cb), NULL);
			gtk_box_pack_start(GTK_BOX(hbox), filters_control.Disabled, FALSE, TRUE, 0);
		}
		prefs_layout_table_item(table, hbox, _("Tag"), &top, 0);
	
		hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
		{
			filters_control.Pattern = gnome_entry_new("filter-pattern");
			gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(filters_control.Pattern))), "changed",
					GTK_SIGNAL_FUNC(filters_pattern_changed_cb), NULL);
			gtk_box_pack_start(GTK_BOX(hbox), filters_control.Pattern, TRUE, TRUE, 0);

			label = gtk_label_new(_("Priority"));
			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);

			filters_control.Priority = gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(0, -128, 127, 1, 1, 1)),
					1.0, 0);
			gtk_signal_connect(GTK_OBJECT(filters_control.Priority), "changed",
					GTK_SIGNAL_FUNC(filters_priority_changed_cb), NULL);
			gtk_box_pack_start(GTK_BOX(hbox), filters_control.Priority, FALSE, TRUE, 0);
		}
		prefs_layout_table_item(table, hbox, _("Pattern"), &top, 0);
		
		hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
		{
			filters_control.Types = gtk_option_menu_new();
			{
				GtkWidget *menu, *item;
				GSList *group = NULL;
				
				menu = gtk_menu_new();

				for(i=0; action_types[i]; i++)
				{
					item = gtk_radio_menu_item_new_with_label(group, _(action_types[i]));
					group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
					gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(filters_type_item_activate_cb), GINT_TO_POINTER(i));
					gtk_menu_append(GTK_MENU(menu), item);
				}

				gtk_option_menu_set_menu(GTK_OPTION_MENU(filters_control.Types), menu);
			}		
			gtk_box_pack_start(GTK_BOX(hbox), filters_control.Types, TRUE, TRUE, 0);

			filters_control.Action = gnome_entry_new("filter-action");
			gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(filters_control.Action))), "changed",
					GTK_SIGNAL_FUNC(filters_action_changed_cb), NULL);
			gtk_box_pack_start(GTK_BOX(hbox), filters_control.Action, TRUE, TRUE, 0);
			
			filters_control.SelectAction = gtk_button_new_with_label(_("Select"));
			gtk_signal_connect(GTK_OBJECT(filters_control.SelectAction), "clicked",
					GTK_SIGNAL_FUNC(filters_select_action_cb), NULL);
			gtk_box_pack_start(GTK_BOX(hbox), filters_control.SelectAction, TRUE, TRUE, 0);
		}
		prefs_layout_table_item(table, hbox, _("Action"), &top, 0);
	}
	gtk_box_pack_start(GTK_BOX(filters_control.Layout), table, FALSE, TRUE, 0);

	/* Set signals.
	 */
	gtk_signal_connect(GTK_OBJECT(filters_control.Window), "destroy",
				 GTK_SIGNAL_FUNC(filters_destroy_cb), NULL);
	
	/* Add layout to window.
	 */
	gnome_app_set_contents(GNOME_APP(filters_control.Window), filters_control.Layout);
	
	/* Populate the list.
	 */
	filters_populate_list();

	/* Open the window.
	 */	
	gtk_widget_show_all(filters_control.Window);
}

void filters_populate_list(void)
{
	GList *node;

	gtk_clist_freeze(GTK_CLIST(filters_control.FilterList));
	gtk_clist_clear(GTK_CLIST(filters_control.FilterList));

	filters_control.Current = NULL;
	filters_set_widget_info(NULL);

	for(node = g_list_first(euc.Filters.List); node; node = g_list_next(node))
	{
		Filter *filter;
		gchar *cols[5];
		
		filter = (Filter *)node->data;
		memset(cols, 0, sizeof(cols));

		g_snprintf(filter->PriText, sizeof(filter->PriText), "%d", filter->Priority);
		cols[0] = filter->PriText;
		cols[1] = filter->Tag;
		cols[2] = filter->Pattern;
		cols[3] = _(action_types[filter->Type]);
		cols[4] = filter->Action;

		filter->Row = gtk_clist_append(GTK_CLIST(filters_control.FilterList), cols);
	}

	gtk_clist_thaw(GTK_CLIST(filters_control.FilterList));
}

void filters_set_widget_info(Filter *filter)
{
	gtk_widget_set_sensitive(filters_control.Del, (gboolean)filter);
	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(filters_control.Tag))), filter ? filter->Tag : "");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(filters_control.Disabled), filter ? filter->Disabled : FALSE);
	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(filters_control.Pattern))), filter ? filter->Pattern : "");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(filters_control.Priority), filter ? (gfloat)filter->Priority : 0);
	gtk_option_menu_set_history(GTK_OPTION_MENU(filters_control.Types), filter ? filter->Type : 0);
	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(filters_control.Action))), filter ? filter->Action : "");
}

void filters_set_row_info(gint row, gint column, const gchar *text)
{
	gtk_clist_set_text(GTK_CLIST(filters_control.FilterList),
			row, column,
			text);
}

/* Callbacks.
 */
void filters_edit_cb(GtkWidget *widget, gpointer data)
{
	filters_open_window();
}

void filters_destroy_cb(GtkWidget *widget, gpointer data)
{
	filters_write();

	filters_control.Window = NULL;
	filters_control.Current = NULL;
}

void filters_select_row_cb(GtkWidget *clist, guint row, guint column, GdkEvent *event, gpointer data)
{
	filters_control.Current = (Filter *)g_list_nth_data(euc.Filters.List, row);

	filters_set_widget_info(filters_control.Current);
}

void filters_new_filter_cb(GtkButton *button, gpointer data)
{
	Filter *filter;
	glong count;

	if(!(filter = filter_new()))
	{
		   euc_request(filters_control.Window,
					    _("Eucalyptus error"),
					    _("Unable to create new filter!"),
					    euc_ok_button, 0,
					    EUCRF_ASYNC);
		   return;
	}

	list_append(&euc.Filters, filter);

	filters_populate_list();

	count = g_list_length(euc.Filters.List);

	gtk_clist_select_row(GTK_CLIST(filters_control.FilterList), count - 1, 0);
}

void filters_del_filter_cb(GtkButton *button, gpointer data)
{
	Filter *filter;

	if(!(filter = filters_control.Current))
		return;

	filters_control.Current = NULL;
	list_remove(&euc.Filters, filter);
	filter_free(filter);

	filters_populate_list();

	filters_set_widget_info(NULL);
}

void filters_sort_cb(GtkWidget *button, gpointer data)
{
	gtk_clist_freeze(GTK_CLIST(filters_control.FilterList));

	pthread_mutex_lock(&euc.mutex.Filters);
	euc.Filters.List = g_list_sort(euc.Filters.List, filters_compare_filters);
	pthread_mutex_unlock(&euc.mutex.Filters);

	filters_populate_list();

	gtk_clist_thaw(GTK_CLIST(filters_control.FilterList));
}

gint filters_compare_filters(gconstpointer f1, gconstpointer f2)
{
	   Filter *filter1, *filter2;

	   filter1 = (Filter *)f1;
	   filter2 = (Filter *)f2;

	   return -(filter1->Priority - filter2->Priority);
}

void filters_tag_changed_cb(GtkWidget *widget, gpointer data)
{
	Filter *filter;
	gchar *text;

	if(!(filter = filters_control.Current))
		return;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(filter->Tag, text, sizeof(filter->Tag) - 1);
	filters_set_row_info(filter->Row, FILTER_COLUMN_TAG, text);
	g_free(text);
}

void filters_disabled_toggled_cb(GtkWidget *widget, gpointer data)
{
	Filter *filter;

	if(!(filter = filters_control.Current))
		return;

	filter->Disabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void filters_pattern_changed_cb(GtkWidget *widget, gpointer data)
{
	Filter *filter;
	gchar *text;

	if(!(filter = filters_control.Current))
		return;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(filter->Pattern, text, sizeof(filter->Pattern) - 1);
	filters_set_row_info(filter->Row, FILTER_COLUMN_PATTERN, text);
	g_free(text);
}

void filters_priority_changed_cb(GtkWidget *widget, gpointer data)
{
	Filter *filter;

	if(!(filter = filters_control.Current))
		return;

	filter->Priority = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));

	g_snprintf(filter->PriText, sizeof(filter->PriText), "%d", filter->Priority);
	filters_set_row_info(filter->Row, FILTER_COLUMN_PRI, filter->PriText);
}

void filters_type_item_activate_cb(GtkWidget *widget, gpointer data)
{
	Filter *filter;
	gint i = GPOINTER_TO_INT(data);

	if(!(filter = filters_control.Current))
		return;

	filter->Type = i;

	filters_set_row_info(filter->Row, FILTER_COLUMN_TYPE, _(action_types[filter->Type]));
}

void filters_action_changed_cb(GtkWidget *widget, gpointer data)
{
	Filter *filter;
	gchar *text;

	if(!(filter = filters_control.Current))
		return;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(filter->Action, text, sizeof(filter->Action) - 1);
	filters_set_row_info(filter->Row, FILTER_COLUMN_ACTION, text);
	g_free(text);
}

void filters_select_action_cb(GtkWidget *widget, gpointer data)
{
}


