/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefs_msgs.c
 *   	 Prefs handling code for Mail settings panel
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "eucalyptus.h"

void prefs_mail_set_defaults(void)
{
}

GtkWidget *prefs_setup_messages(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("General"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			guint top = 0;

			prefs_control.OpenFirstNew = gtk_check_button_new_with_label(_("Automatically open first new/unread message?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.OpenFirstNew), prefs_control.Prefs->OpenFirstNew);
			gtk_signal_connect(GTK_OBJECT(prefs_control.OpenFirstNew), "toggled",
					GTK_SIGNAL_FUNC(prefs_setting_openfirstnew_cb), NULL);
			prefs_layout_table_item(table, prefs_control.OpenFirstNew, NULL, &top, 0);

			prefs_control.DisableReturnReceipt = gtk_check_button_new_with_label(_("Disable return-receipts?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.DisableReturnReceipt), prefs_control.Prefs->DisableReturnReceipt);
			gtk_signal_connect(GTK_OBJECT(prefs_control.DisableReturnReceipt), "toggled",
					GTK_SIGNAL_FUNC(prefs_setting_disablereturnreceipt_cb), NULL);
			prefs_layout_table_item(table, prefs_control.DisableReturnReceipt, NULL, &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Parser"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			guint top = 0;

			prefs_control.TabWidth = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.TabWidth), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.TabWidth), prefs_control.Prefs->TabWidth);
			gtk_signal_connect(GTK_OBJECT(prefs_control.TabWidth), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_tabwidth_cb), NULL);
			prefs_layout_table_item(table, prefs_control.TabWidth, _("Tab width"), &top, 0);

			prefs_control.DisplayFilter = gnome_file_entry_new("prefs:display-filter", _("Select display filter program"));
			gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prefs_control.DisplayFilter))), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_displayfilter_cb), NULL);
			prefs_layout_table_item(table, prefs_control.DisplayFilter, _("Display filter"), &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	return page;
}

GtkWidget *prefs_setup_messages_display(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Reader"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			guint top = 0;

			prefs_control.MsgFont = gnome_font_picker_new();
			gnome_font_picker_set_title(GNOME_FONT_PICKER(prefs_control.MsgFont), _("Select message font"));
			gnome_font_picker_set_mode(GNOME_FONT_PICKER(prefs_control.MsgFont), GNOME_FONT_PICKER_MODE_FONT_INFO);
			gnome_font_picker_fi_set_use_font_in_label(GNOME_FONT_PICKER(prefs_control.MsgFont), TRUE, 10);
			gnome_font_picker_fi_set_show_size(GNOME_FONT_PICKER(prefs_control.MsgFont), TRUE);
			gnome_font_picker_set_font_name(GNOME_FONT_PICKER(prefs_control.MsgFont), prefs_control.Prefs->MsgFont);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MsgFont), "font-set",
					GTK_SIGNAL_FUNC(prefs_setting_msgfont_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MsgFont, _("Message font"), &top, 0);

			prefs_control.HeaderOrder = gtk_entry_new_with_max_length(sizeof(prefs_control.Prefs->HeaderOrder));
			gtk_entry_set_text(GTK_ENTRY(prefs_control.HeaderOrder), prefs_control.Prefs->HeaderOrder);
			gtk_signal_connect(GTK_OBJECT(prefs_control.HeaderOrder), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_headerorder_cb), NULL);
			prefs_layout_table_item(table, prefs_control.HeaderOrder, _("Header order"), &top, 0);

			prefs_control.ShowOnlyHeaders = gtk_check_button_new_with_label(_("Show only these headers?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.ShowOnlyHeaders), prefs_control.Prefs->ShowOnlyHeaders);
			gtk_signal_connect(GTK_OBJECT(prefs_control.ShowOnlyHeaders), "toggled",
					GTK_SIGNAL_FUNC(prefs_setting_showonlyheaders_cb), NULL);
			prefs_layout_table_item(table, prefs_control.ShowOnlyHeaders, NULL, &top, 0);

			prefs_control.HideHeaders = gtk_entry_new_with_max_length(sizeof(prefs_control.Prefs->HideHeaders));
			gtk_entry_set_text(GTK_ENTRY(prefs_control.HideHeaders), prefs_control.Prefs->HideHeaders);
			gtk_signal_connect(GTK_OBJECT(prefs_control.HideHeaders), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_hideheaders_cb), NULL);
			prefs_layout_table_item(table, prefs_control.HideHeaders, _("Hide headers"), &top, 0);

			prefs_control.StripCodes = gtk_check_button_new_with_label(_("Strip codes?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.StripCodes), prefs_control.Prefs->StripCodes);
			gtk_signal_connect(GTK_OBJECT(prefs_control.StripCodes), "toggled",
					GTK_SIGNAL_FUNC(prefs_setting_stripcodes_cb), NULL);
			prefs_layout_table_item(table, prefs_control.StripCodes, NULL, &top, 0);

			prefs_control.WrapText = gtk_check_button_new_with_label(_("Wrap text?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.WrapText), prefs_control.Prefs->WrapText);
			gtk_signal_connect(GTK_OBJECT(prefs_control.WrapText), "toggled",
					GTK_SIGNAL_FUNC(prefs_setting_wraptext_cb), NULL);
			prefs_layout_table_item(table, prefs_control.WrapText, NULL, &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Colors"));
	{
		table = gtk_table_new(1, 4, FALSE);
		{
			   guint top = 0;

			   prefs_control.PersonalPen = gnome_color_picker_new();
			   gnome_color_picker_set_title(GNOME_COLOR_PICKER(prefs_control.PersonalPen),
						    _("Select color for personal messages"));
			   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(prefs_control.PersonalPen),
						    prefs_control.Prefs->PersonalPen.red, 
						    prefs_control.Prefs->PersonalPen.green, 
						    prefs_control.Prefs->PersonalPen.blue, 
						    0);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.PersonalPen), "color-set",
						    GTK_SIGNAL_FUNC(prefs_setting_personalpen_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.PersonalPen, _("Personal messages"), &top, 0);

			   prefs_control.PriorityPen = gnome_color_picker_new();
			   gnome_color_picker_set_title(GNOME_COLOR_PICKER(prefs_control.PriorityPen),
						    _("Select color for priority messages"));
			   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(prefs_control.PriorityPen),
						    prefs_control.Prefs->PriorityPen.red, 
						    prefs_control.Prefs->PriorityPen.green, 
						    prefs_control.Prefs->PriorityPen.blue, 
						    0);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.PriorityPen), "color-set",
						    GTK_SIGNAL_FUNC(prefs_setting_prioritypen_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.PriorityPen, _("Priority messages"), &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Window"));
	{
		table = gtk_table_new(3, 4, FALSE);
		{
			guint top = 0;

			prefs_control.MsgWindowLeft = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MsgWindowLeft), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MsgWindowLeft);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MsgWindowLeft), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_msgwindowleft_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MsgWindowLeft, _("Left"), &top, 0);
			top--;

			prefs_control.MsgWindowTop = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MsgWindowTop), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MsgWindowTop);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MsgWindowTop), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_msgwindowtop_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MsgWindowTop, _("Top"), &top, 2);

			prefs_control.MsgWindowWidth = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MsgWindowWidth), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MsgWindowWidth);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MsgWindowWidth), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_msgwindowwidth_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MsgWindowWidth, _("Width"), &top, 0);
			top--;

			prefs_control.MsgWindowHeight = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MsgWindowHeight), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MsgWindowHeight);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MsgWindowHeight), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_msgwindowheight_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MsgWindowHeight, _("Height"), &top, 2);

			prefs_control.SnapshotMsg = gtk_button_new_with_label(_("Snapshot"));
			gtk_signal_connect(GTK_OBJECT(prefs_control.SnapshotMsg), "clicked",
					GTK_SIGNAL_FUNC(prefs_setting_snapshotmsg_cb), NULL);
			gtk_table_attach(GTK_TABLE(table), prefs_control.SnapshotMsg,
					1, 2, top, top + 1,
					GTK_FILL, GTK_FILL,
					0, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);


	return page;
}

GtkWidget *prefs_setup_messages_security(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Security program"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			guint top = 0, i;
			gchar *progs[] = { "PGP 2.x", "PGP 5+", "GPG 1.x", NULL, };

			/* Security program.
			 * Choices: PGP 2.x, PGP 5.x, GPG 1.x
			 */
			prefs_control.SecurityProgram = gtk_option_menu_new();
			{
				GtkWidget *menu, *item;
				GSList *group = NULL;
				
				menu = gtk_menu_new();

				for(i=0; progs[i]; i++)
				{
					item = gtk_radio_menu_item_new_with_label(group, _(progs[i]));
					group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
					gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(prefs_setting_securityprogram_cb), GINT_TO_POINTER(i));
					gtk_menu_append(GTK_MENU(menu), item);
				}

				gtk_option_menu_set_menu(GTK_OPTION_MENU(prefs_control.SecurityProgram), menu);
			}
			gtk_option_menu_set_history(GTK_OPTION_MENU(prefs_control.SecurityProgram), prefs_control.Prefs->SecurityProgram);
			prefs_layout_table_item(table, prefs_control.SecurityProgram, _("Security program"), &top, 0);

		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	return page;
}

GtkWidget *prefs_setup_messages_sending(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Sending"));
	{
		table = gtk_table_new(3, 4, FALSE);
		{
			GtkWidget *radio = NULL;
			guint top = 0, i;

			/* SMTP server radio item.
			 */
			radio = gtk_radio_button_new_with_label(NULL, _("Server"));
			gtk_signal_connect(GTK_OBJECT(radio), "toggled",
						 GTK_SIGNAL_FUNC(prefs_setting_smtptype_cb), GINT_TO_POINTER(0));
			prefs_layout_table_item(table, radio, NULL, &top, 0);
			top--;

			/* SMTP server and port.
			 */
			prefs_control.SMTPServer = gtk_entry_new_with_max_length(sizeof(prefs_control.Prefs->SMTPServer));
			gtk_entry_set_text(GTK_ENTRY(prefs_control.SMTPServer), prefs_control.Prefs->SMTPServer);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SMTPServer), "changed",
						 GTK_SIGNAL_FUNC(prefs_setting_smtpserver_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SMTPServer, _("Host name"), &top, 2);

			prefs_control.SMTPPort = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.SMTPPort), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.SMTPPort), prefs_control.Prefs->SMTPPort);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SMTPPort), "changed",
						 GTK_SIGNAL_FUNC(prefs_setting_smtpport_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SMTPPort, _("Port"), &top, 2);

			/* SMTP command radio button.
			 */
			radio = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio), _("Command"));
			gtk_signal_connect(GTK_OBJECT(radio), "toggled",
						 GTK_SIGNAL_FUNC(prefs_setting_smtptype_cb), GINT_TO_POINTER(1));
			prefs_layout_table_item(table, radio, NULL, &top, 0);
			top--;

			/* SMTP command.
			 */
			prefs_control.SMTPCommand = gtk_entry_new_with_max_length(sizeof(prefs_control.Prefs->SMTPCommand));
			gtk_entry_set_text(GTK_ENTRY(prefs_control.SMTPCommand), prefs_control.Prefs->SMTPCommand);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SMTPCommand), "changed",
						 GTK_SIGNAL_FUNC(prefs_setting_smtpcommand_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SMTPCommand, _("Command"), &top, 2);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Misc."));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			guint top = 0;

			prefs_control.SendInterval = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.SendInterval), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.SendInterval), prefs_control.Prefs->SendInterval);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SendInterval), "changed",
						 GTK_SIGNAL_FUNC(prefs_setting_sendinterval_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SendInterval, _("Send mail every (mins.)"), &top, 0);

			prefs_control.InitialMailSend = gtk_check_button_new_with_label(_("Send mail on startup?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.InitialMailSend), prefs_control.Prefs->InitialMailSend);
			gtk_signal_connect(GTK_OBJECT(prefs_control.InitialMailSend), "toggled",
						 GTK_SIGNAL_FUNC(prefs_setting_initialmailsend_cb), NULL);
			prefs_layout_table_item(table, prefs_control.InitialMailSend, NULL, &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Window"));
	{
		table = gtk_table_new(3, 4, FALSE);
		{
			guint top = 0;

			prefs_control.SMTPWindowLeft = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.SMTPWindowLeft), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->SMTPWindowLeft);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SMTPWindowLeft), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_smtpwindowleft_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SMTPWindowLeft, _("Left"), &top, 0);
			top--;

			prefs_control.SMTPWindowTop = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.SMTPWindowTop), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->SMTPWindowTop);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SMTPWindowTop), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_smtpwindowtop_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SMTPWindowTop, _("Top"), &top, 2);

			prefs_control.SMTPWindowWidth = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.SMTPWindowWidth), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->SMTPWindowWidth);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SMTPWindowWidth), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_smtpwindowwidth_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SMTPWindowWidth, _("Width"), &top, 0);
			top--;

			prefs_control.SMTPWindowHeight = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.SMTPWindowHeight), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->SMTPWindowHeight);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SMTPWindowHeight), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_smtpwindowheight_cb), NULL);
			prefs_layout_table_item(table, prefs_control.SMTPWindowHeight, _("Height"), &top, 2);

			prefs_control.SnapshotSMTP = gtk_button_new_with_label(_("Snapshot"));
			gtk_signal_connect(GTK_OBJECT(prefs_control.SnapshotSMTP), "clicked",
					GTK_SIGNAL_FUNC(prefs_setting_snapshotsmtp_cb), NULL);
			gtk_table_attach(GTK_TABLE(table), prefs_control.SnapshotSMTP,
					1, 2, top, top + 1,
					GTK_FILL, GTK_FILL,
					0, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);


	return page;
}

GtkWidget *prefs_setup_messages_receiving(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Servers"));
	{
		   table = gtk_table_new(4, 2, FALSE);
		   {
				GtkWidget *scrw;
				glong i;
			   gchar *titles[] = { N_("Host"), N_("Port"), N_("User"), };

			   scrw = gtk_scrolled_window_new(NULL, NULL);
			   {
					 prefs_control.ServerList = gtk_clist_new_with_titles(3, titles);
					 for(i=0; i<3; i++)
						    gtk_clist_set_column_auto_resize(GTK_CLIST(prefs_control.ServerList), i, TRUE);
					 gtk_signal_connect(GTK_OBJECT(prefs_control.ServerList), "select-row",
								  GTK_SIGNAL_FUNC(prefs_setting_selectpopserver_cb), NULL);

					 gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrw),
								  GTK_POLICY_AUTOMATIC,
								  GTK_POLICY_AUTOMATIC);

					 gtk_container_add(GTK_CONTAINER(scrw), prefs_control.ServerList);
			   }
			   gtk_table_attach(GTK_TABLE(table), scrw,
						    0, 1, 0, 4,
						    GTK_FILL | GTK_EXPAND,
						    GTK_FILL | GTK_EXPAND,
						    0, 0);

			   prefs_control.NewPOP = gtk_button_new_with_label(_("New"));
			   gtk_signal_connect(GTK_OBJECT(prefs_control.NewPOP), "clicked",
						    GTK_SIGNAL_FUNC(prefs_setting_newpop_cb), NULL);
			   gtk_table_attach(GTK_TABLE(table), prefs_control.NewPOP,
						    1, 2, 0, 1,
						    GTK_FILL, GTK_FILL,
						    0, 0);

			   prefs_control.EditPOP = gtk_button_new_with_label(_("Edit"));
			   gtk_signal_connect(GTK_OBJECT(prefs_control.EditPOP), "clicked",
						    GTK_SIGNAL_FUNC(prefs_setting_editpop_cb), NULL);
			   gtk_table_attach(GTK_TABLE(table), prefs_control.EditPOP,
						    1, 2, 1, 2,
						    GTK_FILL, GTK_FILL,
						    0, 0);

			   prefs_control.DelPOP = gtk_button_new_with_label(_("Del"));
			   gtk_signal_connect(GTK_OBJECT(prefs_control.DelPOP), "clicked",
						    GTK_SIGNAL_FUNC(prefs_setting_delpop_cb), NULL);
			   gtk_table_attach(GTK_TABLE(table), prefs_control.DelPOP,
						    1, 2, 2, 3,
						    GTK_FILL, GTK_FILL,
						    0, 0);
		   }
		   gtk_container_add(GTK_CONTAINER(frame), table);

		   prefs_populate_server_list();
	   }
	   gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Misc."));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			guint top = 0;

			prefs_control.GetInterval = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.GetInterval), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.GetInterval), prefs_control.Prefs->GetInterval);
			gtk_signal_connect(GTK_OBJECT(prefs_control.GetInterval), "changed",
						 GTK_SIGNAL_FUNC(prefs_setting_getinterval_cb), NULL);
			prefs_layout_table_item(table, prefs_control.GetInterval, _("Get mail every (mins.)"), &top, 0);

			prefs_control.InitialMailGet = gtk_check_button_new_with_label(_("Get mail on startup?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.InitialMailGet), prefs_control.Prefs->InitialMailGet);
			gtk_signal_connect(GTK_OBJECT(prefs_control.InitialMailGet), "toggled",
						 GTK_SIGNAL_FUNC(prefs_setting_initialmailget_cb), NULL);
			prefs_layout_table_item(table, prefs_control.InitialMailGet, NULL, &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Window"));
	{
		table = gtk_table_new(3, 4, FALSE);
		{
			guint top = 0;

			prefs_control.POPWindowLeft = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.POPWindowLeft), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->POPWindowLeft);
			gtk_signal_connect(GTK_OBJECT(prefs_control.POPWindowLeft), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_popwindowleft_cb), NULL);
			prefs_layout_table_item(table, prefs_control.POPWindowLeft, _("Left"), &top, 0);
			top--;

			prefs_control.POPWindowTop = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.POPWindowTop), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->POPWindowTop);
			gtk_signal_connect(GTK_OBJECT(prefs_control.POPWindowTop), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_popwindowtop_cb), NULL);
			prefs_layout_table_item(table, prefs_control.POPWindowTop, _("Top"), &top, 2);

			prefs_control.POPWindowWidth = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.POPWindowWidth), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->POPWindowWidth);
			gtk_signal_connect(GTK_OBJECT(prefs_control.POPWindowWidth), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_popwindowwidth_cb), NULL);
			prefs_layout_table_item(table, prefs_control.POPWindowWidth, _("Width"), &top, 0);
			top--;

			prefs_control.POPWindowHeight = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.POPWindowHeight), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->POPWindowHeight);
			gtk_signal_connect(GTK_OBJECT(prefs_control.POPWindowHeight), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_popwindowheight_cb), NULL);
			prefs_layout_table_item(table, prefs_control.POPWindowHeight, _("Height"), &top, 2);

			prefs_control.SnapshotPOP = gtk_button_new_with_label(_("Snapshot"));
			gtk_signal_connect(GTK_OBJECT(prefs_control.SnapshotPOP), "clicked",
					GTK_SIGNAL_FUNC(prefs_setting_snapshotpop_cb), NULL);
			gtk_table_attach(GTK_TABLE(table), prefs_control.SnapshotPOP,
					1, 2, top, top + 1,
					GTK_FILL, GTK_FILL,
					0, 0);
			
			prefs_control.POPWindowType = gtk_option_menu_new();
			{
				GtkWidget *menu, *item;
				GSList *group = NULL;
				glong i;
				gchar *types[] = { N_("Full"), N_("Summary"), NULL, };
				
				menu = gtk_menu_new();

				for(i=0; types[i]; i++)
				{
					item = gtk_radio_menu_item_new_with_label(group, _(types[i]));
					group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
					gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(prefs_setting_popwindowtype_cb), GINT_TO_POINTER(i));
					gtk_menu_append(GTK_MENU(menu), item);
				}

				gtk_option_menu_set_menu(GTK_OPTION_MENU(prefs_control.POPWindowType), menu);
			}
			gtk_option_menu_set_history(GTK_OPTION_MENU(prefs_control.POPWindowType), prefs_control.Prefs->POPWindowType);
			top++;
			prefs_layout_table_item(table, prefs_control.POPWindowType, _("Type"), &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);


	return page;
}

void prefs_edit_server(Server *server)
{
	GtkWidget *window, *protocol, *host, *username, *password, *port,
			*deletemsgs, *autoretrieve, *noheaders, *ignorefilters,
			*sizelimit, *disabled, *command, *table, *sep, *radio;
	
	if(!server && server->w.Window)
		return;

	server->w.Window = window = gnome_dialog_new(_("Eucalyptus: Edit server"),
			GNOME_STOCK_BUTTON_OK,
			NULL);

	table = gtk_table_new(10, 5, FALSE);
	{
		guint top = 0;

		radio = gtk_radio_button_new(NULL);
		gtk_signal_connect(GTK_OBJECT(radio), "toggled",
				GTK_SIGNAL_FUNC(prefs_server_type_cb), GINT_TO_POINTER(0));
	     gtk_object_set_data(GTK_OBJECT(radio), "server", server);
		gtk_table_attach(GTK_TABLE(table), radio,
					 0, 1, 0, 1,
					 GTK_FILL, GTK_FILL,
					 0, 0);

		/* Type option menu.
		 */
			protocol = gtk_option_menu_new();
			{
				GtkWidget *menu, *item;
				GSList *group = NULL;
				glong i;
				gchar *protocols[] = { "POP3", "IMAP4", NULL, };
				
				menu = gtk_menu_new();

				for(i=0; protocols[i]; i++)
				{
					item = gtk_radio_menu_item_new_with_label(group, _(protocols[i]));
					group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
					gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(prefs_server_protocol_cb), GINT_TO_POINTER(i));
					gtk_object_set_data(GTK_OBJECT(item), "server", server);

					gtk_menu_append(GTK_MENU(menu), item);
				}

				gtk_option_menu_set_menu(GTK_OPTION_MENU(protocol), menu);
			}		
			gtk_option_menu_set_history(GTK_OPTION_MENU(protocol), server->Protocol);
			prefs_layout_table_item(table, protocol, _("Protocol"), &top, 1);

		/* Host.
		 */
		host = gtk_entry_new_with_max_length(sizeof(server->Host));
		gtk_entry_set_text(GTK_ENTRY(host), server->Host);
		gtk_signal_connect(GTK_OBJECT(host), "changed",
				GTK_SIGNAL_FUNC(prefs_server_host_cb), server);
		prefs_layout_table_item(table, host, _("Host"), &top, 1);

		username = gtk_entry_new_with_max_length(sizeof(server->Username));
		gtk_entry_set_text(GTK_ENTRY(username), server->Username);
		gtk_signal_connect(GTK_OBJECT(username), "changed",
				GTK_SIGNAL_FUNC(prefs_server_username_cb), server);
		prefs_layout_table_item(table, username, _("Username"), &top, 1);

		password = gtk_entry_new_with_max_length(sizeof(server->Password));
		gtk_entry_set_visibility(GTK_ENTRY(password), FALSE);
		gtk_entry_set_text(GTK_ENTRY(password), server->Password);
		gtk_signal_connect(GTK_OBJECT(password), "changed",
				GTK_SIGNAL_FUNC(prefs_server_password_cb), server);
		prefs_layout_table_item(table, password, _("Password"), &top, 1);

		sep = gtk_hseparator_new();
		gtk_table_attach(GTK_TABLE(table), sep,
				1, 3, top, top + 1,
				GTK_FILL, GTK_FILL,
				0, 0);
		top++;

		server->adj.SizeLimit = gtk_adjustment_new(server->SizeLimit, 0, 65535, 1, 10, 10);
		sizelimit = gtk_spin_button_new(GTK_ADJUSTMENT(server->adj.SizeLimit), 1, 0);
		gtk_signal_connect(GTK_OBJECT(sizelimit), "changed",
				GTK_SIGNAL_FUNC(prefs_server_sizelimit_cb), server);
		prefs_layout_table_item(table, sizelimit, _("Size limit (KB)"), &top, 1);

		/* Next column.
		 */
		top = 1;

		server->adj.Port = gtk_adjustment_new(server->Port, 0, 65535, 1, 10, 10);
		port = gtk_spin_button_new(GTK_ADJUSTMENT(server->adj.Port), 1, 0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(port), server->Port);
		gtk_signal_connect(GTK_OBJECT(port), "changed",
				GTK_SIGNAL_FUNC(prefs_server_port_cb), server);
		prefs_layout_table_item(table, port, _("Port"), &top, 3);

		deletemsgs = gtk_check_button_new_with_label(_("Delete messages?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(deletemsgs), server->DeleteOnServer);
		gtk_signal_connect(GTK_OBJECT(deletemsgs), "toggled",
				GTK_SIGNAL_FUNC(prefs_server_deletemsgs_cb), server);
		prefs_layout_table_item(table, deletemsgs, NULL, &top, 3);

		autoretrieve = gtk_check_button_new_with_label(_("Auto-retrieve?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autoretrieve), server->DeleteOnServer);
		gtk_signal_connect(GTK_OBJECT(autoretrieve), "toggled",
				GTK_SIGNAL_FUNC(prefs_server_autoretrieve_cb), server);
		prefs_layout_table_item(table, autoretrieve, NULL, &top, 3);

		noheaders = gtk_check_button_new_with_label(_("Don't retrieve headers?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(noheaders), server->DontRetrieveHeaders);
		gtk_signal_connect(GTK_OBJECT(noheaders), "toggled",
				GTK_SIGNAL_FUNC(prefs_server_noheaders_cb), server);
		prefs_layout_table_item(table, noheaders, NULL, &top, 3);

		ignorefilters = gtk_check_button_new_with_label(_("Ignore filters?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ignorefilters), server->IgnoreFilters);
		gtk_signal_connect(GTK_OBJECT(ignorefilters), "toggled",
				GTK_SIGNAL_FUNC(prefs_server_ignorefilters_cb), server);
		prefs_layout_table_item(table, ignorefilters, NULL, &top, 3);

		/* Large separator.
		 */
		sep = gtk_hseparator_new();
		gtk_table_attach(GTK_TABLE(table), sep,
				0, 5, top, top + 1,
				GTK_FILL, GTK_FILL,
				0, 0);
		top++;

		radio = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(radio));
		gtk_signal_connect(GTK_OBJECT(radio), "toggled",
				GTK_SIGNAL_FUNC(prefs_server_type_cb), GINT_TO_POINTER(1));
	     gtk_object_set_data(GTK_OBJECT(radio), "server", server);
		gtk_table_attach(GTK_TABLE(table), radio,
					 0, 1, top, top + 1,
					 GTK_FILL, GTK_FILL,
					 0, 0);

		command = gtk_entry_new_with_max_length(sizeof(server->Command));
		gtk_entry_set_text(GTK_ENTRY(command), server->Command);
		gtk_signal_connect(GTK_OBJECT(command), "changed",
				GTK_SIGNAL_FUNC(prefs_server_command_cb), server);
		prefs_layout_table_item(table, command, _("Command"), &top, 1);

		sep = gtk_hseparator_new();
		gtk_table_attach(GTK_TABLE(table), sep,
				0, 5, top, top + 1,
				GTK_FILL, GTK_FILL,
				0, 0);
		top++;

		disabled = gtk_check_button_new_with_label(_("Disabled?"));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disabled), server->Disabled);
		gtk_signal_connect(GTK_OBJECT(disabled), "toggled",
				GTK_SIGNAL_FUNC(prefs_server_disabled_cb), server);
		prefs_layout_table_item(table, disabled, NULL, &top, 0);
	}
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(window)->vbox), table, TRUE, TRUE, 0);

	gnome_dialog_set_default(GNOME_DIALOG(window), 0);

	gnome_dialog_set_close(GNOME_DIALOG(window), TRUE);

	gnome_dialog_button_connect_object(GNOME_DIALOG(window), 0,
			GTK_SIGNAL_FUNC(prefs_server_close_cb), (gpointer)window);
	gnome_dialog_button_connect_object(GNOME_DIALOG(window), 0,
			GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)window);
	
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
			GTK_SIGNAL_FUNC(prefs_server_close_cb), server);

	gtk_widget_show_all(window);
}

void prefs_populate_server_list(void)
{
	   GList *node;

	   gtk_clist_freeze(GTK_CLIST(prefs_control.ServerList));
	   gtk_clist_clear(GTK_CLIST(prefs_control.ServerList));

	   for(node = g_list_first(prefs_control.Servers.List); node; node = g_list_next(node))
	   {
			 Server *server;
			 gchar *cols[3];

			 server = (Server *)node->data;

			 g_snprintf(server->t.PortText, sizeof(server->t.PortText), "%d", server->Port);
			 cols[0] = server->Host;
			 cols[1] = server->t.PortText;
			 cols[2] = server->Username;

			 gtk_clist_append(GTK_CLIST(prefs_control.ServerList), cols);
	   }

	   gtk_clist_thaw(GTK_CLIST(prefs_control.ServerList));
}

/* Callbacks.
 */
void prefs_setting_openfirstnew_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->OpenFirstNew = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_disablereturnreceipt_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->DisableReturnReceipt = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_msgfont_cb(GtkWidget *widget, gpointer data)
{
	gchar *text;

	text = gnome_font_picker_get_font_name(GNOME_FONT_PICKER(widget));
	strncpy(prefs_control.Prefs->MsgFont, text, sizeof(prefs_control.Prefs->MsgFont) - 1);
}

void prefs_setting_headerorder_cb(GtkWidget *widget, gpointer data)
{
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(prefs_control.Prefs->HeaderOrder, text, sizeof(prefs_control.Prefs->HeaderOrder) - 1);
	g_free(text);
}

void prefs_setting_hideheaders_cb(GtkWidget *widget, gpointer data)
{
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(prefs_control.Prefs->HideHeaders, text, sizeof(prefs_control.Prefs->HideHeaders) - 1);
	g_free(text);
}

void prefs_setting_showonlyheaders_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->ShowOnlyHeaders = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_stripcodes_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->StripCodes = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_wraptext_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->WrapText = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_tabwidth_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->TabWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_displayfilter_cb(GtkWidget *widget, gpointer data)
{
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(prefs_control.Prefs->DisplayFilter, text, sizeof(prefs_control.Prefs->DisplayFilter) - 1);
	g_free(text);
}

void prefs_setting_msgwindowleft_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MsgWindowLeft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_msgwindowtop_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MsgWindowTop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_msgwindowwidth_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MsgWindowWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_msgwindowheight_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MsgWindowHeight = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_snapshotmsg_cb(GtkWidget *widget, gpointer data)
{
	   GList *node, *fnode;
	   GdkWindow *window = NULL;

	   for(fnode = g_list_first(euc.Folders.List); fnode; fnode = g_list_next(fnode))
	   {
			 Folder *folder;

			 folder = (Folder *)fnode->data;

		   for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
		   {
			   MailMsg *msg;

			   msg = (MailMsg *)node->data;
			   if(msg->Window)
			   {
				   window = msg->Window->window;
				   break;
			   }
		   }

		   if(window)
		   {
				 gint x, y, w, h;

			   gdk_window_get_position(window, &x, &y);
			   gdk_window_get_size(window, &w, &h);
		   
			   prefs_control.Prefs->MsgWindowLeft = x;
			   prefs_control.Prefs->MsgWindowTop = y;
			   prefs_control.Prefs->MsgWindowWidth = w;
			   prefs_control.Prefs->MsgWindowHeight = h;

			   gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MsgWindowLeft), x);
			   gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MsgWindowTop), y);
			   gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MsgWindowWidth), w);
			   gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MsgWindowHeight), h);
			   break;
		   }
	   }
}

void prefs_setting_smtpwindowleft_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->SMTPWindowLeft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_smtpwindowtop_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->SMTPWindowTop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_smtpwindowwidth_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->SMTPWindowWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_smtpwindowheight_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->SMTPWindowHeight = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_snapshotsmtp_cb(GtkWidget *widget, gpointer data)
{
}

void prefs_setting_smtpwindowtype_cb(GtkWidget *widget, gpointer data)
{
}

void prefs_setting_popwindowleft_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->POPWindowLeft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_popwindowtop_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->POPWindowTop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_popwindowwidth_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->POPWindowWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_popwindowheight_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->POPWindowHeight = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_snapshotpop_cb(GtkWidget *widget, gpointer data)
{
}

void prefs_setting_initialmailsend_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->InitialMailSend = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_smtpserver_cb(GtkWidget *widget, gpointer data)
{
	   gchar *text;

	   text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	   strncpy(prefs_control.Prefs->SMTPServer, text, sizeof(prefs_control.Prefs->SMTPServer) - 1);
	   g_free(text);
}

void prefs_setting_smtpport_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->SMTPPort = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_smtpcommand_cb(GtkWidget *widget, gpointer data)
{
	   gchar *text;

	   text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	   strncpy(prefs_control.Prefs->SMTPCommand, text, sizeof(prefs_control.Prefs->SMTPCommand) - 1);
	   g_free(text);
}

void prefs_setting_smtptype_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->SMTPType = GPOINTER_TO_INT(data);
}

void prefs_setting_securityprogram_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->SecurityProgram = GPOINTER_TO_INT(data);
}

void prefs_setting_sendinterval_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->SendInterval = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_getinterval_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->GetInterval = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_initialmailget_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->InitialMailGet = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_selectpopserver_cb(GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer data)
{
	   prefs_control.CurrentServer = g_list_nth_data(prefs_control.Servers.List, row);

	   gtk_widget_set_sensitive(prefs_control.EditPOP, (gboolean)prefs_control.CurrentServer);
	   gtk_widget_set_sensitive(prefs_control.DelPOP, (gboolean)prefs_control.CurrentServer);
}

void prefs_setting_newpop_cb(GtkWidget *widget, gpointer data)
{
	   Server *server;

	   if(server = pop_server_new(PST_POP3))
	   {
			 glong count;

			 prefs_control.CurrentServer = server;

			 list_append(&prefs_control.Servers, server);
			 prefs_populate_server_list();

			 count = g_list_length(prefs_control.Servers.List);

			 gtk_clist_select_row(GTK_CLIST(prefs_control.ServerList), count - 1, 0);

			 prefs_edit_server(server);
	   }
}

void prefs_setting_editpop_cb(GtkWidget *widget, gpointer data)
{
	prefs_edit_server(prefs_control.CurrentServer);
}

void prefs_setting_delpop_cb(GtkWidget *widget, gpointer data)
{
	   if(!prefs_control.CurrentServer)
			 return;

	   list_remove(&prefs_control.Servers, prefs_control.CurrentServer);
	   pop_server_free(prefs_control.CurrentServer);

	   prefs_populate_server_list();

	   prefs_control.CurrentServer = NULL;

	   gtk_widget_set_sensitive(prefs_control.EditPOP, (gboolean)prefs_control.CurrentServer);
	   gtk_widget_set_sensitive(prefs_control.DelPOP, (gboolean)prefs_control.CurrentServer);
}

void prefs_setting_popwindowtype_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->POPWindowType = GPOINTER_TO_INT(data);
}

void prefs_setting_personalpen_cb(GtkWidget *widget, guint r, guint g, guint b, guint a, gpointer data)
{
	   gushort red, green, blue, alpha;

	   gnome_color_picker_get_i16(GNOME_COLOR_PICKER(prefs_control.PersonalPen),
			   &red, &green, &blue, &alpha);

	   prefs_control.Prefs->PersonalPen.red = red;
	   prefs_control.Prefs->PersonalPen.green = green;
	   prefs_control.Prefs->PersonalPen.blue = blue;

	   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(prefs_control.PersonalPen),
			   red, green, blue, alpha);
}

void prefs_setting_prioritypen_cb(GtkWidget *widget, guint r, guint g, guint b, guint a, gpointer data)
{
	   gushort red, green, blue, alpha;

	   gnome_color_picker_get_i16(GNOME_COLOR_PICKER(prefs_control.PriorityPen),
			   &red, &green, &blue, &alpha);

	   prefs_control.Prefs->PriorityPen.red = red;
	   prefs_control.Prefs->PriorityPen.green = green;
	   prefs_control.Prefs->PriorityPen.blue = blue;

	   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(widget),
			   red, green, blue, alpha);
}

/* Server window callbacks.
 */
void prefs_server_close_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->w.Window = NULL;

	gtk_object_unref(server->adj.SizeLimit);
	server->adj.SizeLimit = NULL;
	gtk_object_unref(server->adj.Port);
	server->adj.Port = NULL;

	prefs_populate_server_list();

	prefs_control.CurrentServer = NULL;
}

void prefs_server_type_cb(GtkWidget *widget, gpointer data)
{
	Server *server;

	server = gtk_object_get_data(GTK_OBJECT(widget), "server");
	server->Type = GPOINTER_TO_INT(data);
}

void prefs_server_protocol_cb(GtkWidget *widget, gpointer data)
{
	Server *server;

	server = gtk_object_get_data(GTK_OBJECT(widget), "server");
	server->Protocol = GPOINTER_TO_INT(data);
}

void prefs_server_host_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(server->Host, text, sizeof(server->Host) - 1);
	g_free(text);
}

void prefs_server_username_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(server->Username, text, sizeof(server->Username) - 1);
	g_free(text);

}

void prefs_server_password_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(server->Password, text, sizeof(server->Password) - 1);
	g_free(text);
}

void prefs_server_sizelimit_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->SizeLimit = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_server_port_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->Port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_server_deletemsgs_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->DeleteOnServer = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_server_autoretrieve_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->AutoRetrieve = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_server_noheaders_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->DontRetrieveHeaders = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_server_ignorefilters_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->IgnoreFilters = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_server_command_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;
	gchar *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(server->Command, text, sizeof(server->Command) - 1);
	g_free(text);
}

void prefs_server_disabled_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->Disabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}


