/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefs_ui.c
 *   	 Prefs layout code
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <string.h>

#include "eucalyptus.h"

enum
{
	PREFS_END,
	PREFS_ITEM,
	PREFS_SUBITEM
};

struct
{
	gchar type;
	gchar *name;
	GtkWidget * (*setup)(void);
} prefs_setup[] =
{
	{ PREFS_ITEM, N_("General"), prefs_setup_general, },
	{ PREFS_SUBITEM, N_("Folders"), prefs_setup_general_folders, },
	{ PREFS_ITEM, N_("Messages"), prefs_setup_messages, },
	{ PREFS_SUBITEM, N_("Display"), prefs_setup_messages_display, },
	{ PREFS_SUBITEM, N_("Security"), prefs_setup_messages_security, },
	{ PREFS_SUBITEM, N_("Sending"), prefs_setup_messages_sending, },
	{ PREFS_SUBITEM, N_("Receiving"), prefs_setup_messages_receiving, },
	{ PREFS_ITEM, N_("Compose"), prefs_setup_compose, },
	{ PREFS_SUBITEM, N_("Reply/Forward"), prefs_setup_compose_replyforward, },
	{ PREFS_SUBITEM, N_("Security"), prefs_setup_compose_security, },
	{ PREFS_SUBITEM, N_("Spelling"), prefs_setup_compose_spelling, },
	{ PREFS_ITEM, N_("Address book"), prefs_setup_addrbook, },
	{ PREFS_END, },
};

gboolean prefs_open_window(void)
{
	GtkCTreeNode *parent = NULL;
	GList *node;
	GtkWidget *hbox;
	glong i;
	gboolean success = TRUE;

	if(prefs_control.Window)
		return TRUE;

	/* Initialize lists.
	 */
	list_init(&prefs_control.Servers);
	list_init(&prefs_control.Dictionaries);

	/* Copy the current settings.
	 */
	prefs_control.Prefs = g_memdup(&prefs, sizeof(struct Prefs));
	prefs_control.Backup = g_memdup(&prefs, sizeof(struct Prefs));

	/* Copy lists.
	 */
	for(node = g_list_first(euc.Servers.List); node; node = g_list_next(node))
	{
		Server *server, *s = (Server *)node->data;

		if(server = pop_server_new(s->Type))
		{
			memcpy(server, s, sizeof(Server));

			list_append(&prefs_control.Servers, server);
		}
	}
	for(node = g_list_first(euc.Dictionaries.List); node; node = g_list_next(node))
	{
		Dictionary *dict, *d = (Dictionary *)node->data;

		if(dict = compose_dictionary_new())
		{
			memcpy(dict, d, sizeof(Dictionary));

			list_append(&prefs_control.Dictionaries, dict);
		}
	}

	/* Set up some adjustments.
	 */
	prefs_control.adj.TabWidth = gtk_adjustment_new(prefs_control.Prefs->TabWidth, 0, 80, 1, 5, 4);
	prefs_control.adj.DeleteSentMail = gtk_adjustment_new(prefs_control.Prefs->DeleteSentMail, 0, 1000, 1, 10, 10);
	prefs_control.adj.SendInterval = gtk_adjustment_new(prefs_control.Prefs->SendInterval, 0, 65535, 1, 5, 5);
	prefs_control.adj.SMTPPort = gtk_adjustment_new(prefs_control.Prefs->SMTPPort, 1, 1024, 1, 10, 10);
	prefs_control.adj.MainWindowLeft = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.MainWindowTop = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.MainWindowWidth = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.MainWindowHeight = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.FolderWindowLeft = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.FolderWindowTop = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.FolderWindowWidth = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.FolderWindowHeight = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.MsgWindowLeft = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.MsgWindowTop = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.MsgWindowWidth = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.MsgWindowHeight = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.ComposeWindowLeft = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.ComposeWindowTop = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.ComposeWindowWidth = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.ComposeWindowHeight = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.AddrWindowLeft = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.AddrWindowTop = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.AddrWindowWidth = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.AddrWindowHeight = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.POPWindowLeft = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.POPWindowTop = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.POPWindowWidth = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.POPWindowHeight = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.SMTPWindowLeft = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.SMTPWindowTop = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.SMTPWindowWidth = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);
	prefs_control.adj.SMTPWindowHeight = gtk_adjustment_new(0, 0, 65535, 1, 10, 10);

	/* Set up main window and list.
	 */ 
	prefs_control.Window = gnome_dialog_new(_("Eucalyptus settings"),
			GNOME_STOCK_BUTTON_OK,
			GNOME_STOCK_BUTTON_APPLY,
			GNOME_STOCK_BUTTON_CANCEL,
			GNOME_STOCK_BUTTON_HELP,
			NULL);

	hbox = gtk_hbox_new(FALSE, 0);
	{
		GtkWidget *sep;

		/* Create list for settings areas.
		 */
		prefs_control.List = gtk_ctree_new(1, 0);
		gtk_clist_set_column_auto_resize(GTK_CLIST(prefs_control.List), 0, TRUE);
		gtk_signal_connect(GTK_OBJECT(prefs_control.List), "select-row",
				GTK_SIGNAL_FUNC(prefs_select_settings_area_cb), NULL);
		gtk_box_pack_start(GTK_BOX(hbox), prefs_control.List, FALSE, TRUE, 0);

		sep = gtk_vseparator_new();
		gtk_box_pack_start(GTK_BOX(hbox), sep, FALSE, TRUE, 0);

		prefs_control.Layout = gtk_notebook_new();
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(prefs_control.Layout), FALSE);
		gtk_notebook_set_show_border(GTK_NOTEBOOK(prefs_control.Layout), FALSE);
		gtk_box_pack_start(GTK_BOX(hbox), prefs_control.Layout, FALSE, TRUE, 0);
	}
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(prefs_control.Window)->vbox), 
			hbox, TRUE, TRUE, 0);

	/* Populate list.
	 */
	for(i=0; prefs_setup[i].type; i++)
	{
		GtkWidget *page;
		GtkCTreeNode *n, *p = NULL;
		gchar *text[2];

		text[0] = prefs_setup[i].name;
		text[1] = NULL;

		if(prefs_setup[i].type == PREFS_SUBITEM)
			p = parent;

		n = gtk_ctree_insert_node(GTK_CTREE(prefs_control.List),
				p, NULL,
				text, 0,
				NULL, NULL, NULL, NULL,
				(gboolean)p, TRUE);

		page = (*prefs_setup[i].setup) ();
		gtk_notebook_append_page(GTK_NOTEBOOK(prefs_control.Layout), page, NULL);
				
		if(prefs_setup[i].type == PREFS_ITEM)
			parent = n;
	}

	gnome_dialog_set_default(GNOME_DIALOG(prefs_control.Window), 2);

	gnome_dialog_set_close(GNOME_DIALOG(prefs_control.Window), TRUE);

	gnome_dialog_button_connect(GNOME_DIALOG(prefs_control.Window), 0, // OK
			GTK_SIGNAL_FUNC(prefs_okay_cb), NULL);
	gnome_dialog_button_connect(GNOME_DIALOG(prefs_control.Window), 1, // Apply
			GTK_SIGNAL_FUNC(prefs_apply_cb), NULL);
	gnome_dialog_button_connect(GNOME_DIALOG(prefs_control.Window), 2, // Cancel
			GTK_SIGNAL_FUNC(prefs_cancel_cb), NULL);
	gnome_dialog_button_connect(GNOME_DIALOG(prefs_control.Window), 3, // Help
			GTK_SIGNAL_FUNC(prefs_help_cb), NULL);

	gnome_dialog_button_connect_object(GNOME_DIALOG(prefs_control.Window), 0,
			GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)prefs_control.Window);
	gnome_dialog_button_connect_object(GNOME_DIALOG(prefs_control.Window), 2,
			GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)prefs_control.Window);
	
	gtk_signal_connect(GTK_OBJECT(prefs_control.Window), "close",
			GTK_SIGNAL_FUNC(prefs_close_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(prefs_control.Window), "destroy",
			GTK_SIGNAL_FUNC(prefs_destroy_cb), NULL);

	gtk_widget_show_all(prefs_control.Window);

	return success;
}

void prefs_layout_table_item(GtkWidget *table, GtkWidget *widget, gchar *label_text, guint *top, guint left)
{
	if(label_text)
	{
		GtkWidget *label;
		
		label = gtk_label_new(label_text);
		gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
		gtk_table_attach(GTK_TABLE(table), label,
				left, left + 1, *top, *top + 1,
				GTK_FILL,
				GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	}

	gtk_table_attach(GTK_TABLE(table), widget,
			left + 1, left + 2, *top, *top + 1,
			GTK_EXPAND | GTK_FILL,
			GTK_FILL,
			GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	
	(*top)++;
}

void prefs_apply_changes(void)
{
	GList *node;

	pthread_mutex_lock(&euc.mutex.Prefs);
	memcpy(&prefs, prefs_control.Prefs, sizeof(struct Prefs));

	/* Clear out old list.
	 */
	pthread_mutex_lock(&euc.mutex.Servers);
	while(node = g_list_first(euc.Servers.List))
	{
		   Server *server = (Server *)node->data;

		   list_remove(&euc.Servers, server);

		   /* FIXME: Make sure server isn't running
		    * a thread.
		    */

		   pop_server_free(server);
	}

	/* Copy prefs to running.
	 */
	for(node = g_list_first(prefs_control.Servers.List); node; node = g_list_next(node))
	{
		Server *server, *s;
		
		s = (Server *)node->data;
		if(server = pop_server_new(s->Type))
		{
			memcpy(server, s, sizeof(Server));

			list_append(&euc.Servers, server);
		}
	}
	pthread_mutex_unlock(&euc.mutex.Servers);

	/* Clear old list.
	 */
	pthread_mutex_lock(&euc.mutex.Dictionaries);
	while(node = g_list_first(euc.Dictionaries.List))
	{
		   Dictionary *dict = (Dictionary *)node->data;

		   list_remove(&euc.Dictionaries, dict);

		   /* FIXME: Make sure dict isn't connected
		    * to a server.
		    */

		   compose_dictionary_free(dict);
	}

	/* Build new list from config.
	 */
	for(node = g_list_first(prefs_control.Dictionaries.List); node; node = g_list_next(node))
	{
		Dictionary *dict, *d;
		
		d = (Dictionary *)node->data;
		if(dict = compose_dictionary_new())
		{
			memcpy(dict, d, sizeof(Dictionary));

			list_append(&euc.Dictionaries, dict);
		}
	}
	pthread_mutex_unlock(&euc.mutex.Dictionaries);

	pthread_mutex_unlock(&euc.mutex.Prefs);
}

/* Callbacks.
 */
void edit_general_prefs_cb(GtkWidget *widget, gpointer data)
{
	prefs_open_window();
}

void prefs_select_settings_area_cb(GtkCTree *tree, gint row, gint column, GdkEvent *event, gpointer data)
{
	gboolean (*setup)(void);

	gtk_notebook_set_page(GTK_NOTEBOOK(prefs_control.Layout), row);
}

void prefs_destroy_cb(GtkWidget *widget, gpointer data)
{
	GList *node;

	prefs_control.Window = NULL;

	g_free(prefs_control.Prefs);
	prefs_control.Prefs = NULL;
	g_free(prefs_control.Backup);
	prefs_control.Backup = NULL;

	while(node = g_list_first(prefs_control.Servers.List))
	{
		   Server *server = (Server *)node->data;

		   list_remove(&prefs_control.Servers, server);
		   pop_server_free(server);
	}

	while(node = g_list_first(prefs_control.Dictionaries.List))
	{
		   Dictionary *dict = (Dictionary *)node->data;

		   list_remove(&prefs_control.Dictionaries, dict);
		   compose_dictionary_free(dict);
	}

#if 0
	gtk_object_unref(prefs_control.adj.TabWidth);
	gtk_object_unref(prefs_control.adj.DeleteSentMail);
	gtk_object_unref(prefs_control.adj.SendInterval);
	gtk_object_unref(prefs_control.adj.GetInterval);
	gtk_object_unref(prefs_control.adj.SMTPPort);
	gtk_object_unref(prefs_control.adj.MainWindowLeft);
	gtk_object_unref(prefs_control.adj.MainWindowTop);
	gtk_object_unref(prefs_control.adj.MainWindowWidth);
	gtk_object_unref(prefs_control.adj.MainWindowHeight);
	gtk_object_unref(prefs_control.adj.FolderWindowLeft);
	gtk_object_unref(prefs_control.adj.FolderWindowTop);
	gtk_object_unref(prefs_control.adj.FolderWindowWidth);
	gtk_object_unref(prefs_control.adj.FolderWindowHeight);
	gtk_object_unref(prefs_control.adj.MsgWindowLeft);
	gtk_object_unref(prefs_control.adj.MsgWindowTop);
	gtk_object_unref(prefs_control.adj.MsgWindowWidth);
	gtk_object_unref(prefs_control.adj.MsgWindowHeight);
	gtk_object_unref(prefs_control.adj.ComposeWindowLeft);
	gtk_object_unref(prefs_control.adj.ComposeWindowTop);
	gtk_object_unref(prefs_control.adj.ComposeWindowWidth);
	gtk_object_unref(prefs_control.adj.ComposeWindowHeight);
	gtk_object_unref(prefs_control.adj.AddrWindowLeft);
	gtk_object_unref(prefs_control.adj.AddrWindowTop);
	gtk_object_unref(prefs_control.adj.AddrWindowWidth);
	gtk_object_unref(prefs_control.adj.AddrWindowHeight);
	gtk_object_unref(prefs_control.adj.POPWindowLeft);
	gtk_object_unref(prefs_control.adj.POPWindowTop);
	gtk_object_unref(prefs_control.adj.POPWindowWidth);
	gtk_object_unref(prefs_control.adj.POPWindowHeight);
	gtk_object_unref(prefs_control.adj.SMTPWindowLeft);
	gtk_object_unref(prefs_control.adj.SMTPWindowTop);
	gtk_object_unref(prefs_control.adj.SMTPWindowWidth);
	gtk_object_unref(prefs_control.adj.SMTPWindowHeight);
#endif
}

void prefs_close_cb(GtkWidget *widget, gpointer data)
{
}

void prefs_okay_cb(GtkWidget *widget, gpointer data)
{
	prefs_apply_changes();

	prefs_save();
}

void prefs_apply_cb(GtkWidget *widget, gpointer data)
{
	prefs_apply_changes();
}

void prefs_cancel_cb(GtkWidget *widget, gpointer data)
{
	pthread_mutex_lock(&euc.mutex.Prefs);
	memcpy(&prefs, prefs_control.Backup, sizeof(struct Prefs));
	pthread_mutex_unlock(&euc.mutex.Prefs);
}

void prefs_help_cb(GtkWidget *widget, gpointer data)
{
}


