/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefsviewers.c
 *         Prefs handling code for viewers settings panel
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <string.h>

#include "eucalyptus.h"

gchar *viewer_mime_types[] =
{
	"text/*",
	"image/*",
	"audio/*",
	"video/*",
	"application/*",
	"message/*",
	NULL,
};

enum
{
	VIEWERS_COLUMN_TYPE,
	VIEWERS_COLUMN_ACTION,
	VIEWERS_COLUMN_PREPROCESS,
	VIEWERS_COLUMNS
};

struct
{
	GtkWidget		  	*Window,
				  	*Layout,
					*ViewersList,
					*ScrolledWindow,
					*New, *Del,
					*Disabled,
					*Type,
					*Action, *SelectAction,
					*Preprocess, *SelectPreprocess;
	Viewer   			*Current;
} viewers_control;

Viewer *viewer_new(void)
{
	Viewer *vnode;

	if(vnode = g_malloc0(sizeof(Viewer)))
	{
	}

	return vnode;
}

void viewer_free(Viewer *vnode)
{
	if(vnode)
	{


		g_free(vnode);
	}
}

gboolean viewers_read(void)
{
	gchar text[WORKLEN], work[WORKLEN * 4];
	gboolean success = TRUE;
	FILE *fp;

	if(fp = fopen(uprefs.Mailcap, "r"))
	{
		while(fgets(text, sizeof(text) - 1, fp))
		{
			Viewer *vnode;
			glong l;
			gchar *c, *b;

			if(text[0] == '#')
				continue;

			if(c = strchr(text, '\n'))
				*c = '\0';
			if(c = strchr(text, '\r'))
				*c = '\0';

			if(!*text)
				continue;

			if(text[l = (strlen(text) - 1)] == '\\')
			{
				text[l] = '\0';
				strncat(work, text, sizeof(work) - 1);
				continue;
			}

			strncat(work, text, sizeof(work) - 1);
			header_remove_whitespace(work, RWSF_NOATOMS);

			if(vnode = viewer_new())
			{
				gint iter = 0;
				gchar *p, *c;
				
				p = work;
				for(;;)
				{
					if(c = strchr(work, ';'))
						*c = 0;

					while(*p && isspace(*p))
						p++;

					switch(iter)
					{
						case 0:	/* Mime-type */
							strncpy(vnode->Type, p, sizeof(vnode->Type) - 1);
							break;

						case 1:	/* Action */
							strncpy(vnode->Action, p, sizeof(vnode->Action) - 1);
							break;

						default:	/* Anything else */
							if(!strncasecmp(p, "x-preprocess=", 13))
							{
								strncpy(vnode->Preprocess, p + 13, sizeof(vnode->Preprocess) - 1);
								header_remove_quotes(vnode->Preprocess);
								break;
							}

							if(!strcasecmp(p, "x-disabled"))
								vnode->Disabled = TRUE;

							break;
					}

					if(!c)
						break;

					p = c + 1;
				}

				list_append(&euc.Viewers, vnode);
			}

			memset(work, 0, sizeof(work));
		}

		fclose(fp);
	}
	else
		success = FALSE;

	return success;
}

gboolean viewers_save(void)
{
	gboolean success = TRUE;
	FILE *fp;

	if(fp = fopen(uprefs.Mailcap, "w"))
     {
		GList *node;

          fprintf(fp, "# Eucalyptus .mailcap file\n"
                      "# $VER: .mailcap 1.1 (2.1.00)\n"
                      "#\n"
                      "# File format is as follows:\n"
                      "#   mime-type; action; [optional-parameters; ...]\n"
                      "# where <mime-type> is any valid MIME type or wildcard\n"
                      "# pattern, and <action> is the command to execute to\n"
                      "# view that type.\n"
                      "#\n"
				  "# The following optional parameters are understood:\n"
                      "# <x-preprocess>, with the value being any command to\n"
                      "# execute before Eucalyptus displays the appropriate type.\n"
                      "# <x-preprocess> will be valid for internally-viewed\n"
                      "# types as well as external, so that if a type is\n"
                      "# specified without the <action> parameter, the type\n"
                      "# data will be vnodeed through the <x-preprocess>\n"
                      "# command and then displayed using the Eucalyptus\n"
                      "# internal viewer.\n"
				  "# <x-disabled>, which disables the viewer from being\n"
				  "# used, without removing it from the list.\n"
                      "#\n");

		for(node = g_list_first(euc.Viewers.List); node; node = g_list_next(node))
          {
			Viewer *vnode = (Viewer *)node->data;

               fprintf(fp, "%s; %s", vnode->Type, vnode->Action);

               if(*vnode->Preprocess)
               {
                    fprintf(fp, "; x-preprocess=\"%s\"", vnode->Preprocess);
               }

			if(vnode->Disabled)
			{
				fprintf(fp, "; x-disabled");
			}
			
               fprintf(fp, "\n");
          }

          fclose(fp);
     }
	else
		success = FALSE;

	return success;
}

void viewers_set_defaults(void)
{
}

gboolean viewers_open_window(void)
{
	GtkWidget *hbox, *label, *table;
	glong i;
	gchar *titles[] = { N_("Type"), N_("Action"), N_("Preprocess"), NULL, };
	
	if(viewers_control.Window)
	{
		gtk_widget_show_all(viewers_control.Window);
		return TRUE;
	}
	
	/* Create the window.
	 */
	viewers_control.Window = gnome_app_new(APPNAME, _("Viewers"));
	
	/* Set signals.
	 */
	gtk_signal_connect(GTK_OBJECT(viewers_control.Window), "destroy",
			GTK_SIGNAL_FUNC(viewers_destroy_cb), NULL);
	
	/* Create the layout.
	 */
	viewers_control.Layout = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	
	/* Fill it.
	 */
	viewers_control.ScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
	{
		titles[0] = _(titles[0]); 
		titles[1] = _(titles[1]); 
		titles[2] = _(titles[2]);
		viewers_control.ViewersList = gtk_clist_new_with_titles(3, titles);
		gtk_signal_connect(GTK_OBJECT(viewers_control.ViewersList), "select-row",
				GTK_SIGNAL_FUNC(viewers_select_row_cb), NULL);

		/* Add to scrolled window.
		 */
		gtk_container_add(GTK_CONTAINER(viewers_control.ScrolledWindow), viewers_control.ViewersList);
		
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(viewers_control.ScrolledWindow),
				GTK_POLICY_AUTOMATIC,	/* horizontal */
				GTK_POLICY_AUTOMATIC);	/* vertical */
	}
	gtk_box_pack_start(GTK_BOX(viewers_control.Layout), viewers_control.ScrolledWindow, TRUE, TRUE, 0);

	hbox = gtk_hbox_new(TRUE, GNOME_PAD_SMALL);
	{
		viewers_control.New = gtk_button_new_with_label(_("New"));
		gtk_signal_connect(GTK_OBJECT(viewers_control.New), "clicked",
				GTK_SIGNAL_FUNC(viewers_new_cb), NULL);
		gtk_box_pack_start(GTK_BOX(hbox), viewers_control.New, TRUE, TRUE, 0);

		viewers_control.Del = gtk_button_new_with_label(_("Del"));
		gtk_signal_connect(GTK_OBJECT(viewers_control.Del), "clicked",
				GTK_SIGNAL_FUNC(viewers_del_cb), NULL);
		gtk_widget_set_sensitive(viewers_control.Del, FALSE);
		gtk_box_pack_start(GTK_BOX(hbox), viewers_control.Del, TRUE, TRUE, 0);
	}
	gtk_box_pack_start(GTK_BOX(viewers_control.Layout), hbox, FALSE, TRUE, 0);
	
	table = gtk_table_new(3, 2, FALSE);
	{
		guint top = 0;

		viewers_control.Type = gnome_entry_new("viewer-type");
		for(i=0; viewer_mime_types[i]; i++)
			gnome_entry_append_history(GNOME_ENTRY(viewers_control.Type), FALSE, viewer_mime_types[i]);
		gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(viewers_control.Type))), "changed",
				GTK_SIGNAL_FUNC(viewers_type_changed_cb), NULL);
		prefs_layout_table_item(table, viewers_control.Type, _("Type"), &top, 0);

		top--;
		viewers_control.Disabled = gtk_check_button_new_with_label(_("Disabled?"));
		gtk_signal_connect(GTK_OBJECT(viewers_control.Disabled), "toggled",
				GTK_SIGNAL_FUNC(viewers_disabled_toggled_cb), NULL);
		prefs_layout_table_item(table, viewers_control.Disabled, NULL, &top, 1);
		
		viewers_control.Action = gnome_entry_new("viewer-action");
		gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(viewers_control.Action))), "changed",
				GTK_SIGNAL_FUNC(viewers_action_changed_cb), NULL);
		prefs_layout_table_item(table, viewers_control.Action, _("Action"), &top, 0);
			
		viewers_control.SelectAction = gtk_button_new_with_label(_("Select"));
		gtk_signal_connect(GTK_OBJECT(viewers_control.SelectAction), "clicked",
				GTK_SIGNAL_FUNC(viewers_select_action_cb), NULL);
		gtk_table_attach(GTK_TABLE(table), viewers_control.SelectAction,
				2, 3, top - 1, top,
				GTK_FILL, GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
		
		viewers_control.Preprocess = gnome_entry_new("viewer-preprocess");
		gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(viewers_control.Preprocess))), "changed",
				GTK_SIGNAL_FUNC(viewers_preprocess_changed_cb), NULL);
		prefs_layout_table_item(table, viewers_control.Preprocess, _("Preprocess"), &top, 0);
		
		viewers_control.SelectPreprocess = gtk_button_new_with_label(_("Select"));
		gtk_signal_connect(GTK_OBJECT(viewers_control.SelectPreprocess), "clicked",
				GTK_SIGNAL_FUNC(viewers_select_preprocess_cb), NULL);
		gtk_table_attach(GTK_TABLE(table), viewers_control.SelectPreprocess,
				2, 3, top - 1, top,
				GTK_FILL, GTK_FILL,
				GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	}
	gtk_box_pack_start(GTK_BOX(viewers_control.Layout), table, FALSE, TRUE, 0);
	
	/* Add layout to window.
	 */
	gnome_app_set_contents(GNOME_APP(viewers_control.Window), viewers_control.Layout);
	
	/* Populate the list.
	 */
	viewers_populate_list();

	/* Open the window.
	 */	
	gtk_widget_show_all(viewers_control.Window);
}

void viewers_populate_list(void)
{
	GList *node;

	gtk_clist_freeze(GTK_CLIST(viewers_control.ViewersList));
	gtk_clist_clear(GTK_CLIST(viewers_control.ViewersList));

	viewers_control.Current = NULL;
	viewers_set_widget_info(NULL);

	for(node = g_list_first(euc.Viewers.List); node; node = g_list_next(node))
	{
		Viewer *vnode;
		gchar *cols[3];
		
		vnode = (Viewer *)node->data;
		memset(cols, 0, sizeof(cols));

		cols[0] = vnode->Type;
		cols[1] = vnode->Action;
		cols[2] = vnode->Preprocess;

		vnode->Row = gtk_clist_append(GTK_CLIST(viewers_control.ViewersList), cols);
	}

	gtk_clist_thaw(GTK_CLIST(viewers_control.ViewersList));
}

void viewers_free(void)
{
	GList *node;

	while(node = g_list_first(euc.Viewers.List))
	{
		Viewer *vnode = (Viewer *)node->data;

		list_remove(&euc.Viewers, vnode);
		viewer_free(vnode);
	}
}

Viewer *viewer_get_node(gchar *type)
{
	GList *node;
     Viewer *vnode, *all = NULL;

     if(!type)
          type = "text/plain";

	for(node = g_list_first(euc.Viewers.List); node; node = g_list_next(node))
	{
          glong l;
          gchar *c;

		vnode = (Viewer *)node->data;

          if(!strcmp(vnode->Type, "*"))
          {
               all = vnode;
               continue;
          }

          l = 0;
          if(c = strchr(vnode->Type, '*'))
               l = c - vnode->Type;

          if(!l)
               l = strlen(vnode->Type);

          if(!strncasecmp(type, vnode->Type, l))
          {
               all = vnode;
               break;
          }
     }

     return all;
}

void viewers_set_widget_info(Viewer *vnode)
{
	gtk_widget_set_sensitive(viewers_control.Del, (gboolean)vnode);
	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(viewers_control.Type))), vnode ? vnode->Type : "");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(viewers_control.Disabled), vnode ? vnode->Disabled : FALSE);
	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(viewers_control.Action))), vnode ? vnode->Action : "");
	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(viewers_control.Preprocess))), vnode ? vnode->Preprocess : "");
}

void viewers_set_row_info(gint row, gint column, const gchar *text)
{
	gtk_clist_set_text(GTK_CLIST(viewers_control.ViewersList),
			row, column,
			text);
}

/* Callbacks.
 */
void viewers_edit_cb(GtkWidget *widget, gpointer data)
{
	viewers_open_window();
}

void viewers_destroy_cb(GtkWidget *widget, gpointer data)
{
	viewers_save();

	viewers_control.Window = NULL;
	viewers_control.Current = NULL;
}

void viewers_type_changed_cb(GtkWidget *widget, gpointer data)
{
	Viewer *vnode;
	gchar *text;
	
	if(!(vnode = viewers_control.Current))
		return;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(vnode->Type, text, sizeof(vnode->Type) - 1);
	viewers_set_row_info(vnode->Row, VIEWERS_COLUMN_TYPE, text);
	g_free(text);
}

void viewers_action_changed_cb(GtkWidget *widget, gpointer data)
{
	Viewer *vnode;
	gchar *text;
	
	if(!(vnode = viewers_control.Current))
		return;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(vnode->Action, text, sizeof(vnode->Action) - 1);
	viewers_set_row_info(vnode->Row, VIEWERS_COLUMN_ACTION, text);
	g_free(text);
}

void viewers_select_action_cb(GtkButton *button, gpointer data)
{
	Viewer *vnode;
	
	if(!(vnode = viewers_control.Current))
		return;

}

void viewers_preprocess_changed_cb(GtkWidget *widget, gpointer data)
{
	Viewer *vnode;
	gchar *text;
	
	if(!(vnode = viewers_control.Current))
		return;

	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	strncpy(vnode->Preprocess, text, sizeof(vnode->Preprocess) - 1);
	viewers_set_row_info(vnode->Row, VIEWERS_COLUMN_PREPROCESS, text);
	g_free(text);
}

void viewers_select_preprocess_cb(GtkButton *button, gpointer data)
{
	Viewer *vnode;
	
	if(!(vnode = viewers_control.Current))
		return;

}

void viewers_disabled_toggled_cb(GtkWidget *widget, gpointer data)
{
	Viewer *vnode;
	
	if(!(vnode = viewers_control.Current))
		return;

	vnode->Disabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void viewers_new_cb(GtkButton *button, gpointer data)
{
	Viewer *vnode;
	glong count;

	if(!(vnode = viewer_new()))
	{
		euc_request(viewers_control.Window,
				_("Eucalyptus error"),
				_("Unable to create new viewer!"),
				euc_ok_button, 0,
				EUCRF_ASYNC);
		return;
	}

	list_append(&euc.Viewers, vnode);

	viewers_populate_list();

	count = g_list_length(euc.Viewers.List);
	gtk_clist_select_row(GTK_CLIST(viewers_control.ViewersList), count - 1, 0);
}

void viewers_del_cb(GtkButton *button, gpointer data)
{
	Viewer *vnode;
	
	if(!(vnode = viewers_control.Current))
		return;

	viewers_control.Current = NULL;
	list_remove(&euc.Viewers, vnode);
	viewer_free(vnode);

	viewers_populate_list();

	viewers_set_widget_info(NULL);
}

void viewers_select_row_cb(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	viewers_control.Current = (Viewer *)g_list_nth_data(euc.Viewers.List, row);
	viewers_set_widget_info(viewers_control.Current);
}

