#ifndef EUCALYPTUS_PROTOS_H
#define EUCALYPTUS_PROTOS_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Function prototypes
 *
 * Copyright 1997-9 Isengard Developments
 * All rights reserved.
 */

#include <gtk/gtk.h>
#include <gnome.h>
#include <libgnomeui/libgnomeui.h>
#include <libgnomeui/gnome-druid.h>
#include <glib.h>

// main.c

// ops.c
gboolean euc_startup(gboolean);
void euc_main_window(void);
void euc_shutdown(void);
void euc_all_done(glong);
void euc_main_iteration(void);
glong euc_read_folders(void);
void euc_scan_dir_for_folders(gchar *);
glong init_stuff(void);
gint euc_compose_timer(gpointer);
gint euc_pop_timer(gpointer);

gint euc_quit_requested_cb(GtkWidget *, gpointer);

// session.c
gboolean euc_session_start_client(void);

// users.c
void user_get_info(void);

// gui.c
gboolean euc_open_window(void);
void euc_close_window(void);
void euc_update_listview(void);
void euc_populate_main_listview(gpointer, gpointer);
gint euc_compare_folders(gconstpointer, gconstpointer);
void busy_pointer(GtkWidget *, gboolean);

void euc_about_requested_cb(GtkWidget *, gpointer);
void euc_select_folder(GtkCTree *, GList *, gint, gpointer);
gint euc_housekeeper(gpointer);
void euc_refresh_folders_cb(GtkWidget *, gpointer);
void euc_refresh_folders_all_cb(GtkWidget *, gpointer);
void euc_save_folders_cb(GtkWidget *, gpointer);
void euc_new_folder_cb(GtkWidget *, gpointer);
void euc_open_folder_cb(GtkWidget *, gpointer);
void euc_edit_folder_cb(GtkWidget *, gpointer);
void euc_close_folder_cb(GtkWidget *, gpointer);
void euc_delete_folder_cb(GtkWidget *, gpointer);
void euc_execute_macro_cb(GtkWidget *, gpointer);
void euc_open_compose_cb(GtkWidget *, gpointer);
void euc_open_compose_form_cb(GtkWidget *, gpointer);
void euc_open_compose_addr_cb(GtkWidget *, gpointer);
void euc_bug_report_cb(GtkWidget *, gpointer);
void euc_folders_drag_end_cb(GtkWidget *, GdkDragContext *, gpointer);

// folders.c
Folder *folder_new(struct FolderData *);
void folder_free(Folder *, gboolean, gboolean);
gboolean folder_open(Folder *);
void folder_close(Folder *);
glong folder_gather_info(Folder *, gboolean);
gboolean folder_display_contents(Folder *);
void folder_update_visual(Folder *);
gboolean folder_write_data(Folder *, gboolean);
void folder_merge_info(struct FolderData *);
gboolean folder_open_msg(Folder *, MailMsg *);
gboolean folder_verify_directory(Folder *);
gboolean folder_get_index(Folder *);
glong folder_import_msgs(Folder *, const gchar *, gboolean);
glong folder_build_index(Folder *);
gboolean folder_update_index(Folder *);
gboolean folder_update_index_entry(Folder *, MailMsg *);
void folder_set_status(Folder *, gchar *);
Folder *folder_get_special(glong);
gboolean folder_get_next_msg_num(Folder *folder, gchar *, gchar *, glong);
void folder_apply_filters(Folder *);
void folder_set_status(Folder *, gchar *);
gchar *folder_get_full_path(Folder *);
gchar *folder_get_file_path(Folder *, gchar *);
void folder_update_info(Folder *);
MailMsg *folder_process_new_msg(Folder *, gchar *, glong, gchar *);
gboolean folder_verify_uidl(Folder *, gchar *);
Folder *folder_find_by_dir(const gchar *);
MailMsg *folder_find_msg_by_filename(Folder *, const gchar *);
Folder *folder_find_by_name(gchar *);
glong folder_check_selections(Folder *, glong);
void folder_mark_deleted(gpointer, gpointer);

gboolean folders_init_required(void);

gboolean folder_select_open_window(gchar, glong, gpointer);
void folder_select_destroyed(GtkWidget *, gpointer);
void folder_select_select_row(GtkCTree *, GList *, gint, gpointer);
void folder_select_okay(GtkButton *, gpointer);
void folder_select_cancel(GtkButton *, gpointer);

void folder_info(Folder *);
void folder_info_clicked(GnomeDialog *, gint, gpointer);
void folder_info_close(GtkWidget *, gpointer);
void folder_info_select_spool_file(GtkButton *, gpointer);
void folder_info_spool_selected(GtkWidget *, gpointer);
void folder_info_spool_destroy(GtkWidget *, gpointer);
void folder_info_select_parent(GtkButton *, gpointer);
void folder_info_sort_select(GtkMenuItem *, gpointer);
void folder_info_name_changed(GtkEntry *, gpointer);

gint folder_close_requested_cb(GtkWidget *, GdkEventAny *, gpointer);
void folder_layout_destroy_cb(GtkWidget *, gpointer);
void folder_select_msg_cb(GtkCTree *, GList *, gint, gpointer);
void folder_unselect_msg_cb(GtkCTree *, GList *, gint, gpointer);
void folder_resize_column_cb(GtkCList *, gint, gint, gpointer);
void folder_destroy_cb(GtkWidget *, gpointer);
void folder_check_cb(GtkWidget *, gpointer);
void folder_refresh_cb(GtkWidget *, gpointer);
void folder_import_cb(GtkWidget *, gpointer);
void folder_apply_filters_cb(GtkWidget *, gpointer);
void folder_expunge_cb(GtkWidget *, gpointer);
void folder_close_cb(GtkWidget *, gpointer);
void folder_reply_cb(GtkWidget *, gpointer);
void folder_reply_all_cb(GtkWidget *, gpointer);
void folder_reply_notext_cb(GtkWidget *, gpointer);
void folder_reply_all_notext_cb(GtkWidget *, gpointer);
void folder_forward_cb(GtkWidget *, gpointer);
void folder_forward_brief_cb(GtkWidget *, gpointer);
void folder_forward_full_cb(GtkWidget *, gpointer);
void folder_select_cb(GtkWidget *, gpointer);
void folder_select_all_cb(GtkWidget *, gpointer);
void folder_status_read_cb(GtkWidget *, gpointer);
void folder_status_unread_cb(GtkWidget *, gpointer);
void folder_status_important_cb(GtkWidget *, gpointer);
void folder_status_read_all_cb(GtkWidget *, gpointer);
void folder_status_unread_all_cb(GtkWidget *, gpointer);
void folder_status_set_cb(GtkWidget *, gpointer);
void folder_read_cb(GtkWidget *, gpointer);
void folder_bounce_cb(GtkWidget *, gpointer);
void folder_print_cb(GtkWidget *, gpointer);
void folder_delete_cb(GtkWidget *, gpointer);
void folder_move_cb(GtkWidget *, gpointer);
void folder_copy_cb(GtkWidget *, gpointer);
void folder_export_cb(GtkWidget *, gpointer);
void folder_get_addr_cb(GtkWidget *, gpointer);
void folder_resume_cb(GtkWidget *, gpointer);
void folder_send_cb(GtkWidget *, gpointer);
void folder_edit_cb(GtkWidget *, gpointer);
void folder_send_again_cb(GtkWidget *, gpointer);
void folder_send_now_cb(GtkWidget *, gpointer);
void folder_sort_cb(GtkWidget *, gpointer);
void folder_compose_cb(GtkWidget *, gpointer);
static void folder_count_item_cb(gpointer, gpointer);

// msgs.c
MailMsg *mailmsg_new(Folder *);
void mailmsg_free(MailMsg *);
gboolean mailmsg_update_info(MailMsg *, gboolean);
MailMsg *mailmsg_find_by_filename(Folder *, gchar *);
void mailmsg_build_date_string(MailMsg *);
gchar *mailmsg_get_temp_name(MailMsg *);
gchar *mailmsg_get_full_path(MailMsg *);
glong mailmsg_check_content_type(MailMsg *);
void mailmsg_generate_receipt(MailMsg *);
struct List *mailmsg_get_header(glong, gpointer, FILE *);
void mailmsg_free_header(struct List *);
gboolean mailmsg_get_parts(MailMsg *);
void mailmsg_free_contents(MailMsg *);
void mailmsg_free_parts(MailMsg *);
void mailmsg_sort_headers(MailMsg *);
gboolean mailmsg_process_multipart(MailMsg *, const gchar *, FILE *);
gboolean mailmsg_open_window(MailMsg *);
void mailmsg_close_window(MailMsg *);
gboolean mailmsg_populate_header_list(MailMsg *);
gboolean mailmsg_populate_parts_list(MailMsg *);
void mailmsg_update_status(MailMsg *);
void mailmsg_adjust_menu(MailMsg *);
void mailmsg_remove(MailMsg *, gchar *);
gboolean mailmsg_create_reply(MailMsg *, gchar, gchar, Composition *);
gboolean mailmsg_copy(gchar *, Folder *, gchar *nbuf, glong);
MailMsg *mailmsg_get_next_msg(MailMsg *, gchar);
gboolean mailmsg_edit_queued(MailMsg *);

MsgPart *msgpart_new(void);
void msgpart_free(MsgPart *);
gboolean msgpart_display(MailMsg *, MsgPart *);
gboolean msgpart_process(MailMsg *, MsgPart *, guint32);
gboolean msgpart_save(gchar *fname, MailMsg *, MsgPart *, gint32);
glong msgpart_get_image(MsgPart *);

#ifdef USE_GTKHTML
void msgpart_html_title_changed_cb(GtkHTML *, gpointer);
void msgpart_html_url_requested_cb(GtkHTML *, const gchar *, GtkHTMLStreamHandle, gpointer);
void msgpart_html_load_done_cb(GtkHTML *, gpointer);
#endif

gint mailmsg_close_requested_cb(GtkWidget *, GdkEventAny *, gpointer);
void mailmsg_layout_destroy_cb(GtkWidget *, gpointer);
void mailmsg_destroy_cb(GtkWidget *, gpointer);
void mailmsg_close_cb(GtkWidget *, gpointer);
void mailmsg_compose_cb(GtkWidget *, gpointer);
void mailmsg_reply_cb(GtkWidget *, gpointer);
void mailmsg_reply_none_cb(GtkWidget *, gpointer);
void mailmsg_reply_all_cb(GtkWidget *, gpointer);
void mailmsg_reply_all_none_cb(GtkWidget *, gpointer);
void mailmsg_forward_cb(GtkWidget *, gpointer);
void mailmsg_forward_some_cb(GtkWidget *, gpointer);
void mailmsg_forward_all_cb(GtkWidget *, gpointer);
void mailmsg_bounce_cb(GtkWidget *, gpointer);
void mailmsg_print_cb(GtkWidget *, gpointer);
void mailmsg_delete_cb(GtkWidget *, gpointer);
void mailmsg_get_addr_cb(GtkWidget *, gpointer);
void mailmsg_status_read_cb(GtkWidget *, gpointer);
void mailmsg_status_unread_cb(GtkWidget *, gpointer);
void mailmsg_status_important_cb(GtkWidget *, gpointer);
void mailmsg_status_set_cb(GtkWidget *, gpointer);
void mailmsg_check_sig_cb(GtkWidget *, gpointer);
void mailmsg_decrypt_cb(GtkWidget *, gpointer);
void mailmsg_extract_key_cb(GtkWidget *, gpointer);
void mailmsg_save_cb(GtkWidget *, gpointer);
void mailmsg_save_attach_cb(GtkWidget *, gpointer);
void mailmsg_save_raw_cb(GtkWidget *, gpointer);
void mailmsg_move_cb(GtkWidget *, gpointer);
void mailmsg_copy_cb(GtkWidget *, gpointer);
void mailmsg_export_cb(GtkWidget *, gpointer);
void mailmsg_prev_msg_cb(GtkWidget *, gpointer);
void mailmsg_next_msg_cb(GtkWidget *, gpointer);
void mailmsg_uudecode_cb(GtkWidget *, gpointer);
void mailmsg_select_part_cb(GtkCList *, gint, gint, GdkEventButton *, gpointer);

// checks.c
glong mailmsg_check_content_type(MailMsg *);
gboolean mailmsg_is_signed(MailMsg *);
gboolean mailmsg_is_encrypted(MailMsg *);
gboolean mailmsg_is_key_transport(MailMsg *);
gboolean mailmsg_check_signature(MailMsg *);
gboolean mailmsg_decrypt(MailMsg *);
gboolean mailmsg_extract_keys(MailMsg *);

// compose.c
Composition *compose_new(void);
void compose_free(Composition *);
void compose_init(Composition *);
gboolean compose_open_window(Composition *);
void bug_report(void);
gboolean compose_write_msg(Composition *, QueueInfo *);
gboolean compose_write_contents(Composition *, FILE *);
gboolean compose_write_attachments(Composition *, FILE *, gchar *, gchar *, gchar *);
void compose_populate_address_list(Composition *);
void compose_populate_attachment_list(Composition *);
gboolean compose_create_attachment_window(Composition *, CompAttachment *);
void compose_destroy_attachment_window(Composition *);
void compose_populate_custom_headers_list(Composition *);
gboolean compose_get_tagline(Composition *);
QueueInfo *compose_get_queue_info(gchar *);
QueueInfo *queueinfo_new(void);
void queueinfo_free(QueueInfo *);
GList *compose_parse_addresses(Composition *, gchar);
gboolean compose_insert_file_uuencoded(Composition *, gchar *);
gboolean compose_insert_form(Composition *, gchar *);
gboolean compose_insert_file(Composition *, glong, gchar *);
gboolean compose_save_file(Composition *, gchar *);
gboolean compose_insert_file_form(Composition *, glong);
gboolean compose_file_req(Composition *, gchar);
gboolean compose_postpone_msg(Composition *);
void compose_delete_postponed_msg(Composition *);
gboolean compose_resume_postponed_msg(MailMsg *);

CompAddress *compose_address_new(void);
void compose_address_free(CompAddress *);
gboolean compose_address_add(Composition *, gchar *from, gchar, glong);
void compose_address_edit(Composition *, CompAddress *, gchar *, gchar, glong);

CompAttachment *compose_attachment_new(void);
void compose_attachment_free(CompAttachment *);

Dictionary *compose_dictionary_new(void);
void compose_dictionary_free(Dictionary *);

void compose_destroy_cb(GtkWidget *, gpointer);
gint compose_close_requested_cb(GtkWidget *, GdkEventAny *, gpointer);
void compose_send_cb(GtkWidget *, gpointer);
void compose_queue_cb(GtkWidget *, gpointer);
void compose_postpone_cb(GtkWidget *, gpointer);
void compose_cancel_cb(GtkWidget *, gpointer);
void compose_insert_file_cb(GtkWidget *, gpointer);
void compose_insert_file_bracketed_cb(GtkWidget *, gpointer);
void compose_insert_file_indented_cb(GtkWidget *, gpointer);
void compose_insert_form_cb(GtkWidget *, gpointer);
void compose_insert_file_uuencoded_cb(GtkWidget *, gpointer);
void compose_undo_cb(GtkWidget *, gpointer);
void compose_redo_cb(GtkWidget *, gpointer);
void compose_cut_cb(GtkWidget *, gpointer);
void compose_copy_cb(GtkWidget *, gpointer);
void compose_paste_cb(GtkWidget *, gpointer);
void compose_clear_cb(GtkWidget *, gpointer);
void compose_erase_cb(GtkWidget *, gpointer);
void compose_save_cb(GtkWidget *, gpointer);
void compose_plain_cb(GtkWidget *, gpointer);
void compose_bold_cb(GtkWidget *, gpointer);
void compose_italic_cb(GtkWidget *, gpointer);
void compose_underline_cb(GtkWidget *, gpointer);
void compose_select_color_cb(GtkWidget *, gpointer);
void compose_left_align_cb(GtkWidget *, gpointer);
void compose_center_align_cb(GtkWidget *, gpointer);
void compose_right_align_cb(GtkWidget *, gpointer);
void compose_spellcheck_cb(GtkWidget *, gpointer);
void compose_defcheck_cb(GtkWidget *, gpointer);
void compose_match_addr_cb(GtkButton *, gpointer);
void compose_manual_addr_activated_cb(GtkButton *, gpointer);
void compose_manual_addr_changed_cb(GtkWidget *, gpointer);
void compose_select_addr_cb(GtkCList *, gint, gint, GdkEventButton *, gpointer);
void compose_unselect_addr_cb(GtkCList *, gint, gint, GdkEventButton *, gpointer);
void compose_select_att_cb(GtkCList *, gint, gint, GdkEventButton *, gpointer);
void compose_get_new_tagline_cb(GtkWidget *, gpointer);
void compose_select_addr_type_cb(GtkWidget *, gpointer);
void compose_clear_addrs_cb(GtkButton *, gpointer);
void compose_add_att_cb(GtkButton *, gpointer);
void compose_destroy_attachment_window_cb(GtkWidget *, gpointer);
void compose_custom_headers_cb(GtkButton *, gpointer);
void compose_use_sigfile_cb(GtkToggleButton *, gpointer);
void compose_use_tagline_cb(GtkToggleButton *, gpointer);
void compose_new_addr_cb(GtkButton *, gpointer);
void compose_del_addr_cb(GtkButton *, gpointer);
void compose_filesel_okay_cb(GtkButton *, gpointer);
void compose_select_to_cb(GtkButton *, gpointer);
void compose_select_cc_cb(GtkButton *, gpointer);
void compose_select_bcc_cb(GtkButton *, gpointer);
void compose_edit_att_cb(GtkButton *, gpointer);
void compose_unselect_att_cb(GtkCList *, gint, gint, GdkEventButton *, gpointer);
void compose_sign_toggled_cb(GtkToggleButton *, gpointer);
void compose_encrypt_toggled_cb(GtkToggleButton *, gpointer);
void compose_encrypttoself_toggled_cb(GtkToggleButton *, gpointer);
void compose_receipt_toggled_cb(GtkToggleButton *, gpointer);

void compose_attachinfo_attach_cb(GnomeDialog *, gpointer);
void compose_attachinfo_changed_cb(GtkWidget *, gpointer);
void compose_attachinfo_close_cb(GtkWidget *, gpointer);

// spell.c

// parsers.c
void parse_address(gchar *, gchar *, glong, gchar *, glong, gboolean);
time_t parse_msg_date(gchar *, struct tm *);
void parse_convert_tabs(gchar *, glong, gint16);

// uudecode.c
gboolean mailmsg_uudecode(MailMsg *);

// content.c
gchar content_transfer_encoding_verify(gchar *);
glong content_type_verify(const gchar *);
gchar *content_type_find_param(const gchar *, const gchar *, gboolean);

// headers.c
void header_remove_comments(gchar *);
void header_remove_whitespace(gchar *, gint16);
void header_remove_quotes(gchar *);
HdrNode *header_find_tag(gchar *, struct List *, gint16);
void header_decode(gchar *);
gboolean header_encode(FILE *, gchar *);
gboolean header_write_quoted_printable(FILE *, gchar *, glong, glong);

// addressbook.c
gboolean addresses_open_window(void);
void addresses_close_window(void);
gboolean address_request_recipient(gchar *, gchar *, gchar *, glong, gboolean);
gboolean addresses_read(void);
gboolean addresses_read_layout(void);
AddressEntry *address_entry_new(void);
void address_entry_free(AddressEntry *);
gchar *address_get_field(AddressEntry *, gchar *);
AddrEntryItem *address_entry_item_new(void);
void address_entry_item_free(AddrEntryItem *);
AddressEntry *address_select_one_from_many_matches(struct List *, GtkWidget *, gboolean);
void address_smart_get_first_last_name(AddressEntry *, gchar *, glong, gchar *, glong);

gboolean address_select_open_window(Composition *, gchar, gboolean);
void address_select_close_window(Composition *);
gboolean address_select_add(Composition *, gchar);
void address_select_populate_list(Composition *);

void addresses_open_cb(GtkWidget *, gpointer);

gint address_select_quit_requested_cb(GtkWidget *, gpointer);
void address_select_item_cb(GtkCTree *, gint, gint, GdkEventButton *, gpointer);
void address_select_add_to_cb(GtkWidget *, gpointer);
void address_select_add_cc_cb(GtkWidget *, gpointer);
void address_select_add_bcc_cb(GtkWidget *, gpointer);
void address_select_new_item_cb(GtkWidget *, gpointer);
void address_select_new_group_cb(GtkWidget *, gpointer);
void address_select_info_cb(GtkWidget *, gpointer);
void address_select_match_changed_cb(GtkEditable *, gpointer);

// pop.c
void pop_init(void);
void *pop_thread(void *);
gboolean pop_start_threads(void);
void pop_start_server_thread(gpointer, gpointer);
Server *pop_server_new(gchar);
void pop_server_free(Server *);
void pop_server_generate_id(Server *);
Server *pop_server_lookup_id(GList *, gchar *);
gboolean pop_server_connect(Server *);
void pop_server_disconnect(Server *);
gboolean pop_server_get_headers(Server *);
gboolean pop_server_get_messages(Server *);
gboolean pop_server_fetch_msg(Server *, ServerEntry *, glong, gboolean);
glong pop_server_send(Server *, gchar *, gchar *, glong);
void pop_server_reset(Server *);
void pop_server_status(Server *, const gchar *, glong);
ServerEntry *pop_server_entry_new(void);
void pop_server_entry_free(ServerEntry *);
glong pop_read_line(glong, gchar *, glong);

gboolean pop_open_window(Server *);
void pop_close_window(Server *);
void pop_populate_progress_window(Server *);
void pop_update_progress(Server *, ProgressMsg *);

void pop_check_mail_cb(GtkWidget *, gpointer);
void pop_get_new_mail_cb(GtkWidget *, gpointer);
void pop_open_retriever_cb(GtkWidget *, gpointer);
void pop_stop_cb(GtkWidget *, gpointer);

// smtp.c
void smtp_init(void);
void *smtp_thread(void *);
gint smtp_timer(gpointer);
gboolean smtp_thread_start(void);
void smtp_thread_stop(gboolean);
int smtp_message_file_gatherer(const struct dirent *);
struct List *smtp_parse_addressees(gchar *addr, gint16 flags);
void smtp_free_addressees(struct List *);
gboolean smtp_send_mail_file(int, gchar *, struct List *);
glong smtp_send(glong, gchar *, gchar *, glong);
void smtp_close_server(void);
gboolean smtp_send_msg_command(gchar *);
gboolean smtp_send_msg_internal(gchar *);
void smtp_set_status(gchar *, gchar *, glong);
void smtp_set_item_status(QueuedMail *qm, gchar *text, gint16 flags);

gboolean smtp_open_window(void);
void smtp_close_window(void);
void smtp_update_progress(ProgressMsg *);
void smtp_populate_progress_window(void);

void smtp_send_mail_cb(GtkWidget *, gpointer);
void smtp_open_sender_cb(GtkWidget *, gpointer);
void smtp_stop_cb(GtkWidget *, gpointer);

// menus.c & #?menu.c

// misc.c
void euc_temp_filename(gchar *, gchar *, glong);

// utils.c
gboolean check_style_char(gchar *, gint);

// checks.c
gboolean mailmsg_is_signed(MailMsg *);
gboolean mailmsg_is_encrypted(MailMsg *);
gboolean mailmsg_is_key_transport(MailMsg *);

// base64.c
gchar base64_get_value(glong);
gboolean base64_valid(glong);

// prefs.c
gboolean prefs_init(void);
gboolean prefs_load(void);
gboolean prefs_save(void);
gboolean prefs_save_section(gchar);

void edit_general_prefs_cb(GtkWidget *, gpointer);

// prefs_ui.c
gboolean prefs_open_window(void);
GtkWidget *prefs_setup_general(void);
GtkWidget *prefs_setup_general_folders(void);
GtkWidget *prefs_setup_messages(void);
GtkWidget *prefs_setup_messages_display(void);
GtkWidget *prefs_setup_messages_security(void);
GtkWidget *prefs_setup_messages_sending(void);
GtkWidget *prefs_setup_messages_receiving(void);
GtkWidget *prefs_setup_compose(void);
GtkWidget *prefs_setup_compose_replyforward(void);
GtkWidget *prefs_setup_compose_security(void);
GtkWidget *prefs_setup_compose_spelling(void);
GtkWidget *prefs_setup_addrbook(void);
void prefs_layout_table_item(GtkWidget *, GtkWidget *, gchar *, guint *, guint);

void prefs_select_settings_area_cb(GtkCTree *, gint, gint, GdkEvent *, gpointer);
void prefs_destroy_cb(GtkWidget *, gpointer);
void prefs_close_cb(GtkWidget *, gpointer);
void prefs_okay_cb(GtkWidget *, gpointer);
void prefs_apply_cb(GtkWidget *, gpointer);
void prefs_cancel_cb(GtkWidget *, gpointer);
void prefs_help_cb(GtkWidget *, gpointer);

// prefs_addrbook.c
void prefs_setting_addrwindowleft_cb(GtkWidget *, gpointer);
void prefs_setting_addrwindowtop_cb(GtkWidget *, gpointer);
void prefs_setting_addrwindowwidth_cb(GtkWidget *, gpointer);
void prefs_setting_addrwindowheight_cb(GtkWidget *, gpointer);
void prefs_setting_snapshotaddr_cb(GtkWidget *, gpointer);

// prefs_compose.c
void prefs_populate_dict_list(void);
void prefs_edit_dictionary(Dictionary *);

void prefs_setting_composewindowleft_cb(GtkWidget *, gpointer);
void prefs_setting_composewindowtop_cb(GtkWidget *, gpointer);
void prefs_setting_composewindowwidth_cb(GtkWidget *, gpointer);
void prefs_setting_composewindowheight_cb(GtkWidget *, gpointer);
void prefs_setting_snapshotcompose_cb(GtkWidget *, gpointer);
void prefs_setting_usetagline_cb(GtkWidget *, gpointer);
void prefs_setting_usesignature_cb(GtkWidget *, gpointer);
void prefs_setting_usequotedprintable_cb(GtkWidget *, gpointer);
void prefs_setting_eoltype_cb(GtkWidget *, gpointer);
void prefs_setting_quoteprefix_cb(GtkWidget *, gpointer);
void prefs_setting_replyheader_cb(GtkWidget *, gpointer);
void prefs_setting_forwardheader_cb(GtkWidget *, gpointer);
void prefs_setting_maxlinelength_cb(GtkWidget *, gpointer);
void prefs_setting_replydefault_cb(GtkWidget *, gpointer);
void prefs_setting_forwarddefault_cb(GtkWidget *, gpointer);
void prefs_setting_closemsgonreply_cb(GtkWidget *, gpointer);
void prefs_setting_selectdictserver_cb(GtkCList *, gint, gint, GdkEvent *, gpointer);
void prefs_setting_newdict_cb(GtkWidget *, gpointer);
void prefs_setting_editdict_cb(GtkWidget *, gpointer);
void prefs_setting_deldict_cb(GtkWidget *, gpointer);
void prefs_setting_signmsgs_cb(GtkWidget *, gpointer);
void prefs_setting_encryptmsgs_cb(GtkWidget *, gpointer);

// prefs_general.c
void prefs_setting_askbeforequitting_cb(GtkWidget *, gpointer);
void prefs_setting_mainwindowleft_cb(GtkWidget *, gpointer);
void prefs_setting_mainwindowtop_cb(GtkWidget *, gpointer);
void prefs_setting_mainwindowwidth_cb(GtkWidget *, gpointer);
void prefs_setting_mainwindowheight_cb(GtkWidget *, gpointer);
void prefs_setting_snapshotmain_cb(GtkWidget *, gpointer);
void prefs_setting_deletesentmail_cb(GtkWidget *, gpointer);
void prefs_setting_folderwindowleft_cb(GtkWidget *, gpointer);
void prefs_setting_folderwindowtop_cb(GtkWidget *, gpointer);
void prefs_setting_folderwindowwidth_cb(GtkWidget *, gpointer);
void prefs_setting_folderwindowheight_cb(GtkWidget *, gpointer);
void prefs_setting_snapshotfolder_cb(GtkWidget *, gpointer);
void prefs_setting_integrateddisplay_cb(GtkWidget *, gpointer);
void prefs_setting_previewpane_cb(GtkWidget *, gpointer);

// prefs_msgs.c
void prefs_edit_server(Server *);
void prefs_populate_server_list(void);

void prefs_setting_openfirstnew_cb(GtkWidget *, gpointer);
void prefs_setting_disablereturnreceipt_cb(GtkWidget *, gpointer);
void prefs_setting_msgfont_cb(GtkWidget *, gpointer);
void prefs_setting_headerorder_cb(GtkWidget *, gpointer);
void prefs_setting_hideheaders_cb(GtkWidget *, gpointer);
void prefs_setting_showonlyheaders_cb(GtkWidget *, gpointer);
void prefs_setting_stripcodes_cb(GtkWidget *, gpointer);
void prefs_setting_wraptext_cb(GtkWidget *, gpointer);
void prefs_setting_tabwidth_cb(GtkWidget *, gpointer);
void prefs_setting_displayfilter_cb(GtkWidget *, gpointer);
void prefs_setting_msgwindowleft_cb(GtkWidget *, gpointer);
void prefs_setting_msgwindowtop_cb(GtkWidget *, gpointer);
void prefs_setting_msgwindowwidth_cb(GtkWidget *, gpointer);
void prefs_setting_msgwindowheight_cb(GtkWidget *, gpointer);
void prefs_setting_snapshotmsg_cb(GtkWidget *, gpointer);
void prefs_setting_smtpwindowleft_cb(GtkWidget *, gpointer);
void prefs_setting_smtpwindowtop_cb(GtkWidget *, gpointer);
void prefs_setting_smtpwindowwidth_cb(GtkWidget *, gpointer);
void prefs_setting_smtpwindowheight_cb(GtkWidget *, gpointer);
void prefs_setting_snapshotsmtp_cb(GtkWidget *, gpointer);
void prefs_setting_popwindowleft_cb(GtkWidget *, gpointer);
void prefs_setting_popwindowtop_cb(GtkWidget *, gpointer);
void prefs_setting_popwindowwidth_cb(GtkWidget *, gpointer);
void prefs_setting_popwindowheight_cb(GtkWidget *, gpointer);
void prefs_setting_snapshotpop_cb(GtkWidget *, gpointer);
void prefs_setting_initialmailsend_cb(GtkWidget *, gpointer);
void prefs_setting_smtpserver_cb(GtkWidget *, gpointer);
void prefs_setting_smtpport_cb(GtkWidget *, gpointer);
void prefs_setting_smtpcommand_cb(GtkWidget *, gpointer);
void prefs_setting_smtptype_cb(GtkWidget *, gpointer);
void prefs_setting_securityprogram_cb(GtkWidget *, gpointer);
void prefs_setting_sendinterval_cb(GtkWidget *, gpointer);
void prefs_setting_getinterval_cb(GtkWidget *, gpointer);
void prefs_setting_initialmailget_cb(GtkWidget *, gpointer);
void prefs_setting_selectpopserver_cb(GtkCList *, gint, gint, GdkEvent *, gpointer);
void prefs_setting_newpop_cb(GtkWidget *, gpointer);
void prefs_setting_editpop_cb(GtkWidget *, gpointer);
void prefs_setting_delpop_cb(GtkWidget *, gpointer);
void prefs_setting_popwindowtype_cb(GtkWidget *, gpointer);
void prefs_setting_previewpane_cb(GtkWidget *, gpointer);
void prefs_setting_personalpen_cb(GtkWidget *, guint, guint, guint, guint, gpointer);
void prefs_setting_prioritypen_cb(GtkWidget *, guint, guint, guint, guint, gpointer);

void prefs_server_destroy_cb(GtkWidget *, gpointer);
void prefs_server_close_cb(GtkWidget *, gpointer);
void prefs_server_type_cb(GtkWidget *, gpointer);
void prefs_server_protocol_cb(GtkWidget *, gpointer);
void prefs_server_host_cb(GtkWidget *, gpointer);
void prefs_server_username_cb(GtkWidget *, gpointer);
void prefs_server_password_cb(GtkWidget *, gpointer);
void prefs_server_sizelimit_cb(GtkWidget *, gpointer);
void prefs_server_port_cb(GtkWidget *, gpointer);
void prefs_server_deletemsgs_cb(GtkWidget *, gpointer);
void prefs_server_autoretrieve_cb(GtkWidget *, gpointer);
void prefs_server_noheaders_cb(GtkWidget *, gpointer);
void prefs_server_ignorefilters_cb(GtkWidget *, gpointer);
void prefs_server_command_cb(GtkWidget *, gpointer);
void prefs_server_disabled_cb(GtkWidget *, gpointer);

// prefs_viewers.c
Viewer *viewer_new(void);
void viewer_free(Viewer *vnode);
void viewers_set_defaults(void);
gboolean viewers_open_window(void);
void viewers_populate_list(void);
gboolean viewers_read(void);
gboolean viewers_save(void);
void viewers_free(void);
Viewer *viewer_get_node(gchar *);
void viewers_set_row_info(gint, gint, const gchar *);
void viewers_set_widget_info(Viewer *);

void viewers_edit_cb(GtkWidget *, gpointer);
void viewers_destroy_cb(GtkWidget *, gpointer);
void viewers_type_changed_cb(GtkWidget *widget, gpointer);
void viewers_action_changed_cb(GtkWidget *widget, gpointer);
void viewers_select_action_cb(GtkButton *, gpointer);
void viewers_preprocess_changed_cb(GtkWidget *widget, gpointer);
void viewers_select_preprocess_cb(GtkButton *, gpointer);
void viewers_disabled_toggled_cb(GtkWidget *widget, gpointer);
void viewers_new_cb(GtkButton *, gpointer);
void viewers_del_cb(GtkButton *, gpointer);
void viewers_select_row_cb(GtkCList *, gint, gint, GdkEventButton *, gpointer);

// prefs_filters.c
void filters_set_defaults(void);
gboolean filters_open_window(void);
void filters_close_window(void);
gboolean filters_read(void);
gboolean filters_write(void);
Filter *filter_new(void);
void filter_free(Filter *);
void filters_free(void);
void filters_populate_list(void);
void filters_set_widget_info(Filter *);
void filters_set_row_info(gint, gint, const gchar *);

void filters_edit_cb(GtkWidget *, gpointer);
void filters_destroy_cb(GtkWidget *, gpointer);
void filters_select_row_cb(GtkWidget *, guint, guint, GdkEvent *, gpointer);
void filters_new_filter_cb(GtkButton *, gpointer);
void filters_del_filter_cb(GtkButton *, gpointer);
void filters_sort_cb(GtkWidget *, gpointer);
gint filters_compare_filters(gconstpointer, gconstpointer);
void filters_tag_changed_cb(GtkWidget *, gpointer);
void filters_disabled_toggled_cb(GtkWidget *, gpointer);
void filters_pattern_changed_cb(GtkWidget *, gpointer);
void filters_priority_changed_cb(GtkWidget *, gpointer);
void filters_type_item_activate_cb(GtkWidget *, gpointer);
void filters_action_changed_cb(GtkWidget *, gpointer);
void filters_select_action_cb(GtkWidget *, gpointer);

// prefs_user.c
void prefs_user_set_defaults(void);
void edit_user_prefs_cb(GtkWidget *widget, gpointer data);
gboolean prefs_user_open_window(void);
void prefs_user_okay_cb(GnomeDialog *dialog, gint button, gpointer data);
void prefs_user_destroy_cb(GnomeDialog *dialog, gpointer data);

// prefs_init.c
glong prefs_new_config(void);
gboolean prefs_druid_cancel(GnomeDruid *, struct EucDruid *);
gboolean prefs_druid_finish_back(GnomeDruidPage *, gpointer, struct EucDruid *);
void prefs_druid_finish_prepare(GnomeDruidPage *, gpointer, struct EucDruid *);
void prefs_druid_finish(GnomeDruidPage *, gpointer, struct EucDruid *);

// request.c
gint euc_request(GtkWidget *, const gchar *, const gchar *, const gchar **, gint, gint16);
void euc_request_destroy(GtkButton *, gpointer);
FileReq *euc_file_request_new(glong, gchar *, gchar *);

void euc_file_request_okay_cb(GtkWidget *, gpointer);
void euc_file_request_cancel_cb(GtkWidget *, gpointer);
void euc_file_request_destroy_cb(GtkWidget *, gpointer);

#include "path.h"
#include "match.h"
#include "prune.h"
#include "_string.h"
#include "list.h"

#endif /* EUCALYPTUS_PROTOS_H */

