/*
 * regexp.c
 * by Brian W. Kernighan and Rob Pike
 * from Dr. Dobb's Journal, #298, April 1999
 * Copyright (c) 1999 Miller Freeman, Inc.
 *
 * Regular expressions
 *
 * GNU/glib-ized by Paul A. Schifferer
 */

#include "regexp.h"

/* match(): Search for 're' anywhere in 'text'.
 */
gint match(gchar *re, gchar *text)
{
	if(re[0] == '^')
		return matchhere(re + 1, text);
	
	do
	{
		/* must look at empty string */
		if(matchhere(re, text))
			return 1;
	} while(*text++ != '\0');
	
	return 0;
}

/* matchhere(): Search for 're' at beginning of 'text'.
 */
gint matchhere(gchar *re, gchar *text)
{
	if(re[0] == '\0')
		return 1;
	
	if(re[1] == '*')
		return matchstar(re[0], re + 2, text);
		
	if(re[0] == '$' && re[1] == '\0')
		return *text == '\0';
		
	if(*text != '\0' && (re[0] == '.' || re[0] == *text))
		return matchhere(re + 1, text + 1);
		
	return 0;
}

/* matchstar(): Search for c*re at beginning of text.
 */
gint matchstar(gint c, gchar *re, gchar *text)
{
	do
	{
		/* A '*' matches zero or more instances.
		 */
		if(matchhere(re, text))
			return 1;
	} while(*text != '\0' && (*text++ == c || c == '.'));
	
	return 0;
}