/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: glue/semaphore.c
 *   	 Semaphore wrapper functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <linux/ipc.h>
#include <linux/sem.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <glib.h>

#include "semaphore.h"

gboolean obtain_semaphore(gint sem_id, glong sem)
{
	struct sembuf sem_op[] = { {0, }, -1, 0, };
	gboolean success = FALSE;
	
	sem_op[0].sem_num = sem;
	if(-1 != semop(sem_id, sem_op, 1))
		success = TRUE;
		
	return success;
}

gboolean release_semaphore(gint sem_id, glong sem)
{
	struct sembuf sem_op[] = { {0, }, +1, 0, };
	gboolean success = FALSE;
	
	sem_op[0].sem_num = sem;
	if(-1 != semop(sem_id, sem_op, 1))
		success = TRUE;
		
	return success;
}

gint init_semaphore(glong num)
{
	gint sem_id;
	
	sem_id = semget(IPC_PRIVATE, num, 0600);
	if(sem_id < 0)
	{
		fprintf(stderr, "%s: semget()\n",
				strerror(errno));
		return -1;
	}
	else
	{
		union semun arg;
		ushort *initv;
		
		if(initv = (ushort *)g_malloc0(num * sizeof(ushort)))
		{
			glong i;
			
			for(i=0; i<num; i++)
				initv[i] = 1;
	
			arg.array = initv;
			if(-1 == semctl(sem_id, num, SETALL, arg))
			{
				fprintf(stderr, "semctl(SETALL) failed!\n");
				
				free_semaphore(sem_id);
				sem_id = -1;
			}
			
			g_free(initv);
		}
	}
	
	return sem_id;
}

void free_semaphore(glong sem_id)
{
	union semun arg;
	
	semctl(sem_id, 0, IPC_RMID, arg);
}

